@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -44,7 +44,6 @@ import javax.swing.plaf.basic.ComboPopup;
 import org.jvnet.lafwidget.animation.FadeStateListener;
 import org.jvnet.lafwidget.layout.TransitionLayout;
 import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
 import org.jvnet.substance.api.SubstanceConstants.Side;
 import org.jvnet.substance.api.renderers.SubstanceDefaultComboBoxRenderer;
 import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
@@ -220,12 +219,21 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 
 			if (SubstanceComboBoxUI.this.arrowButton != null) {
 				if (cb.getComponentOrientation().isLeftToRight()) {
-					SubstanceComboBoxUI.this.arrowButton.setBounds(width
-							- buttonWidth - insets.right, 0, buttonWidth
-							+ insets.right, height);
+					SubstanceComboBoxUI.this.arrowButton
+							.setBounds(
+									width - buttonWidth - insets.right,
+									0,
+									buttonWidth
+											+ SubstanceSizeUtils
+													.getComboBorderInsets(SubstanceSizeUtils
+															.getComponentFontSize(comboBox)).right,
+									height);
 				} else {
-					SubstanceComboBoxUI.this.arrowButton.setBounds(0, 0,
-							buttonWidth + insets.left, height);
+					int left = SubstanceSizeUtils
+							.getComboBorderInsets(SubstanceSizeUtils
+									.getComponentFontSize(comboBox)).left;
+					SubstanceComboBoxUI.this.arrowButton.setBounds(insets.left
+							- left, 0, buttonWidth + left, height);
 				}
 			}
 			if (SubstanceComboBoxUI.this.editor != null) {
@@ -436,19 +444,30 @@ public class SubstanceComboBoxUI extends BasicComboBoxUI {
 		}
 
 		Graphics2D graphics = (Graphics2D) g.create();
-		SubstanceColorScheme colorScheme = SubstanceColorSchemeUtilities
-				.getColorScheme(this.comboBox, state);
 		float alpha = SubstanceColorSchemeUtilities.getAlpha(this.comboBox,
 				state) * 0.8f;
 
-		Rectangle newBounds = new Rectangle(1, 1, this.comboBox.getWidth() - 2,
-				this.comboBox.getHeight() - 2);
+		int componentFontSize = SubstanceSizeUtils
+				.getComponentFontSize(this.comboBox);
+		int borderDelta = (int) Math.floor(SubstanceSizeUtils
+				.getBorderStrokeWidth(componentFontSize) / 2.0);
+		Shape contour = SubstanceOutlineUtilities.getBaseOutline(this.comboBox
+				.getWidth(), this.comboBox.getHeight(),
+				0.5f * SubstanceSizeUtils
+						.getClassicButtonCornerRadius(componentFontSize), null,
+				borderDelta);
+		graphics.clip(contour);
+
+		// Rectangle newBounds = new Rectangle(1, 1, this.comboBox.getWidth() -
+		// 2,
+		// this.comboBox.getHeight() - 2);
 
 		graphics.setComposite(TransitionLayout.getAlphaComposite(this.comboBox,
 				alpha, g));
 
-		SubstanceHighlightUtils.paintHighlight(graphics, this.comboBox,
-				newBounds, 0.0f, null, colorScheme, colorScheme, 0.0f);
+		SubstanceHighlightUtils.paintHighlight(graphics, null, this.comboBox,
+				new Rectangle(0, 0, this.comboBox.getWidth(), this.comboBox
+						.getHeight()), 0.0f, null, state, state, 0.0f);
 		graphics.dispose();
 	}
 
