@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -44,9 +44,10 @@ import javax.swing.plaf.basic.BasicListUI;
 import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
-import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.api.renderers.SubstanceDefaultListCellRenderer;
+import org.jvnet.substance.painter.decoration.DecorationAreaType;
+import org.jvnet.substance.painter.decoration.SubstanceDecorationUtilities;
 import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
 import org.jvnet.substance.painter.utils.SubstanceFillBackgroundDelegate;
 import org.jvnet.substance.utils.*;
@@ -99,20 +100,15 @@ public class SubstanceListUI extends BasicListUI {
 	protected RolloverFadeListener substanceFadeRolloverListener;
 
 	/**
-	 * Map of previous fade states (for state-aware theme transitions).
+	 * Map of previous fade states (for state-aware color scheme transitions).
 	 */
 	private Map<Integer, ComponentState> prevStateMap;
 
 	/**
-	 * Map of next fade states (for state-aware theme transitions).
+	 * Map of next fade states (for state-aware color scheme transitions).
 	 */
 	private Map<Integer, ComponentState> nextStateMap;
 
-	/**
-	 * The current default color scheme.
-	 */
-	private SubstanceColorScheme currDefaultColorScheme;
-
 	/**
 	 * Listener for fade animations on list rollovers.
 	 * 
@@ -319,16 +315,20 @@ public class SubstanceListUI extends BasicListUI {
 		public CellRepaintCallback(JList list, int cellIndex) {
 			this.list = list;
 			this.cellIndex = cellIndex;
-			this.rendererComponent = list
-					.getCellRenderer()
-					.getListCellRendererComponent(
-							list,
-							list.getModel().getElementAt(cellIndex),
-							cellIndex,
-							list.getSelectionModel().isSelectedIndex(cellIndex),
-							list.hasFocus()
-									&& (cellIndex == list
-											.getLeadSelectionIndex()));
+			// verify if this index still exists in the model (the list may have
+			// shrunk!) - issue 413
+			if (cellIndex < list.getModel().getSize()) {
+				this.rendererComponent = list.getCellRenderer()
+						.getListCellRendererComponent(
+								list,
+								list.getModel().getElementAt(cellIndex),
+								cellIndex,
+								list.getSelectionModel().isSelectedIndex(
+										cellIndex),
+								list.hasFocus()
+										&& (cellIndex == list
+												.getLeadSelectionIndex()));
+			}
 		}
 
 		/*
@@ -575,7 +575,7 @@ public class SubstanceListUI extends BasicListUI {
 			return;
 		}
 
-		boolean isWatermarkBleed = SubstanceCoreUtilities.toDrawWatermark(list);
+		boolean isWatermarkBleed = updateInfo.toDrawWatermark;
 
 		int cx = rowBounds.x;
 		int cy = rowBounds.y;
@@ -601,7 +601,7 @@ public class SubstanceListUI extends BasicListUI {
 			// optimization - only render background if it's different
 			// from the list background
 			if ((background != null)
-					&& !list.getBackground().equals(background)) {
+					&& (!list.getBackground().equals(background) || this.updateInfo.isInDecorationArea)) {
 				g2d.setColor(background);
 				g2d.fillRect(cx, cy, cw, ch);
 			}
@@ -615,10 +615,8 @@ public class SubstanceListUI extends BasicListUI {
 		ComponentState currState = getCellState(row, rendererComponent);
 
 		// Compute the alpha values for the animation.
-		float startAlpha = SubstanceColorSchemeUtilities.getHighlightAlpha(
-				list, prevState);
-		float endAlpha = SubstanceColorSchemeUtilities.getHighlightAlpha(list,
-				currState);
+		float startAlpha = this.updateInfo.getHighlightAlpha(prevState);
+		float endAlpha = this.updateInfo.getHighlightAlpha(currState);
 
 		// if the renderer is disabled, do not show any highlights
 		float totalAlpha = endAlpha;
@@ -643,17 +641,22 @@ public class SubstanceListUI extends BasicListUI {
 			}
 		}
 
-		SubstanceColorScheme prevScheme = SubstanceColorSchemeUtilities
-				.getHighlightColorScheme(list, prevState);
-		SubstanceColorScheme currScheme = SubstanceColorSchemeUtilities
-				.getHighlightColorScheme(list, currState);
+		SubstanceColorScheme prevScheme = this.updateInfo
+				.getHighlightColorScheme(prevState);
+		SubstanceColorScheme currScheme = this.updateInfo
+				.getHighlightColorScheme(currState);
+		SubstanceColorScheme prevBorderScheme = this.updateInfo
+				.getHighlightBorderColorScheme(prevState);
+		SubstanceColorScheme currBorderScheme = this.updateInfo
+				.getHighlightBorderColorScheme(currState);
 
 		Rectangle cellRect = new Rectangle(cx, cy, cw, ch);
 		if (totalAlpha > 0.0f) {
 			g2d.setComposite(TransitionLayout.getAlphaComposite(list,
 					totalAlpha, g));
-			SubstanceHighlightUtils.paintHighlight(g2d, rendererComponent,
-					cellRect, 0.8f, null, currScheme, prevScheme, fadeCoef);
+			SubstanceHighlightUtils.paintHighlight(g2d, this.rendererPane,
+					rendererComponent, cellRect, 0.8f, null, currScheme,
+					prevScheme, currBorderScheme, prevBorderScheme, fadeCoef);
 			g2d.setComposite(TransitionLayout.getAlphaComposite(list, g));
 		}
 
@@ -672,7 +675,8 @@ public class SubstanceListUI extends BasicListUI {
 	public ComponentState getPrevCellState(int cellIndex) {
 		if (prevStateMap.containsKey(cellIndex))
 			return prevStateMap.get(cellIndex);
-		return ComponentState.DEFAULT;
+		return getCellState(cellIndex, null);
+		// return ComponentState.DEFAULT;
 	}
 
 	/**
@@ -686,16 +690,14 @@ public class SubstanceListUI extends BasicListUI {
 	 */
 	public ComponentState getCellState(int cellIndex,
 			Component rendererComponent) {
-		ButtonModel synthModel = new DefaultButtonModel();
 		boolean isEnabled = this.list.isEnabled();
 		if (rendererComponent != null) {
 			isEnabled = isEnabled && rendererComponent.isEnabled();
 		}
-		synthModel.setEnabled(isEnabled);
-		synthModel.setRollover((rolledOverIndex >= 0)
-				&& (rolledOverIndex == cellIndex));
-		synthModel.setSelected(list.isSelectedIndex(cellIndex));
-		return ComponentState.getState(synthModel, null);
+		boolean isRollover = (rolledOverIndex >= 0)
+				&& (rolledOverIndex == cellIndex);
+		boolean isSelected = list.isSelectedIndex(cellIndex);
+		return ComponentState.getState(isEnabled, isRollover, isSelected);
 	}
 
 	/**
@@ -717,11 +719,11 @@ public class SubstanceListUI extends BasicListUI {
 
 		Graphics2D g2d = (Graphics2D) g.create();
 		SubstanceStripingUtils.setup(c);
-		this.currDefaultColorScheme = SubstanceColorSchemeUtilities
-				.getColorScheme(c, ComponentState.DEFAULT);
+		this.updateInfo = new UpdateOptimizationInfo();
 		this.paint(g2d, c);
 		SubstanceStripingUtils.tearDown(c);
 		g2d.dispose();
+		this.updateInfo = null;
 	}
 
 	/**
@@ -731,6 +733,83 @@ public class SubstanceListUI extends BasicListUI {
 	 * @return The current default color scheme.
 	 */
 	public SubstanceColorScheme getDefaultColorScheme() {
-		return this.currDefaultColorScheme;
+		if (this.updateInfo != null)
+			return this.updateInfo.defaultScheme;
+		return null;
+	}
+
+	public SubstanceColorScheme getHighlightColorScheme(ComponentState state) {
+		if (this.updateInfo != null)
+			return updateInfo.getHighlightColorScheme(state);
+		return null;
+	}
+
+	private UpdateOptimizationInfo updateInfo;
+
+	private class UpdateOptimizationInfo {
+		public boolean toDrawWatermark;
+
+		private Map<ComponentState, SubstanceColorScheme> highlightSchemeMap;
+
+		private Map<ComponentState, SubstanceColorScheme> borderSchemeMap;
+
+		private Map<ComponentState, Float> highlightAlphaMap;
+
+		public SubstanceColorScheme defaultScheme;
+
+		public Map<Component, Boolean> opacity;
+
+		public DecorationAreaType decorationAreaType;
+
+		public boolean isInDecorationArea;
+
+		public UpdateOptimizationInfo() {
+			this.toDrawWatermark = SubstanceCoreUtilities.toDrawWatermark(list);
+			this.defaultScheme = SubstanceColorSchemeUtilities.getColorScheme(
+					list, ComponentState.DEFAULT);
+			this.highlightAlphaMap = new EnumMap<ComponentState, Float>(
+					ComponentState.class);
+			this.highlightSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
+					ComponentState.class);
+			this.borderSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
+					ComponentState.class);
+			this.opacity = new HashMap<Component, Boolean>();
+			this.decorationAreaType = SubstanceDecorationUtilities
+					.getDecorationType(list);
+
+			SubstanceSkin skin = SubstanceCoreUtilities.getSkin(list);
+			this.isInDecorationArea = (this.decorationAreaType != null)
+					&& skin
+							.isRegisteredAsDecorationArea(this.decorationAreaType)
+					& TransitionLayout.isOpaque(list);
+		}
+
+		public SubstanceColorScheme getHighlightColorScheme(ComponentState state) {
+			if (!this.highlightSchemeMap.containsKey(state)) {
+				this.highlightSchemeMap.put(state,
+						SubstanceColorSchemeUtilities.getColorScheme(list,
+								ColorSchemeAssociationKind.HIGHLIGHT, state));
+			}
+			return this.highlightSchemeMap.get(state);
+		}
+
+		public SubstanceColorScheme getHighlightBorderColorScheme(
+				ComponentState state) {
+			if (!this.borderSchemeMap.containsKey(state)) {
+				this.borderSchemeMap.put(state, SubstanceColorSchemeUtilities
+						.getColorScheme(list,
+								ColorSchemeAssociationKind.HIGHLIGHT_BORDER,
+								state));
+			}
+			return this.borderSchemeMap.get(state);
+		}
+
+		public float getHighlightAlpha(ComponentState state) {
+			if (!this.highlightAlphaMap.containsKey(state)) {
+				this.highlightAlphaMap.put(state, SubstanceColorSchemeUtilities
+						.getHighlightAlpha(list, state));
+			}
+			return this.highlightAlphaMap.get(state);
+		}
 	}
 }
