@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -30,11 +30,13 @@
 package org.jvnet.substance;
 
 import java.awt.*;
+import java.awt.image.BufferedImage;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.util.*;
 
 import javax.swing.*;
+import javax.swing.plaf.IconUIResource;
 import javax.swing.plaf.UIResource;
 import javax.swing.plaf.basic.BasicLookAndFeel;
 
@@ -114,9 +116,9 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * Indicates whether option dialogs (error, question, warning, info) should
-	 * use constant themes for icon coloring. Note that since version 4.0, the
-	 * default setting is <code>true</code> (use constant theme). To use
-	 * theme-consistent coloring, call
+	 * use constant color schemes for icon coloring. Note that since version
+	 * 4.0, the default setting is <code>true</code> (use constant color
+	 * scheme). To use color scheme-consistent coloring, call
 	 * {@link #setToUseConstantThemesOnDialogs(boolean)} and pass
 	 * <code>false</code>.
 	 * 
@@ -160,12 +162,12 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * {@link Boolean#FALSE}.
 	 * 
 	 * <p>
-	 * In order to compute whether the current watermark should be painted on
-	 * some component, the component's hierarchy is traversed bottom up. The
-	 * first component that has this property set, defines the watermark
-	 * visibility. Finally, if neither component not its ancestors define this
-	 * property, the global setting on {@link UIManager} is checked. If there is
-	 * no global setting, the watermark is <b>not</b> ignored (it is painted).
+	 * In order to compute whether the current watermark should be painted on a
+	 * given component, its hierarchy is traversed bottom up. The first
+	 * component that has this property set defines the watermark visibility. If
+	 * neither component nor its ancestors define this property, the global
+	 * setting on {@link UIManager} is checked. If there is no global setting,
+	 * the watermark is <b>not</b> ignored (it is painted).
 	 * </p>
 	 * 
 	 * <p>
@@ -181,10 +183,10 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	public static final String WATERMARK_VISIBLE = "substancelaf.watermark.visible";
 
 	/**
-	 * Client property name for ignoring default (minimum) dimension for a
-	 * single button. This property can be set either as a client property on a
-	 * specific button or as a global setting on {@link UIManager}. The value
-	 * should be either {@link Boolean#TRUE} or {@link Boolean#FALSE}.
+	 * Client property name for ignoring the default (minimum) dimension for a
+	 * single button. This property can be set either on the specific button or
+	 * as a global setting on {@link UIManager}. The value should be either
+	 * {@link Boolean#TRUE} or {@link Boolean#FALSE}.
 	 * <p>
 	 * Note that {@link SubstanceButtonShaper} implementations are not required
 	 * to respect this property. The current implementations of the default
@@ -216,12 +218,12 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * Client property name for specifying that a single button / all
-	 * application buttons should never paint the background. This property can
-	 * be set either as a client property on a specific button or as a global
-	 * setting on {@link UIManager}. The value should be either
-	 * {@link Boolean#TRUE} or {@link Boolean#FALSE}. Note that unlike the
-	 * {@link #FLAT_PROPERTY}, a button marked with this property will
-	 * <b>never</b> show the background (will always be painted flat).
+	 * application buttons should not paint the background. This property can be
+	 * set either on the specific button or as a global setting on
+	 * {@link UIManager}. The value should be either {@link Boolean#TRUE} or
+	 * {@link Boolean#FALSE}. Note that unlike the {@link #FLAT_PROPERTY}, a
+	 * button marked with this property will <b>never</b> show the background
+	 * (will always be painted flat).
 	 * 
 	 * <p>
 	 * Example of marking a button to never paint background:
@@ -248,8 +250,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * Client property name for specifying a straight side for a single button.
-	 * This property must be set as a client property on a specific button. The
-	 * value can be:
+	 * This property must be set on the specific button. The value can be:
 	 * 
 	 * <p>
 	 * <ul>
@@ -258,7 +259,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * </ul>
 	 * 
 	 * <p>
-	 * Note that not all {@link SubstanceButtonShaper} implementations are
+	 * Note that not the {@link SubstanceButtonShaper} implementations are
 	 * required to respect this property. The default
 	 * {@link StandardButtonShaper} and {@link ClassicButtonShaper} respect this
 	 * property.
@@ -281,8 +282,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * Client property name for specifying an open side for a single button.
-	 * This property must be set as a client property on a specific button. The
-	 * value can be:
+	 * This property must be set on the specific button. The value can be:
 	 * 
 	 * <p>
 	 * <ul>
@@ -309,10 +309,9 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	/**
 	 * Client property name for specifying the corner radius for buttons.
 	 * Currently, this property is respected only on toolbar buttons. This
-	 * property can be set either as a client property on a specific toolbar
-	 * button, a client property on a specific toolbar (will hold for all
-	 * buttons in the toolbar) or as a global setting on {@link UIManager}. The
-	 * value should be a {@link Float} instance.
+	 * property can be set on the specific toolbar button, on the specific
+	 * toolbar (will hold for all buttons in the toolbar) or as a global setting
+	 * on {@link UIManager}. The value should be a positive {@link Float}.
 	 * 
 	 * <p>
 	 * Example of specifying a (toolbar) button to have corner radius of 5
@@ -351,9 +350,12 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * Property name for specifying that the component should be painted flat
 	 * (no background / border) when it's inactive. This property should be
 	 * specified on a specific component or its parent and must have either
-	 * {@link Boolean#TRUE} or {@link Boolean#FALSE} value. For example, this is
-	 * how to mark a button to appear flat:
+	 * {@link Boolean#TRUE} or {@link Boolean#FALSE} value.
+	 * 
 	 * <p>
+	 * Example how to mark a button to appear flat:
+	 * </p>
+	 * 
 	 * <code>
 	 * JButton button = new JButton("text");<br>
 	 * button.putClientProperty(SubstanceLookAndFeel.FLAT_PROPERTY, <br>
@@ -384,7 +386,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	public static final String HEAP_STATUS_TRACE_FILE = "substancelaf.heapStatusTraceFile";
 
 	/**
-	 * Client property name for specifying that contents of some frame, dialog,
+	 * Client property name for specifying that contents of a frame, dialog,
 	 * internal frame, desktop icon or tab have been modified and not saved. The
 	 * property can be set on:
 	 * <p>
@@ -392,8 +394,8 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * <li>{@link JInternalFrame} - the <b>close</b> button of the specific
 	 * internal frame will be animated (in case that the internal frame has
 	 * decorated title pane). In addition, once such internal frame is iconified
-	 * (to a {@link JInternalFrame.JDesktopIcon}), the close button of the
-	 * matching desktop icon is animated as well.</li>
+	 * (to a {@link JInternalFrame.JDesktopIcon}), the close button of the its
+	 * desktop icon is animated as well.</li>
 	 * <li>{@link JRootPane} - the <b>close</b> button of the title pane of the
 	 * matching frame / dialog will be animated (in case that the frame / dialog
 	 * have decorated title pane).</li>
@@ -405,8 +407,8 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * </ul>
 	 * </p>
 	 * <p>
-	 * The animation cycles between red, orange and yellow themes. In most cases
-	 * (all but tabs not marked with
+	 * The animation cycles between red, orange and yellow color schemes. In
+	 * most cases (all but tabs not marked with
 	 * {@link #TABBED_PANE_CLOSE_BUTTONS_MODIFIED_ANIMATION} property), the
 	 * animation will be visible only when the mouse hovers over the close
 	 * button of the matching container (frame, dialog, internal frame, desktop
@@ -493,13 +495,11 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	public final static String WINDOW_MODIFIED = "windowModified";
 
 	/**
-	 * Client property name for specifying that a single tab / all tabs of a
-	 * single tabbed pane / all tabs of all tabbed panes should have close
-	 * buttons. This property can be specified on a single tab component, on a
-	 * {@link JTabbedPane} itself (will hold for all tab components that don't
-	 * define this property) or on {@link UIManager}. The value should be either
-	 * {@link Boolean#TRUE} or {@link Boolean#FALSE}. By default, the close
-	 * buttons are not displayed.
+	 * Client property name for adding close buttons on tabs. This property can
+	 * be specified on a single tab component, on a {@link JTabbedPane} itself
+	 * (will hold for all tab components that don't define this property) or on
+	 * {@link UIManager}. The value should be either {@link Boolean#TRUE} or
+	 * {@link Boolean#FALSE}. By default, the close buttons are not displayed.
 	 * 
 	 * <p>
 	 * Example of setting that all tabs in the application will have close
@@ -801,17 +801,17 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * Client property name for specifying the kind of focus indication on
 	 * buttons, check boxes and radio buttons. The value should be one of
 	 * {@link SubstanceConstants.FocusKind} enum. This property can be set
-	 * either as client property on some component or as global property on
+	 * either on the specific component or as global property on
 	 * {@link UIManager}.
 	 * 
 	 * <p>
 	 * In order to compute the kind of focus indication for some component, the
 	 * component's hierarchy is traversed bottom up. The first component that
-	 * has this property set, defines the focus indication kind. Finally, if
-	 * neither component not its ancestors define this property, the global
-	 * setting on {@link UIManager} is checked. If there is no global setting,
-	 * the default {@link SubstanceConstants.FocusKind#ALL_INNER} is used. Here
-	 * is an example to illustrate the above:
+	 * has this property set, defines the focus indication kind. If neither
+	 * component nor its ancestors define this property, the global setting on
+	 * {@link UIManager} is checked. If there is no global setting, the default
+	 * {@link SubstanceConstants.FocusKind#ALL_INNER} is used. Here is an
+	 * example to illustrate the above:
 	 * </p>
 	 * 
 	 * <code>
@@ -951,24 +951,15 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * <p>
-	 * Client property name for specifying that icons on buttons and toggle
-	 * buttons should match the color of the current theme when they are in
-	 * default state. The button is in default state when it's not pressed, not
-	 * selected, not armed and not rolled over. The value should be an instance
-	 * of {@link Boolean}. By default, all buttons show regular (full-color
-	 * original) icons. The value can be set on the specific button or globally
-	 * on {@link UIManager}.
+	 * Client property name for specifying that icons on controls sch as
+	 * buttons, toggle buttons, labels, tabs and menu items should match the
+	 * color of the current color scheme when they are in default state. The
+	 * control is in default state when it's not pressed, not selected, not
+	 * armed and not rolled over. The value should be an instance of
+	 * {@link Boolean}. By default, all controls show regular (full-color
+	 * original) icons. The value can be set globally on {@link UIManager}.
 	 * </p>
 	 * 
-	 * <p>
-	 * Example of marking a button to have theme-matching icon in default state:
-	 * </p>
-	 * <code>
-	 * JButton jb = new JButton("sample", myIcon);<br>
-	 * jb.putClientProperty(SubstanceLookAndFeel.USE_THEMED_DEFAULT_ICONS, <br>
-	 * &nbsp;&nbsp;Boolean.TRUE);
-	 * </code>
-	 * 
 	 * @since version 3.3
 	 */
 	@SubstanceApi
@@ -976,37 +967,35 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * <p>
-	 * Client property name for specifying that the colorization amount applied
-	 * to the background and foreground of the current theme and the specific
-	 * control. By default, when the application does not use any custom colors,
-	 * all the controls are painted with the colors of the current theme / skin.
-	 * The colors coming from the look-and-feel implement the marker
-	 * {@link UIResource} interface which allows the UI delegates to
-	 * differentiate between application-specific colors which are not changed,
-	 * and the LAF-provide colors that are changed on LAF switch.
+	 * Client property name for specifying the colorization amount applied to
+	 * the background and foreground of the current color scheme and the
+	 * specific control. By default, when the application does not use any
+	 * custom colors, all the controls are painted with the colors of the
+	 * current color scheme / skin. The colors coming from the look-and-feel
+	 * implement the marker {@link UIResource} interface which allows the UI
+	 * delegates to differentiate between application-specific colors which are
+	 * not changed, and the LAF-provide colors that are changed on LAF switch.
 	 * </p>
 	 * 
 	 * <p>
-	 * Up until version 4.2, the support for custom background and foreground
-	 * colors in Substance was not very consistent. On some controls, such as
-	 * buttons, the background color was ignored completely (in favour of the
-	 * current theme), while on others it was used to color the entire bounds of
-	 * the control (such as check box). This new client property installs the
-	 * "smart colorization" mode which uses the colors of the current theme
-	 * <b>and</b> the custom background and foreground colors (where installed
-	 * by application) to colorize the relevant portions of the control. For
-	 * example, on checkbox the custom background color will be used to colorize
-	 * the check box itself, while the custom foreground color will be applied
-	 * to the check box text and the check mark.
+	 * This new client property installs the "smart colorization" mode which
+	 * uses the colors of the current color scheme and the custom background /
+	 * foreground colors (when installed by application) to colorize the
+	 * relevant portions of the control. For example, on checkbox the custom
+	 * background color will be used to colorize the check box itself, while the
+	 * custom foreground color will be applied to the check box text and the
+	 * check mark.
 	 * </p>
 	 * 
 	 * <p>
 	 * The value of this property specifies the actual colorization amount.
-	 * Value of 0.0 results in no colorization at all. Values closer to 1.0
-	 * result in controls being colorized in almost full amount of the custom
-	 * colors. Note that in order to maintain the gradients (fill, border, etc),
-	 * even value of 1.0 does not result in full custom color being applied to
-	 * the relevant visuals of the control.
+	 * Value of 0.0 results in Substance completely <strong>ignoring</strong>
+	 * the custom application background and foreground colors set on the
+	 * components - no colorization. Values closer to 1.0 result in almost full
+	 * usage of the custom application background and foreground colors set on
+	 * the components. Note that in order to maintain the gradients (fill,
+	 * border, etc), even value of 1.0 does not result in full custom color
+	 * being applied to the relevant visuals of the control.
 	 * </p>
 	 * 
 	 * <p>
@@ -1017,10 +1006,10 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * </p>
 	 * 
 	 * <p>
-	 * Starting from version 5.0, the default value for this property (when it
-	 * is not set at all) is 0.5. This means that applications that install
-	 * custom background / foreground colors on their UI controls will see them
-	 * colorized with 50% "strength", even without setting this property.
+	 * The default colorization amount (when this property is not set at all) is
+	 * 0.5. This means that applications that install custom background /
+	 * foreground colors on their UI controls will see them colorized with 50%
+	 * "strength", even without setting this property.
 	 * </p>
 	 * 
 	 * <p>
@@ -1028,7 +1017,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * </p>
 	 * 
 	 * <p>
-	 * Example of marking a button to have custom background color and
+	 * Example of marking a button to have a custom background color and
 	 * colorizing it with 40%:
 	 * </p>
 	 * <code>
@@ -1038,6 +1027,13 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * &nbsp;&nbsp;new Double(0.4));
 	 * </code>
 	 * 
+	 * <p>
+	 * Note that components in decoration areas registered on the current skin
+	 * will ignore the colorization on custom background color. The background
+	 * of such components is always painted by the skin's decoration painter to
+	 * ensure consistent background painting of the relevant decoration area.
+	 * </p>
+	 * 
 	 * @since version 4.2
 	 * @see Component#setBackground(Color)
 	 * @see Component#setForeground(Color)
@@ -1079,7 +1075,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	public static final String BUTTON_SHAPER_PROPERTY = "substancelaf.buttonShaper";
 
 	/**
-	 * Property name for specifying skin to be used on the specific root pane.
+	 * Property name for specifying a skin to be used on the specific root pane.
 	 * This property can only be installed on a {@link JRootPane} and will
 	 * affect all the controls in that root pane. The value must be an instance
 	 * of {@link SubstanceSkin}. After setting this property, call
@@ -1406,11 +1402,9 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * Sets the {@link FontPolicy} to be used with Substance family. If the
-	 * specified policy is <code>null</code>, the default will be reset. Note
-	 * that after calling this method you need to call
-	 * {@link UIManager#setLookAndFeel(LookAndFeel)} with Substance and
-	 * {@link SwingUtilities#updateComponentTreeUI(Component)} to have the new
-	 * font policy applied to your windows.
+	 * specified policy is <code>null</code>, the default will be reset. This
+	 * method does not require Substance to be the current look-and-feel, and
+	 * will cause Substance to be set as the current application look-and-feel.
 	 * 
 	 * @param fontPolicy
 	 *            The {@link FontPolicy} to be used with Substance family, or
@@ -1691,7 +1685,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 		LafWidgetRepository.getRepository().unsetLafSupport();
 
 		// clear caches
-		SubstanceCoreUtilities.resetCaches();
+		LazyResettableHashMap.reset();
 
 		this.currentKeyboardFocusManager
 				.removePropertyChangeListener(this.focusOwnerChangeListener);
@@ -1896,11 +1890,11 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 
 	/**
 	 * Checks whether the <code>JOptionPane</code>s created with predefined
-	 * message types should use constant themes for the icons.
+	 * message types should use constant color schemes for the icons.
 	 * 
 	 * @return <code>true</code> if the <code>JOptionPane</code>s created with
-	 *         predefined message types should use constant themes for the
-	 *         icons, <code>false</code> otherwise.
+	 *         predefined message types should use constant color schemes for
+	 *         the icons, <code>false</code> otherwise.
 	 * @see #setToUseConstantThemesOnDialogs(boolean)
 	 */
 	@SubstanceApi
@@ -1914,8 +1908,8 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 * 
 	 * @param toUseConstantThemesOnDialogs
 	 *            if <code>true</code>, the <code>JOptionPane</code>s created
-	 *            with predefined message types should use constant themes for
-	 *            the icons.
+	 *            with predefined message types should use constant color
+	 *            schemes for the icons.
 	 * @see #isToUseConstantThemesOnDialogs()
 	 */
 	@SubstanceApi
@@ -2005,7 +1999,7 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 			}
 
 			if (isSubstance)
-				SubstanceCoreUtilities.resetCaches();
+				LazyResettableHashMap.reset();
 
 			currentSkin = newSkin;
 
@@ -2230,6 +2224,35 @@ public abstract class SubstanceLookAndFeel extends BasicLookAndFeel {
 	 *         <code>false</code> otherwise.
 	 */
 	public static boolean isCurrentLookAndFeel() {
-		return (currentSkin != null);
+		return ((UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) && (currentSkin != null));
+	}
+
+	/*
+	 * (non-Javadoc)
+	 * 
+	 * @see javax.swing.LookAndFeel#getDisabledIcon(javax.swing.JComponent,
+	 * javax.swing.Icon)
+	 */
+	@Override
+	public Icon getDisabledIcon(JComponent component, Icon icon) {
+		if (icon == null)
+			return null;
+		SubstanceColorScheme colorScheme = SubstanceColorSchemeUtilities
+				.getColorScheme(component, ComponentState.DISABLED_UNSELECTED);
+		BufferedImage result = SubstanceImageCreator.getColorSchemeImage(
+				component, icon, colorScheme, 0.5f);
+		float alpha = SubstanceColorSchemeUtilities.getAlpha(component,
+				ComponentState.DISABLED_UNSELECTED);
+		if (alpha < 1.0f) {
+			BufferedImage intermediate = SubstanceCoreUtilities.getBlankImage(
+					result.getWidth(), result.getHeight());
+			Graphics2D g2d = intermediate.createGraphics();
+			g2d.setComposite(AlphaComposite.SrcOver.derive(alpha));
+			g2d.drawImage(result, 0, 0, null);
+			g2d.dispose();
+			result = intermediate;
+		}
+
+		return new IconUIResource(new ImageIcon(result));
 	}
 }
