@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -35,7 +35,6 @@ import java.beans.PropertyChangeEvent;
 import java.security.*;
 
 import javax.swing.*;
-import javax.swing.border.Border;
 import javax.swing.event.MouseInputAdapter;
 import javax.swing.event.MouseInputListener;
 import javax.swing.plaf.ComponentUI;
@@ -96,7 +95,7 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 	/**
 	 * <code>JRootPane</code> providing the look and feel for.
 	 */
-	private JRootPane root;
+	protected JRootPane root;
 
 	/**
 	 * Window listener that stops all Substance thread when the last frame is
@@ -411,7 +410,9 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 						protected void processNewPosition() {
 							SwingUtilities.invokeLater(new Runnable() {
 								public void run() {
-									// currentRootPaneGC = null;
+									if (window == null)
+										return;
+									
 									if (!window.isShowing()
 											|| !window.isDisplayable()) {
 										currentRootPaneGC = null;
@@ -595,7 +596,7 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 	 * 
 	 * @return Layout manager.
 	 */
-	private LayoutManager createLayoutManager() {
+	protected LayoutManager createLayoutManager() {
 		return new SubstanceRootLayout();
 	}
 
@@ -616,7 +617,7 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 		JComponent oldTitlePane = this.getTitlePane();
 
 		if (oldTitlePane != null) {
-			// fix for defect 109 - memory leak on theme change
+			// fix for defect 109 - memory leak on skin change
 			if (oldTitlePane instanceof SubstanceTitlePane)
 				((SubstanceTitlePane) oldTitlePane).uninstall();
 			// oldTitlePane.setVisible(false);
@@ -643,18 +644,12 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 		screenBounds.x = 0;
 		screenBounds.y = 0;
 		Insets screenInsets = Toolkit.getDefaultToolkit().getScreenInsets(gc);
-		Border rootBorder = this.root.getBorder();
-		Insets insets = (rootBorder == null) ? new Insets(0, 0, 0, 0)
-				: rootBorder.getBorderInsets(this.root);
 		Rectangle maxBounds = new Rectangle(
-				(screenBounds.x + screenInsets.left) - insets.left,
-				(screenBounds.y + screenInsets.top) - insets.right,
-				screenBounds.width
-						- ((screenInsets.left + screenInsets.right)
-								- insets.left - insets.right),
+				(screenBounds.x + screenInsets.left),
+				(screenBounds.y + screenInsets.top), screenBounds.width
+						- ((screenInsets.left + screenInsets.right)),
 				screenBounds.height
-						- ((screenInsets.top + screenInsets.bottom)
-								- insets.top - insets.bottom));
+						- ((screenInsets.top + screenInsets.bottom)));
 		if (tla instanceof JFrame)
 			((JFrame) tla).setMaximizedBounds(maxBounds);
 		if (MemoryAnalyzer.isRunning()) {
@@ -679,7 +674,7 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 	 * 
 	 * @return The associated root pane.
 	 */
-	private JRootPane getRootPane() {
+	protected JRootPane getRootPane() {
 		return this.root;
 	}
 
@@ -724,9 +719,7 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 	 * A custom layout manager that is responsible for the layout of
 	 * layeredPane, glassPane, menuBar and titlePane, if one has been installed.
 	 */
-	// NOTE: Ideally this would extends JRootPane.RootLayout, but that
-	// would force this to be non-static.
-	private static class SubstanceRootLayout implements LayoutManager2 {
+	protected class SubstanceRootLayout implements LayoutManager2 {
 		/**
 		 * Returns the amount of space the layout would like to have.
 		 * 
@@ -1262,12 +1255,18 @@ public class SubstanceRootPaneUI extends BasicRootPaneUI {
 				return;
 			}
 
+			JComponent windowTitlePane = SubstanceRootPaneUI.this.getTitlePane();
+			// fix for issue 444 - ignore double clicks when the title pane
+			// is not showing (for example under JRootPane.NONE decoration style).
+			if (windowTitlePane == null)
+				return;
+			
 			Point convertedPoint = SwingUtilities.convertPoint(w,
-					ev.getPoint(), SubstanceRootPaneUI.this.getTitlePane());
+					ev.getPoint(), windowTitlePane);
 
 			int state = f.getExtendedState();
-			if ((SubstanceRootPaneUI.this.getTitlePane() != null)
-					&& SubstanceRootPaneUI.this.getTitlePane().contains(
+			if ((windowTitlePane != null)
+					&& windowTitlePane.contains(
 							convertedPoint)) {
 				if (((ev.getClickCount() % 2) == 0)
 						&& ((ev.getModifiers() & InputEvent.BUTTON1_MASK) != 0)) {
