@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -30,7 +30,6 @@
 package org.jvnet.substance;
 
 import java.awt.*;
-import java.awt.event.*;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.util.HashSet;
@@ -362,60 +361,6 @@ public class SubstanceScrollPaneUI extends BasicScrollPaneUI {
 		super.uninstallListeners(c);
 	}
 
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see javax.swing.plaf.basic.BasicScrollPaneUI#createMouseWheelListener()
-	 */
-	@Override
-	protected MouseWheelListener createMouseWheelListener() {
-		return new MouseWheelHandler();
-	}
-
-	/**
-	 * Custom mouse wheel handler. Provides faster scrolling - issue 87.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	protected class MouseWheelHandler implements MouseWheelListener {
-		/*
-		 * (non-Javadoc)
-		 * 
-		 * @see
-		 * java.awt.event.MouseWheelListener#mouseWheelMoved(java.awt.event.
-		 * MouseWheelEvent)
-		 */
-		public void mouseWheelMoved(java.awt.event.MouseWheelEvent e) {
-			if (SubstanceScrollPaneUI.this.scrollpane.isWheelScrollingEnabled()
-					&& (e.getScrollAmount() != 0)) {
-				JScrollBar toScroll = SubstanceScrollPaneUI.this.scrollpane
-						.getVerticalScrollBar();
-				int direction = 0;
-
-				// find which scrollbar to scroll, or return if none
-				if ((toScroll == null) || !toScroll.isVisible()
-						|| (e.getModifiers() == InputEvent.ALT_MASK)) {
-					toScroll = SubstanceScrollPaneUI.this.scrollpane
-							.getHorizontalScrollBar();
-
-					if ((toScroll == null) || !toScroll.isVisible()) {
-						return;
-					}
-				}
-
-				direction = (e.getWheelRotation() < 0) ? (-1) : 1;
-
-				SubstanceScrollBarUI ui = (SubstanceScrollBarUI) toScroll
-						.getUI();
-				if (e.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL) {
-					ui.scrollByUnits(direction, 2 * e.getScrollAmount());
-				} else if (e.getScrollType() == MouseWheelEvent.WHEEL_BLOCK_SCROLL) {
-					ui.scrollByBlock(direction);
-				}
-			}
-		}
-	}
-
 	/**
 	 * Layout manager to adjust the bounds of scrollbars and the viewport when
 	 * the default ({@link SubstanceScrollPaneBorder}) border is set on the
