@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -44,8 +44,7 @@ import javax.swing.plaf.basic.BasicSliderUI;
 
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
-import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.painter.border.SubstanceBorderPainter;
 import org.jvnet.substance.painter.gradient.ClassicGradientPainter;
 import org.jvnet.substance.painter.gradient.SubstanceGradientPainter;
@@ -159,10 +158,13 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 					trackRight - trackLeft, trackBottom - trackTop);
 		} else {
 			if (this.slider.getComponentOrientation().isLeftToRight()) {
-				trackLeft = 6 + this.insetCache.left + this.focusInsets.left;
+				trackLeft = trackRect.x + this.insetCache.left
+						+ this.focusInsets.left;
 				trackRight = trackLeft + this.getTrackWidth() - 1;
 			} else {
-				trackRight = this.slider.getWidth() - 8 - this.insetCache.right;
+				trackRight = trackRect.x + trackRect.width
+						- this.insetCache.right - this.focusInsets.right;
+				// this.slider.getWidth() - 8 - this.insetCache.right;
 				trackLeft = trackRight - this.getTrackWidth() + 1;
 			}
 			trackBottom = this.trackRect.height - 1;
@@ -213,16 +215,23 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 								: ComponentState.DISABLED_UNSELECTED);
 
 		this.paintSliderTrack(graphics, drawInverted, trackScheme,
-				SubstanceColorSchemeUtilities.getBorderColorScheme(this.slider,
-						currState), width, height);
+				SubstanceColorSchemeUtilities.getColorScheme(this.slider,
+						ColorSchemeAssociationKind.BORDER, currState), width,
+				height);
 
 		SubstanceColorScheme trackSchemeSelected = SubstanceColorSchemeUtilities
 				.getColorScheme(this.slider, currState);
+		SubstanceColorScheme trackBorderSchemeSelected = SubstanceColorSchemeUtilities
+				.getColorScheme(this.slider, ColorSchemeAssociationKind.BORDER,
+						currState);
 		FadeState fadeState = SubstanceFadeUtilities.getFadeState(this.slider,
 				FadeKind.ROLLOVER, FadeKind.SELECTION, FadeKind.PRESS);
 		if (fadeState != null) {
 			SubstanceColorScheme prevScheme = SubstanceColorSchemeUtilities
 					.getColorScheme(this.slider, prevState);
+			SubstanceColorScheme prevBorderScheme = SubstanceColorSchemeUtilities
+					.getColorScheme(this.slider,
+							ColorSchemeAssociationKind.BORDER, prevState);
 			float cyclePos = fadeState.getFadePosition();
 			if (!fadeState.isFadingIn())
 				cyclePos = 1.0f - cyclePos;
@@ -231,20 +240,22 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 				graphics.setComposite(TransitionLayout.getAlphaComposite(
 						this.slider, 1.0f - cyclePos, g));
 				this.paintSliderTrackSelected(graphics, drawInverted,
-						paintRect, prevScheme, width, height);
+						paintRect, prevScheme, prevBorderScheme, width, height);
 			}
 			if (currState != ComponentState.DEFAULT) {
 				graphics.setComposite(TransitionLayout.getAlphaComposite(
 						this.slider, cyclePos, g));
 				this.paintSliderTrackSelected(graphics, drawInverted,
-						paintRect, trackSchemeSelected, width, height);
+						paintRect, trackSchemeSelected,
+						trackBorderSchemeSelected, width, height);
 			}
 		} else {
 			boolean hasFill = currState.isKindActive(FadeKind.ROLLOVER)
 					|| currState.isKindActive(FadeKind.PRESS);
 			if (hasFill) {
 				this.paintSliderTrackSelected(graphics, drawInverted,
-						paintRect, trackSchemeSelected, width, height);
+						paintRect, trackSchemeSelected,
+						trackBorderSchemeSelected, width, height);
 			}
 		}
 
@@ -303,8 +314,9 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 					fillColorScheme, 0, false, false);
 
 			GeneralPath contourInner = SubstanceOutlineUtilities
-					.getBaseOutline(width + 1, height + 1, radius, null,
-							borderThickness + borderDelta);
+					.getBaseOutline(width + 1, height + 1, radius
+							- borderThickness, null, borderThickness
+							+ borderDelta);
 			borderPainter.paintBorder(cacheGraphics, slider, width + 1,
 					height + 1, contour, contourInner, borderScheme,
 					borderScheme, 0, false);
@@ -329,8 +341,10 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	 *            reversed.
 	 * @param paintRect
 	 *            Selected portion.
-	 * @param scheme
-	 *            Color scheme.
+	 * @param fillScheme
+	 *            Fill color scheme.
+	 * @param borderScheme
+	 *            Border color scheme.
 	 * @param width
 	 *            Track width.
 	 * @param height
@@ -338,7 +352,8 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	 */
 	private void paintSliderTrackSelected(Graphics2D graphics,
 			boolean drawInverted, Rectangle paintRect,
-			SubstanceColorScheme scheme, int width, int height) {
+			SubstanceColorScheme fillScheme, SubstanceColorScheme borderScheme,
+			int width, int height) {
 
 		Graphics2D g2d = (Graphics2D) graphics.create();
 		Insets insets = this.slider.getInsets();
@@ -381,11 +396,11 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 							fillWidth, fillHeight, radius, null, borderDelta);
 					g2d.translate(fillMinX, 0);
 					gp.paintContourBackground(g2d, this.slider, fillWidth,
-							fillHeight, contour, false, scheme, scheme, 0.0f,
-							false, false);
+							fillHeight, contour, false, fillScheme, fillScheme,
+							0.0f, false, false);
 					borderPainter.paintBorder(g2d, this.slider, fillWidth,
-							fillHeight, contour, null, scheme, scheme, 0.0f,
-							false);
+							fillHeight, contour, null, borderScheme,
+							borderScheme, 0.0f, false);
 				}
 			} else {
 				int middleOfThumb = this.thumbRect.y
@@ -410,11 +425,11 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 							fillWidth, fillHeight, radius, null, borderDelta);
 
 					gp.paintContourBackground(g2d, this.slider, fillWidth,
-							fillHeight, contour, false, scheme, scheme, 0.0f,
-							false, false);
+							fillHeight, contour, false, fillScheme, fillScheme,
+							0.0f, false, false);
 					borderPainter.paintBorder(g2d, this.slider, fillWidth,
-							fillHeight, contour, null, scheme, scheme, 0.0f,
-							false);
+							fillHeight, contour, null, borderScheme,
+							borderScheme, 0.0f, false);
 				}
 			}
 		}
@@ -534,6 +549,15 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 				this.paintFocus(graphics);
 		}
 
+		// graphics.setColor(Color.red);
+		// graphics.draw(trackRect);
+		// graphics.setColor(Color.green);
+		// graphics.draw(tickRect);
+		// graphics.setColor(Color.blue);
+		// graphics.draw(thumbRect);
+		// graphics.setColor(Color.magenta);
+		// graphics.draw(labelRect);
+
 		graphics.dispose();
 	}
 
@@ -599,7 +623,7 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	protected void installListeners(final JSlider slider) {
 		super.installListeners(slider);
 
-		// fix for defect 109 - memory leak on changing theme
+		// fix for defect 109 - memory leak on changing skin
 		this.substanceRolloverListener = new RolloverControlListener(this,
 				this.thumbModel);
 		slider.addMouseListener(this.substanceRolloverListener);
@@ -641,7 +665,7 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 	protected void uninstallListeners(JSlider slider) {
 		super.uninstallListeners(slider);
 
-		// fix for defect 109 - memory leak on changing theme
+		// fix for defect 109 - memory leak on changing skin
 		slider.removeMouseListener(this.substanceRolloverListener);
 		slider.removeMouseMotionListener(this.substanceRolloverListener);
 		this.substanceRolloverListener = null;
@@ -707,7 +731,8 @@ public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
 		Rectangle tickBounds = this.tickRect;
 		SubstanceColorScheme tickScheme = SubstanceColorSchemeUtilities
 				.getColorScheme(this.slider,
-						this.slider.isEnabled() ? ComponentState.DEFAULT
+						ColorSchemeAssociationKind.SEPARATOR, this.slider
+								.isEnabled() ? ComponentState.DEFAULT
 								: ComponentState.DISABLED_UNSELECTED);
 		if (this.slider.getOrientation() == JSlider.HORIZONTAL) {
 			int value = this.slider.getMinimum()
