@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -42,8 +42,7 @@ import javax.swing.plaf.UIResource;
 import javax.swing.plaf.basic.BasicSpinnerUI;
 
 import org.jvnet.lafwidget.animation.*;
-import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.api.SubstanceConstants.Side;
 import org.jvnet.substance.utils.*;
 import org.jvnet.substance.utils.border.SubstanceBorder;
@@ -111,14 +110,14 @@ public class SubstanceSpinnerUI extends BasicSpinnerUI {
 										.getArrowStrokeWidth(fontSize),
 								SwingConstants.NORTH, scheme);
 					}
-				});
+				}, "substance.spinner.nextButton");
 		this.nextButton.setIcon(icon);
 
-		float spinnerButtonSize = SubstanceSizeUtils
-				.getSpinnerButtonWidth(SubstanceSizeUtils
+		int spinnerButtonSize = SubstanceSizeUtils
+				.getScrollBarWidth(SubstanceSizeUtils
 						.getComponentFontSize(spinner));
-		this.nextButton.setPreferredSize(new Dimension((int) spinnerButtonSize,
-				(int) spinnerButtonSize));
+		this.nextButton.setPreferredSize(new Dimension(spinnerButtonSize,
+				spinnerButtonSize));
 		this.nextButton.setMinimumSize(new Dimension(5, 5));
 
 		// Set<Side> openSides = EnumSet.of(Side.BOTTOM, Side.TOP, Side.RIGHT);
@@ -148,23 +147,25 @@ public class SubstanceSpinnerUI extends BasicSpinnerUI {
 						// System.out.println(spinner.getFont().getSize());
 						int fontSize = SubstanceSizeUtils
 								.getComponentFontSize(spinner);
+						float spinnerArrowIconHeight = SubstanceSizeUtils
+								.getSpinnerArrowIconHeight(fontSize);
+//						System.out.println("OLD height : "
+//								+ spinnerArrowIconHeight);
 						return SubstanceImageCreator.getArrowIcon(
 								SubstanceSizeUtils
 										.getSpinnerArrowIconWidth(fontSize),
-								SubstanceSizeUtils
-										.getSpinnerArrowIconHeight(fontSize),
-								SubstanceSizeUtils
+								spinnerArrowIconHeight, SubstanceSizeUtils
 										.getArrowStrokeWidth(fontSize),
 								SwingConstants.SOUTH, scheme);
 					}
-				});
+				}, "substance.spinner.prevButton");
 		this.prevButton.setIcon(icon);
 
-		float spinnerButtonSize = SubstanceSizeUtils
-				.getSpinnerButtonWidth(SubstanceSizeUtils
+		int spinnerButtonSize = SubstanceSizeUtils
+				.getScrollBarWidth(SubstanceSizeUtils
 						.getComponentFontSize(this.prevButton));
-		this.prevButton.setPreferredSize(new Dimension((int) spinnerButtonSize,
-				(int) spinnerButtonSize));
+		this.prevButton.setPreferredSize(new Dimension(spinnerButtonSize,
+				spinnerButtonSize));
 		this.prevButton.setMinimumSize(new Dimension(5, 5));
 
 		// Set<Side> openSides = EnumSet.of(Side.BOTTOM, Side.TOP, Side.RIGHT);
@@ -363,18 +364,21 @@ public class SubstanceSpinnerUI extends BasicSpinnerUI {
 			Graphics2D g2d = (Graphics2D) g.create();
 			g2d.setColor(Color.red);
 			SubstanceColorScheme borderColorScheme = SubstanceColorSchemeUtilities
-					.getBorderColorScheme(c, ComponentState.DEFAULT);
+					.getColorScheme(c, ColorSchemeAssociationKind.BORDER,
+							ComponentState.DEFAULT);
 			if (this.spinner.getComponentOrientation().isLeftToRight()) {
 				g2d.translate(this.spinner.getEditor().getX()
 						+ this.spinner.getEditor().getWidth(), -5);
-				SubstanceImageCreator.paintSimpleBorder(g2d, 2 * nextButton
-						.getWidth(), this.spinner.getHeight() + 10,
-						borderColorScheme, borderColorScheme, 0.0f);
+				SubstanceImageCreator.paintSimpleBorder(this.spinner, g2d,
+						2 * nextButton.getWidth(),
+						this.spinner.getHeight() + 10, borderColorScheme,
+						borderColorScheme, 0.0f);
 			} else {
 				g2d.translate(-5, -5);
-				SubstanceImageCreator.paintSimpleBorder(g2d, 5 + this.spinner
-						.getEditor().getX(), this.spinner.getHeight() + 10,
-						borderColorScheme, borderColorScheme, 0.0f);
+				SubstanceImageCreator.paintSimpleBorder(this.spinner, g2d,
+						5 + this.spinner.getEditor().getX(), this.spinner
+								.getHeight() + 10, borderColorScheme,
+						borderColorScheme, 0.0f);
 			}
 			g2d.dispose();
 		}
