@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -91,23 +91,21 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	 */
 	private Map<Component, Long> fadeModifiedIds;
 
-	// /**
-	// * Key - tab component. Value - ID of the (looping) fade transition that
-	// * animates the tab component when it has icon animation (with
-	// * {@link SubstanceLookAndFeel#TABBED_PANE_TAB_ANIMATION_KIND} property).
-	// */
-	// private Map<Component, Long> fadeIconIds;
-
 	/**
-	 * Map of previous fade states (for state-aware theme transitions).
+	 * Map of previous fade states (for state-aware color scheme transitions).
 	 */
 	private Map<Integer, ComponentState> prevStateMap;
 
 	/**
-	 * Map of next fade states (for state-aware theme transitions).
+	 * Map of next fade states (for state-aware color scheme transitions).
 	 */
 	private Map<Integer, ComponentState> nextStateMap;
 
+	/**
+	 * Currently selected index (for selection animations).
+	 */
+	private int currSelectedIndex;
+
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -142,8 +140,9 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	 */
 	protected final class TabbedContainerListener extends ContainerAdapter {
 		/**
-		 * Property change listeners on the tab components. <p/> Fixes defect
-		 * 135 - memory leaks on tabbed panes.
+		 * Property change listeners on the tab components.
+		 * <p/>
+		 * Fixes defect 135 - memory leaks on tabbed panes.
 		 */
 		private Map<Component, List<PropertyChangeListener>> listeners = new HashMap<Component, List<PropertyChangeListener>>();
 
@@ -330,44 +329,6 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 			// this.cleanListeners(tabComponent);
 		}
 
-		// /**
-		// * Cleans all listeners on the component.
-		// *
-		// * @param comp
-		// * Component.
-		// */
-		// private void cleanListeners(Component comp) {
-		// // if (comp instanceof JTabbedPane) {
-		// // JTabbedPane jtp = (JTabbedPane)comp;
-		// // SubstanceTabbedPaneUI ui = (SubstanceTabbedPaneUI)jtp
-		// // .getUI();
-		// // // TabbedContainerListener tcl = ui.containerListener;
-		// // // tcl.stopTrackExistingTabs();
-		// // // tcl.listeners.clear();
-		// // // //jtp.removeContainerListener(tcl);
-		// // //
-		// //
-		// // ui.uninstallUI(jtp);
-		// // }
-		//
-		// if (comp instanceof Container) {
-		// Container cont = (Container) comp;
-		// for (int i = 0; i < cont.getComponentCount(); i++) {
-		// this.cleanListeners(cont.getComponent(i));
-		// }
-		// }
-		// if (comp instanceof JTabbedPane) {
-		// JTabbedPane jtp = (JTabbedPane) comp;
-		// SubstanceTabbedPaneUI ui = (SubstanceTabbedPaneUI) jtp.getUI();
-		// // TabbedContainerListener tcl = ui.containerListener;
-		// // tcl.stopTrackExistingTabs();
-		// // tcl.listeners.clear();
-		// // //jtp.removeContainerListener(tcl);
-		// //
-		//
-		// ui.uninstallUI(jtp);
-		// }
-		// }
 	}
 
 	/**
@@ -663,6 +624,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		super();
 		this.prevStateMap = new HashMap<Integer, ComponentState>();
 		this.nextStateMap = new HashMap<Integer, ComponentState>();
+		this.currSelectedIndex = -1;
 	}
 
 	/*
@@ -710,12 +672,28 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 						int selected = SubstanceTabbedPaneUI.this.tabPane
 								.getSelectedIndex();
 						FadeTracker fadeTracker = FadeTracker.getInstance();
+
+						// fix for issue 437 - track the selection change,
+						// fading out the previously selected tab
+						if ((currSelectedIndex >= 0)
+								&& (currSelectedIndex < SubstanceTabbedPaneUI.this.tabPane
+										.getTabCount())
+								&& SubstanceTabbedPaneUI.this.tabPane
+										.isEnabledAt(currSelectedIndex)) {
+							fadeTracker.trackFadeOut(FadeKind.SELECTION,
+									SubstanceTabbedPaneUI.this.tabPane,
+									currSelectedIndex, true,
+									new TabRepaintCallback(
+											SubstanceTabbedPaneUI.this.tabPane,
+											currSelectedIndex));
+						}
+						currSelectedIndex = selected;
 						if ((selected >= 0)
 								&& (selected < SubstanceTabbedPaneUI.this.tabPane
 										.getTabCount())
 								&& SubstanceTabbedPaneUI.this.tabPane
 										.isEnabledAt(selected)) {
-							fadeTracker.trackFadeIn(FadeKind.ROLLOVER,
+							fadeTracker.trackFadeIn(FadeKind.SELECTION,
 									SubstanceTabbedPaneUI.this.tabPane,
 									selected, true, new TabRepaintCallback(
 											SubstanceTabbedPaneUI.this.tabPane,
@@ -864,9 +842,11 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		int borderThickness = (int) SubstanceSizeUtils
 				.getBorderStrokeWidth(SubstanceSizeUtils
 						.getComponentFontSize(tabPane));
-		GeneralPath contourInner = SubstanceOutlineUtilities.getBaseOutline(
-				width, height + dy, cornerRadius, straightSides,
-				borderThickness + borderInsets);
+		GeneralPath contourInner = borderPainter.isPaintingInnerContour() ? SubstanceOutlineUtilities
+				.getBaseOutline(width, height + dy, cornerRadius
+						- borderThickness, straightSides, borderThickness
+						+ borderInsets)
+				: null;
 
 		borderPainter.paintBorder(resGraphics, tabPane, width, height + dy,
 				contour, contourInner, borderScheme, borderScheme2, cyclePos,
@@ -955,30 +935,6 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 						SwingConstants.TOP, colorScheme, colorScheme2,
 						borderScheme, borderScheme2, false);
 				if (isSelected) {
-
-					// Graphics2D g2d = backgroundImage.createGraphics();
-					// LinearGradientPaint paint = new LinearGradientPaint(
-					// 0,
-					// 0,
-					// 0,
-					// backgroundImage.getHeight(),
-					// new float[] { 0.0f,
-					// (float) skin.getSelectedTabFadeStart(),
-					// (float) skin.getSelectedTabFadeEnd(), 1.0f },
-					// new Color[] { new Color(255, 255, 255, 0),
-					// new Color(255, 255, 255, 0), Color.white,
-					// Color.white });
-					// g2d.setPaint(paint);
-					// g2d.setComposite(AlphaComposite.DstOut);
-					// g2d.fillRect(0, 0, backgroundImage.getWidth(),
-					// backgroundImage.getHeight());
-					// g2d.setComposite(AlphaComposite.SrcOver);
-					// g2d.drawImage(SubstanceTabbedPaneUI.getTabBackground(
-					// tabPane, width, height, isSelected, cyclePos,
-					// tabPlacement, colorScheme, colorScheme2,
-					// borderScheme, borderScheme2, true), 0, 0, null);
-					// g2d.dispose();
-
 					int fw = backgroundImage.getWidth();
 					int fh = backgroundImage.getHeight();
 					BufferedImage fade = SubstanceCoreUtilities.getBlankImage(
@@ -1020,15 +976,20 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	 * @param toPaintBorder
 	 *            Indication whether the button background (including contour)
 	 *            needs to be painted.
-	 * @param colorScheme
+	 * @param fillScheme
 	 *            Color scheme for coloring the background.
-	 * @param colorScheme2
+	 * @param fillScheme2
 	 *            Second color scheme for coloring the background.
+	 * @param markScheme
+	 *            Color scheme for painting the close mark.
+	 * @param markScheme2
+	 *            Second color scheme for painting the close mark.
 	 * @return Image of the close button of specified parameters.
 	 */
 	private static BufferedImage getCloseButtonImage(JTabbedPane tabPane,
 			int width, int height, float cyclePos, boolean toPaintBorder,
-			SubstanceColorScheme colorScheme, SubstanceColorScheme colorScheme2) {
+			SubstanceColorScheme fillScheme, SubstanceColorScheme fillScheme2,
+			SubstanceColorScheme markScheme, SubstanceColorScheme markScheme2) {
 		SubstanceGradientPainter gradientPainter = SubstanceCoreUtilities
 				.getGradientPainter(tabPane);
 		if (gradientPainter == null)
@@ -1036,7 +997,8 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 
 		HashMapKey key = SubstanceCoreUtilities.getHashKey(width, height,
 				toPaintBorder, cyclePos, gradientPainter.getDisplayName(),
-				colorScheme.getDisplayName(), colorScheme2.getDisplayName());
+				fillScheme.getDisplayName(), fillScheme2.getDisplayName(),
+				markScheme.getDisplayName(), markScheme2.getDisplayName());
 		BufferedImage result = SubstanceTabbedPaneUI.closeButtonMap.get(key);
 		if (result == null) {
 			result = SubstanceCoreUtilities.getBlankImage(width, height);
@@ -1046,17 +1008,16 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 				GeneralPath contour = SubstanceOutlineUtilities.getBaseOutline(
 						width, height, 1, null);
 				gradientPainter.paintContourBackground(finalGraphics, tabPane,
-						width, height, contour, false, colorScheme,
-						colorScheme2, cyclePos, true,
-						colorScheme != colorScheme2);
+						width, height, contour, false, fillScheme, fillScheme2,
+						cyclePos, true, fillScheme != fillScheme2);
 				// finalGraphics.drawImage(background, 0, 0, null);
 				SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
 						.getBorderPainter(tabPane);
 				finalGraphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
 						RenderingHints.VALUE_ANTIALIAS_ON);
 				borderPainter.paintBorder(finalGraphics, tabPane, width,
-						height, contour, null, colorScheme, colorScheme2,
-						cyclePos, colorScheme != colorScheme2);
+						height, contour, null, markScheme, markScheme2,
+						cyclePos, markScheme != markScheme2);
 			}
 
 			finalGraphics.setStroke(new BasicStroke(SubstanceSizeUtils
@@ -1070,20 +1031,20 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 				delta--;
 			int iconSize = width - delta;
 
-			if (colorScheme == colorScheme2) {
+			if (markScheme == markScheme2) {
 				// little optimization
 				cyclePos = 0.0f;
 			}
 			if (cyclePos > 0.0) {
 				Icon closeIcon2 = SubstanceImageCreator.getCloseIcon(iconSize,
-						colorScheme2);
+						markScheme2, markScheme2);
 				closeIcon2.paintIcon(tabPane, finalGraphics, delta / 2,
 						delta / 2);
 			}
 
 			if (cyclePos < 1.0) {
 				Icon closeIcon = SubstanceImageCreator.getCloseIcon(iconSize,
-						colorScheme);
+						markScheme, markScheme);
 				finalGraphics.setComposite(AlphaComposite.getInstance(
 						AlphaComposite.SRC_OVER, 1.0f - cyclePos));
 				closeIcon.paintIcon(tabPane, finalGraphics, delta / 2,
@@ -1106,7 +1067,6 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	protected void paintTabBackground(Graphics g, int tabPlacement,
 			final int tabIndex, final int x, final int y, int w, int h,
 			boolean isSelected) {
-
 		Graphics2D graphics = (Graphics2D) g.create();
 		graphics.setComposite(TransitionLayout.getAlphaComposite(this.tabPane,
 				g));
@@ -1117,14 +1077,25 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 			prevState = currState;
 
 		SubstanceColorScheme colorScheme = SubstanceColorSchemeUtilities
-				.getColorScheme(this.tabPane, tabIndex, currState);
+				.getColorScheme(this.tabPane, tabIndex,
+						ColorSchemeAssociationKind.TAB, currState);
 		SubstanceColorScheme colorScheme2 = SubstanceColorSchemeUtilities
-				.getColorScheme(this.tabPane, tabIndex, prevState);
+				.getColorScheme(this.tabPane, tabIndex,
+						ColorSchemeAssociationKind.TAB, prevState);
 
 		SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-				.getBorderColorScheme(this.tabPane, tabIndex, currState);
+				.getColorScheme(this.tabPane, tabIndex,
+						ColorSchemeAssociationKind.TAB_BORDER, currState);
 		SubstanceColorScheme borderScheme2 = SubstanceColorSchemeUtilities
-				.getBorderColorScheme(this.tabPane, tabIndex, prevState);
+				.getColorScheme(this.tabPane, tabIndex,
+						ColorSchemeAssociationKind.TAB_BORDER, prevState);
+
+		SubstanceColorScheme markScheme = SubstanceColorSchemeUtilities
+				.getColorScheme(this.tabPane, tabIndex,
+						ColorSchemeAssociationKind.MARK, currState);
+		SubstanceColorScheme markScheme2 = SubstanceColorSchemeUtilities
+				.getColorScheme(this.tabPane, tabIndex,
+						ColorSchemeAssociationKind.MARK, prevState);
 
 		// fix for defect 138
 		graphics.clip(new Rectangle(x, y, w, h));
@@ -1166,6 +1137,10 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 					temp = borderScheme;
 					borderScheme = borderScheme2;
 					borderScheme2 = temp;
+
+					temp = markScheme;
+					markScheme = markScheme2;
+					markScheme2 = temp;
 				}
 			}
 		}
@@ -1214,8 +1189,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 					if (this.substanceMouseLocation != null) {
 						Rectangle bounds = new Rectangle();
 						bounds = this.getTabBounds(tabIndex, bounds);
-						if ((tabPlacement == SwingConstants.LEFT)
-								|| (tabPlacement == SwingConstants.RIGHT)) {
+						if (toRotateTabsOnPlacement(tabPlacement)) {
 							bounds = new Rectangle(bounds.x, bounds.y,
 									bounds.height, bounds.width);
 						}
@@ -1248,7 +1222,8 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 				BufferedImage closeButtonImage = SubstanceTabbedPaneUI
 						.getCloseButtonImage(this.tabPane, orig.width,
 								orig.height, cyclePos, toPaintCloseBorder,
-								colorScheme, colorScheme2);
+								colorScheme, colorScheme2, markScheme,
+								markScheme2);
 				graphics.drawImage(closeButtonImage, orig.x, orig.y, null);
 			}
 		}
@@ -1302,40 +1277,11 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 						return SubstanceImageCreator.getArrowIcon(fontSize,
 								direction, scheme);
 					}
-				});
+				}, "substance.tabbedpane.scroll." + direction);
 		ssb.setIcon(icon);
 		return ssb;
 	}
 
-	/*
-	 * (non-Javadoc)
-	 * 
-	 * @see javax.swing.plaf.basic.BasicTabbedPaneUI#createChangeListener()
-	 */
-	@Override
-	protected ChangeListener createChangeListener() {
-		return new TabSelectionHandler();
-	}
-
-	/**
-	 * Handler for tab selection.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	protected class TabSelectionHandler implements ChangeListener {
-		/*
-		 * (non-Javadoc)
-		 * 
-		 * @seejavax.swing.event.ChangeListener#stateChanged(javax.swing.event.
-		 * ChangeEvent)
-		 */
-		public void stateChanged(ChangeEvent e) {
-			JTabbedPane tabbedPane = (JTabbedPane) e.getSource();
-			tabbedPane.revalidate();
-			tabbedPane.repaint();
-		}
-	}
-
 	/*
 	 * (non-Javadoc)
 	 * 
@@ -1345,8 +1291,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	@Override
 	protected int calculateTabHeight(int tabPlacement, int tabIndex,
 			int fontHeight) {
-		boolean toSwap = (tabPlacement == SwingConstants.LEFT)
-				|| (tabPlacement == SwingConstants.RIGHT);
+		boolean toSwap = toRotateTabsOnPlacement(tabPlacement);
 		if (toSwap)
 			return this.getTabExtraWidth(tabPlacement, tabIndex)
 					+ super.calculateTabWidth(tabPlacement, tabIndex, this
@@ -1363,8 +1308,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	@Override
 	protected int calculateTabWidth(int tabPlacement, int tabIndex,
 			FontMetrics metrics) {
-		boolean toSwap = (tabPlacement == SwingConstants.LEFT)
-				|| (tabPlacement == SwingConstants.RIGHT);
+		boolean toSwap = toRotateTabsOnPlacement(tabPlacement);
 		if (toSwap)
 			return super.calculateTabHeight(tabPlacement, tabIndex, metrics
 					.getHeight());
@@ -1380,8 +1324,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	 */
 	@Override
 	protected int calculateMaxTabHeight(int tabPlacement) {
-		if ((tabPlacement == SwingConstants.LEFT)
-				|| (tabPlacement == SwingConstants.RIGHT))
+		if (toRotateTabsOnPlacement(tabPlacement))
 			return super.calculateMaxTabHeight(tabPlacement);
 		int result = 0;
 		for (int i = 0; i < this.tabPane.getTabCount(); i++)
@@ -1397,9 +1340,8 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	 */
 	@Override
 	protected int getTabRunOverlay(int tabPlacement) {
-		boolean toSwap = (tabPlacement == SwingConstants.TOP)
-				|| (tabPlacement == SwingConstants.BOTTOM);
-		if (!toSwap)
+		boolean toSwap = this.toRotateTabsOnPlacement(tabPlacement);
+		if (toSwap)
 			return super.getTabRunOverlay(tabPlacement);
 
 		return 0;
@@ -1414,8 +1356,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	@Override
 	protected void paintTab(Graphics g, int tabPlacement, Rectangle[] rects,
 			int tabIndex, Rectangle iconRect, Rectangle textRect) {
-		boolean toSwap = (tabPlacement == SwingConstants.LEFT)
-				|| (tabPlacement == SwingConstants.RIGHT);
+		boolean toSwap = toRotateTabsOnPlacement(tabPlacement);
 		if (toSwap) {
 			Graphics2D tempG = (Graphics2D) g.create();
 			Rectangle tabRect = rects[tabIndex];
@@ -1464,8 +1405,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		// - insets.right);
 		int height = calculateTabAreaHeight(tabPlacement, runCount,
 				maxTabHeight);
-		if ((tabPlacement == SwingConstants.LEFT)
-				|| (tabPlacement == SwingConstants.RIGHT))
+		if (toRotateTabsOnPlacement(tabPlacement))
 			height = Math.max(height, tabPane.getHeight());
 		// - insets.top
 		// - insets.bottom);
@@ -1526,8 +1466,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	protected Rectangle getCloseButtonRectangleForEvents(int tabIndex, int x,
 			int y, int w, int h) {
 		int tabPlacement = this.tabPane.getTabPlacement();
-		boolean toSwap = (tabPlacement == SwingConstants.LEFT)
-				|| (tabPlacement == SwingConstants.RIGHT);
+		boolean toSwap = toRotateTabsOnPlacement(tabPlacement);
 		if (!toSwap) {
 			return this.getCloseButtonRectangleForDraw(tabIndex, x, y, w, h);
 		}
@@ -2130,7 +2069,7 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 			int selectedIndex) {
 		SubstanceColorScheme scheme = SubstanceColorSchemeUtilities
 				.getColorScheme(this.tabPane, selectedIndex,
-						ComponentState.DEFAULT);
+						ColorSchemeAssociationKind.TAB, ComponentState.DEFAULT);
 		this.highlight = scheme.isDark() ? SubstanceColorUtilities
 				.getAlphaColor(scheme.getUltraDarkColor(), 100) : scheme
 				.getLightColor();
@@ -2189,7 +2128,8 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		// selected tab is not in run adjacent to content, OR
 		// selected tab is not visible (SCROLL_TAB_LAYOUT)
 		SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-				.getBorderColorScheme(this.tabPane, selectedIndex,
+				.getColorScheme(this.tabPane, selectedIndex,
+						ColorSchemeAssociationKind.TAB_BORDER,
 						ComponentState.SELECTED);
 		Color darkShadowColor = SubstanceColorUtilities.getMidBorderColor(
 				borderScheme, borderScheme, 0.0);
@@ -2302,7 +2242,8 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		// selected tab is not in run adjacent to content, OR
 		// selected tab is not visible (SCROLL_TAB_LAYOUT)
 		SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-				.getBorderColorScheme(this.tabPane, selectedIndex,
+				.getColorScheme(this.tabPane, selectedIndex,
+						ColorSchemeAssociationKind.TAB_BORDER,
 						ComponentState.SELECTED);
 		Color darkShadowColor = SubstanceColorUtilities.getMidBorderColor(
 				borderScheme, borderScheme, 0.0);
@@ -2413,7 +2354,8 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		// selected tab is not in run adjacent to content, OR
 		// selected tab is not visible (SCROLL_TAB_LAYOUT)
 		SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-				.getBorderColorScheme(this.tabPane, selectedIndex,
+				.getColorScheme(this.tabPane, selectedIndex,
+						ColorSchemeAssociationKind.TAB_BORDER,
 						ComponentState.SELECTED);
 		Color darkShadowColor = SubstanceColorUtilities.getMidBorderColor(
 				borderScheme, borderScheme, 0.0);
@@ -2527,7 +2469,8 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		// selected tab is not in run adjacent to content, OR
 		// selected tab is not visible (SCROLL_TAB_LAYOUT)
 		SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-				.getBorderColorScheme(this.tabPane, selectedIndex,
+				.getColorScheme(this.tabPane, selectedIndex,
+						ColorSchemeAssociationKind.TAB_BORDER,
 						ComponentState.SELECTED);
 		Color darkShadowColor = SubstanceColorUtilities.getMidBorderColor(
 				borderScheme, borderScheme, 0.0);
@@ -2612,11 +2555,10 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 	 * @return The current state for the specified tab.
 	 */
 	protected ComponentState getTabState(int tabIndex) {
-		ButtonModel synthModel = new DefaultButtonModel();
-		synthModel.setEnabled(this.tabPane.isEnabledAt(tabIndex));
-		synthModel.setRollover(this.getRolloverTabIndex() == tabIndex);
-		synthModel.setSelected(this.tabPane.getSelectedIndex() == tabIndex);
-		return ComponentState.getState(synthModel, null);
+		boolean isEnabled = this.tabPane.isEnabledAt(tabIndex);
+		boolean isRollover = this.getRolloverTabIndex() == tabIndex;
+		boolean isSelected = this.tabPane.getSelectedIndex() == tabIndex;
+		return ComponentState.getState(isEnabled, isRollover, isSelected);
 	}
 
 	/*
@@ -2667,8 +2609,66 @@ public class SubstanceTabbedPaneUI extends BasicTabbedPaneUI {
 		}
 	}
 
+	@Override
+	protected void paintIcon(Graphics g, int tabPlacement, int tabIndex,
+			Icon icon, Rectangle iconRect, boolean isSelected) {
+		if (icon == null)
+			return;
+
+		if (SubstanceCoreUtilities.useThemedDefaultIcon()) {
+			ComponentState currState = this.getTabState(tabIndex);
+			FadeTracker fadeTracker = FadeTracker.getInstance();
+			FadeState fadeState = fadeTracker.getFadeState(this.tabPane,
+					tabIndex, FadeKind.ROLLOVER);
+
+			if (fadeState == null) {
+				if (currState.isKindActive(FadeKind.ROLLOVER)
+						|| currState.isKindActive(FadeKind.SELECTION)
+						|| !currState.isKindActive(FadeKind.ENABLE)) {
+					// use the original (full color or disabled) icon
+					super.paintIcon(g, tabPlacement, tabIndex, icon, iconRect,
+							isSelected);
+					return;
+				}
+			}
+
+			Icon themed = SubstanceCoreUtilities.getThemedIcon(this.tabPane,
+					tabIndex, icon);
+			if (fadeState == null) {
+				super.paintIcon(g, tabPlacement, tabIndex, themed, iconRect,
+						isSelected);
+			} else {
+				Graphics2D g2d = (Graphics2D) g.create();
+				super.paintIcon(g2d, tabPlacement, tabIndex, icon, iconRect,
+						isSelected);
+				g2d.setComposite(TransitionLayout.getAlphaComposite(
+						this.tabPane, 1.0f - fadeState.getFadePosition(), g2d));
+				super.paintIcon(g2d, tabPlacement, tabIndex, themed, iconRect,
+						isSelected);
+				g2d.dispose();
+			}
+		} else {
+			super.paintIcon(g, tabPlacement, tabIndex, icon, iconRect,
+					isSelected);
+		}
+	}
+
 	@Override
 	protected MouseListener createMouseListener() {
 		return null;
 	}
+
+	/**
+	 * Extension point to allow horizontal orientation of left / right placed
+	 * tabs.
+	 * 
+	 * @param tabPlacement
+	 *            Tab placement.
+	 * @return Indication whether the tabs in the specified placement should be
+	 *         rotated.
+	 */
+	protected boolean toRotateTabsOnPlacement(int tabPlacement) {
+		return (tabPlacement == SwingConstants.LEFT)
+				|| (tabPlacement == SwingConstants.RIGHT);
+	}
 }
\ No newline at end of file
