@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -43,8 +43,7 @@ import javax.swing.table.*;
 
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
-import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.api.renderers.SubstanceDefaultTableHeaderCellRenderer;
 import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
 import org.jvnet.substance.utils.*;
@@ -76,12 +75,12 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 	protected ListSelectionListener substanceFadeSelectionListener;
 
 	/**
-	 * Map of previous fade states (for state-aware theme transitions).
+	 * Map of previous fade states (for state-aware color scheme transitions).
 	 */
 	private Map<Integer, ComponentState> prevStateMap;
 
 	/**
-	 * Map of next fade states (for state-aware theme transitions).
+	 * Map of next fade states (for state-aware color scheme transitions).
 	 */
 	private Map<Integer, ComponentState> nextStateMap;
 
@@ -157,7 +156,7 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 		if (columnModel != null) {
 			ListSelectionModel lsm = columnModel.getSelectionModel();
 			if (lsm != null) {
-				// fix for defect 109 - memory leak on theme switch
+				// fix for defect 109 - memory leak on skin switch
 				substanceHeaderListener = new TableHeaderListener(this);
 				lsm.addListSelectionListener(substanceHeaderListener);
 			}
@@ -192,7 +191,7 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 				SubstanceTableUI tableUI = (SubstanceTableUI) ui;
 				// System.out.println("Sel anim: "
 				// + tableUI.hasSelectionAnimations());
-				if (!tableUI.hasSelectionAnimations())
+				if (!tableUI._hasSelectionAnimations())
 					return;
 
 				Set<Long> initiatedFadeSequences = new HashSet<Long>();
@@ -320,7 +319,7 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 	 */
 	@Override
 	protected void uninstallListeners() {
-		// fix for defect 109 - memory leak on theme switch
+		// fix for defect 109 - memory leak on skin switch
 		TableColumnModel columnModel = header.getColumnModel();
 		if (columnModel != null) {
 			ListSelectionModel lsm = columnModel.getSelectionModel();
@@ -579,8 +578,8 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 		ComponentState currState = (header.isEnabled() && header.getTable()
 				.isEnabled()) ? ComponentState.DEFAULT
 				: ComponentState.DISABLED_UNSELECTED;
-		Color gridColor = SubstanceColorSchemeUtilities.getBorderColorScheme(
-				header, currState).getDarkColor();
+		Color gridColor = SubstanceColorSchemeUtilities.getColorScheme(header,
+				ColorSchemeAssociationKind.BORDER, currState).getLineColor();
 		return gridColor;
 	}
 
@@ -633,16 +632,10 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 		}
 
 		if (totalHighlightAlpha > 0.0f) {
-			SubstanceColorScheme prevHighlightScheme = SubstanceColorSchemeUtilities
-					.getHighlightColorScheme(header, prevState);
-			SubstanceColorScheme currHighlightScheme = SubstanceColorSchemeUtilities
-					.getHighlightColorScheme(header, currState);
-
 			g2d.setComposite(TransitionLayout.getAlphaComposite(this.header,
 					totalHighlightAlpha, g));
-			SubstanceHighlightUtils.paintHighlight(g2d, component, cellRect,
-					0.0f, null, currHighlightScheme, prevHighlightScheme,
-					fadeCoef);
+			SubstanceHighlightUtils.paintHighlight(g2d, rendererPane, component, cellRect,
+					0.0f, null, currState, prevState, fadeCoef);
 			g2d
 					.setComposite(TransitionLayout.getAlphaComposite(
 							this.header, g));
@@ -680,20 +673,27 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 	@Override
 	public void update(Graphics g, JComponent c) {
 		// fix for issue 175 - table header under resize mode off
-		// was painted in theme-agnostic (gray) color.
+		// was painted in color scheme-agnostic (gray) color.
 		ComponentState backgroundState = (this.header.isEnabled() && this.header
 				.getTable().isEnabled()) ? ComponentState.DEFAULT
 				: ComponentState.DISABLED_UNSELECTED;
-		SubstanceColorScheme scheme = SubstanceColorSchemeUtilities
-				.getColorScheme(c, backgroundState);
 
 		// fix for issue 360 - respect the clip bounds of the
 		// table header
 		Rectangle clip = g.getClipBounds();
 		if (clip == null)
 			clip = c.getBounds();
-		SubstanceHighlightUtils.paintHighlight(g, c, clip, 0.0f, null, scheme,
-				scheme, 0.0f);
+
+		// do not use the highlight scheme for painting the
+		// table header background
+		SubstanceColorScheme fillScheme = SubstanceColorSchemeUtilities
+				.getColorScheme(c, backgroundState);
+		SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
+				.getColorScheme(c, ColorSchemeAssociationKind.BORDER,
+						backgroundState);
+
+		SubstanceHighlightUtils.paintHighlight(g, null, c, clip, 0.0f, null,
+				fillScheme, fillScheme, borderScheme, borderScheme, 0.0f);
 		paint(g, c);
 	}
 
@@ -735,7 +735,7 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 
 		if (prevStateMap.containsKey(columnIndex))
 			return prevStateMap.get(columnIndex);
-		return ComponentState.DEFAULT;
+		return getColumnState(columnIndex);
 	}
 
 	/**
@@ -831,11 +831,9 @@ public class SubstanceTableHeaderUI extends BasicTableHeaderUI {
 			final ComponentState backgroundState = (header.isEnabled() && header
 					.getTable().isEnabled()) ? ComponentState.DEFAULT
 					: ComponentState.DISABLED_UNSELECTED;
-			SubstanceColorScheme scheme = SubstanceColorSchemeUtilities
-					.getColorScheme(header, backgroundState);
-			SubstanceHighlightUtils.paintHighlight(g2d, this.header,
+			SubstanceHighlightUtils.paintHighlight(g2d, null, this.header,
 					new Rectangle(0, 0, this.getWidth(), this.getHeight()),
-					0.0f, null, scheme, scheme, 0.0f);
+					0.0f, null, backgroundState, backgroundState, 0.0f);
 
 			g2d.setColor(getGridColor(this.header));
 			float strokeWidth = SubstanceSizeUtils
