@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -47,8 +47,9 @@ import org.jvnet.lafwidget.animation.*;
 import org.jvnet.lafwidget.layout.TransitionLayout;
 import org.jvnet.substance.api.*;
 import org.jvnet.substance.api.SubstanceConstants.Side;
-import org.jvnet.substance.api.renderers.SubstanceDefaultTableCellRenderer;
-import org.jvnet.substance.api.renderers.SubstanceDefaultTableHeaderCellRenderer;
+import org.jvnet.substance.api.renderers.*;
+import org.jvnet.substance.painter.decoration.DecorationAreaType;
+import org.jvnet.substance.painter.decoration.SubstanceDecorationUtilities;
 import org.jvnet.substance.painter.highlight.SubstanceHighlightUtils;
 import org.jvnet.substance.painter.utils.SubstanceFillBackgroundDelegate;
 import org.jvnet.substance.utils.*;
@@ -71,7 +72,7 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * Holds the currently rolled-over row-column index, or <code>null</code> if
 	 * none such.
 	 */
-	protected Comparable<?> rolledOverId;
+	protected TableId rolledOverId;
 
 	/**
 	 * Row index of the focused cell.
@@ -109,19 +110,13 @@ public class SubstanceTableUI extends BasicTableUI {
 	 */
 	protected RolloverFadeListener substanceFadeRolloverListener;
 
-	// /**
-	// * Delegate for painting the background of list cells.
-	// */
-	// private static SubstanceFillBackgroundDelegate backgroundDelegate = new
-	// SubstanceFillBackgroundDelegate();
-
 	/**
-	 * Map of previous fade states (for state-aware theme transitions).
+	 * Map of previous fade states (for state-aware color scheme transitions).
 	 */
 	private Map<TableCellId, ComponentState> prevStateMap;
 
 	/**
-	 * Map of next fade states (for state-aware theme transitions).
+	 * Map of next fade states (for state-aware color scheme transitions).
 	 */
 	private Map<TableCellId, ComponentState> nextStateMap;
 
@@ -156,6 +151,10 @@ public class SubstanceTableUI extends BasicTableUI {
 		this.rolledOverColumn = -1;
 		this.focusedRow = -1;
 		this.focusedColumn = -1;
+
+		this.rowId = new TableRowId(-1);
+		this.colId = new TableColumnId(-1);
+		this.cellId = new TableCellId(-1, -1);
 	}
 
 	/*
@@ -364,8 +363,12 @@ public class SubstanceTableUI extends BasicTableUI {
 			TableCellRenderer renderer) {
 		TableCellRenderer currRenderer = this.table.getDefaultRenderer(clazz);
 		if (currRenderer != null) {
-			boolean isSubstanceRenderer = (currRenderer instanceof SubstanceDefaultTableCellRenderer)
-					|| (currRenderer instanceof SubstanceDefaultTableCellRenderer.BooleanRenderer);
+			boolean isSubstanceRenderer = currRenderer.getClass()
+					.isAnnotationPresent(SubstanceRenderer.class);
+			// boolean isSubstanceRenderer = (currRenderer instanceof
+			// SubstanceDefaultTableCellRenderer)
+			// || (currRenderer instanceof
+			// SubstanceDefaultTableCellRenderer.BooleanRenderer);
 			if (!isSubstanceRenderer)
 				return;
 		}
@@ -425,13 +428,17 @@ public class SubstanceTableUI extends BasicTableUI {
 					nextStateMap.clear();
 					SubstanceTableUI.this.syncSelection();
 
-					// fix for issue 309 - syncing animations on tables
-					// and table headers.
-					SubstanceTableHeaderUI headerUI = (SubstanceTableHeaderUI) table
-							.getTableHeader().getUI();
-					headerUI.processColumnModelChangeEvent(
-							(TableColumnModel) evt.getOldValue(),
-							(TableColumnModel) evt.getNewValue());
+					JTableHeader tableHeader = table.getTableHeader();
+					// fix for issue 408 - table header may be null.
+					if (tableHeader != null) {
+						// fix for issue 309 - syncing animations on tables
+						// and table headers.
+						SubstanceTableHeaderUI headerUI = (SubstanceTableHeaderUI) tableHeader
+								.getUI();
+						headerUI.processColumnModelChangeEvent(
+								(TableColumnModel) evt.getOldValue(),
+								(TableColumnModel) evt.getNewValue());
+					}
 				}
 
 				// fix for defect 243 - not tracking changes to selection
@@ -593,6 +600,9 @@ public class SubstanceTableUI extends BasicTableUI {
 
 		// Paint the grid.
 		this.paintGrid(g, rMin, rMax, cMin, cMax);
+
+		// Paint the drop lines
+		this.paintDropLines(g);
 	}
 
 	/**
@@ -614,7 +624,8 @@ public class SubstanceTableUI extends BasicTableUI {
 		if (gridColor instanceof UIResource) {
 			SubstanceColorScheme scheme = SubstanceColorSchemeUtilities
 					.getColorScheme(this.table,
-							this.table.isEnabled() ? ComponentState.DEFAULT
+							ColorSchemeAssociationKind.BORDER, this.table
+									.isEnabled() ? ComponentState.DEFAULT
 									: ComponentState.DISABLED_UNSELECTED);
 			gridColor = scheme.getLineColor();
 		}
@@ -685,6 +696,7 @@ public class SubstanceTableUI extends BasicTableUI {
 
 		TableColumnModel cm = this.table.getColumnModel();
 		int columnMargin = cm.getColumnMargin();
+		int rowMargin = this.table.getRowMargin();
 
 		Rectangle cellRect;
 		Rectangle highlightCellRect;
@@ -698,8 +710,8 @@ public class SubstanceTableUI extends BasicTableUI {
 				// cellRect.y -= this.table.getRowMargin() / 2;
 				// cellRect.height += this.table.getRowMargin();
 				// }
-				highlightCellRect.y -= this.table.getRowMargin() / 2;
-				highlightCellRect.height += this.table.getRowMargin();
+				highlightCellRect.y -= rowMargin / 2;
+				highlightCellRect.height += rowMargin;
 
 				for (int column = cMin; column <= cMax; column++) {
 					aColumn = cm.getColumn(column);
@@ -720,8 +732,8 @@ public class SubstanceTableUI extends BasicTableUI {
 			for (int row = rMin; row <= rMax; row++) {
 				cellRect = this.table.getCellRect(row, cMin, false);
 				highlightCellRect = new Rectangle(cellRect);
-				highlightCellRect.y -= this.table.getRowMargin() / 2;
-				highlightCellRect.height += this.table.getRowMargin();
+				highlightCellRect.y -= rowMargin / 2;
+				highlightCellRect.height += rowMargin;
 
 				aColumn = cm.getColumn(cMin);
 
@@ -840,8 +852,6 @@ public class SubstanceTableUI extends BasicTableUI {
 		g2d.setComposite(TransitionLayout.getAlphaComposite(this.table, g));
 
 		TableCellId cellId = new TableCellId(row, column);
-		boolean isRollover = ((this.rolledOverId != null) && this.rolledOverId
-				.equals(cellId));
 
 		ComponentState prevState = this.getPrevCellState(cellId);
 		ComponentState currState = this.getCellState(cellId);
@@ -851,10 +861,10 @@ public class SubstanceTableUI extends BasicTableUI {
 
 		// optimize for tables that don't initiate rollover
 		// or selection animations
-		FadeState state = hasRolloverAnimations() || hasSelectionAnimations() ? SubstanceFadeUtilities
+		FadeState state = updateInfo.hasRolloverAnimations
+				|| updateInfo.hasSelectionAnimations ? SubstanceFadeUtilities
 				.getFadeState(this.table, cellId, FadeKind.SELECTION,
-						FadeKind.ROLLOVER)
-				: null;
+						FadeKind.ROLLOVER) : null;
 
 		boolean hasHighlights = (prevState != ComponentState.DEFAULT)
 				|| (currState != ComponentState.DEFAULT) || (state != null);
@@ -864,27 +874,25 @@ public class SubstanceTableUI extends BasicTableUI {
 		float totalHighlightAlpha = 0.0f;
 		SubstanceColorScheme prevHighlightScheme = null;
 		SubstanceColorScheme currHighlightScheme = null;
+		SubstanceColorScheme prevBorderScheme = null;
+		SubstanceColorScheme currBorderScheme = null;
 		float fadeHighlightCoef = 0.0f;
 
 		if (hasHighlights) {
 			prevHighlightScheme = this.updateInfo
 					.getHighlightColorScheme(prevState);
-			// SubstanceThemeUtilities
-			// .getHighlightColorScheme(this.table, prevState);
 			currHighlightScheme = this.updateInfo
 					.getHighlightColorScheme(currState);
-			// SubstanceThemeUtilities
-			// .getHighlightColorScheme(this.table, currState);
+			prevBorderScheme = this.updateInfo
+					.getHighlightBorderColorScheme(prevState);
+			currBorderScheme = this.updateInfo
+					.getHighlightBorderColorScheme(currState);
 
 			// Compute the alpha values for the animation.
 			float startHighlightAlpha = this.updateInfo
 					.getHighlightAlpha(prevState);
-			// SubstanceThemeUtilities
-			// .getHighlightAlpha(this.table, prevState);
 			float endHighlightAlpha = this.updateInfo
 					.getHighlightAlpha(currState);
-			// SubstanceThemeUtilities
-			// .getHighlightAlpha(this.table, currState);
 
 			totalHighlightAlpha = endHighlightAlpha;
 			// System.out.println("-------- (" + System.currentTimeMillis() %
@@ -925,7 +933,7 @@ public class SubstanceTableUI extends BasicTableUI {
 			// + "] to " + currTheme.getDisplayName() + "[at "
 			// + alphaForCurrBackground + "]");
 
-			if (!this.hasSelectionAnimations()
+			if (!updateInfo.hasSelectionAnimations
 					&& (prevState.isKindActive(FadeKind.SELECTION) || currState
 							.isKindActive(FadeKind.SELECTION))) {
 				// no animations on selected cells in big tables - bug 209
@@ -1000,6 +1008,8 @@ public class SubstanceTableUI extends BasicTableUI {
 			}
 		}
 
+		boolean isRollover = ((this.rolledOverId != null) && this.rolledOverId
+				.equals(cellId));
 		if (this.table.isEditing() && this.table.getEditingRow() == row
 				&& this.table.getEditingColumn() == column) {
 			Component component = this.table.getEditorComponent();
@@ -1011,18 +1021,19 @@ public class SubstanceTableUI extends BasicTableUI {
 			if (totalHighlightAlpha > 0.0f) {
 				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
 						totalHighlightAlpha, g));
-				SubstanceHighlightUtils.paintHighlight(g2d, component,
-						highlightCellRect, highlightBorderAlpha,
+				SubstanceHighlightUtils.paintHighlight(g2d, this.rendererPane,
+						component, highlightCellRect, highlightBorderAlpha,
 						highlightOpenSides, currHighlightScheme,
-						prevHighlightScheme, fadeHighlightCoef);
+						prevHighlightScheme, currBorderScheme,
+						prevBorderScheme, fadeHighlightCoef);
 				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
 						g));
 			}
 
 			if (component instanceof JComponent) {
-				// Play with opaqueness to make our own gradient background
+				// Play with opacity to make our own gradient background
 				// on selected elements to show.
-				// JComponent jRenderer = (JComponent) component;
+				//
 				// Compute the selection status to prevent flicker - JTable
 				// registers a listener on selection changes and repaints
 				// the relevant cell before our listener (in TableUI) gets
@@ -1036,7 +1047,7 @@ public class SubstanceTableUI extends BasicTableUI {
 					// SubstanceCoreUtilities.toDrawWatermark(this.table))
 					newOpaque = false;
 
-				Map<Component, Boolean> opacity = new HashMap<Component, Boolean>();
+				this.updateInfo.opacity.clear();
 				// System.out.println("Pre-painting at index " + row + " ["
 				// +
 				// value
@@ -1044,7 +1055,8 @@ public class SubstanceTableUI extends BasicTableUI {
 				// "transparent")
 				// + " with bg " + rendererComponent.getBackground());
 				if (!newOpaque)
-					SubstanceCoreUtilities.makeNonOpaque(component, opacity);
+					SubstanceCoreUtilities.makeNonOpaque(component,
+							this.updateInfo.opacity);
 				// System.out.println("Painting "
 				// + (newOpaque ? "opaque" : "transparent")
 				// + " with bg " + component.getBackground());
@@ -1055,7 +1067,8 @@ public class SubstanceTableUI extends BasicTableUI {
 				// + "] " + (newOpaque ? "opaque" : "transparent")
 				// + " with bg " + rendererComponent.getBackground());
 				if (!newOpaque)
-					SubstanceCoreUtilities.restoreOpaque(component, opacity);
+					SubstanceCoreUtilities.restoreOpaque(component,
+							this.updateInfo.opacity);
 				// System.out.println("Post-painting at index " + row + " ["
 				// +
 				// value
@@ -1077,15 +1090,14 @@ public class SubstanceTableUI extends BasicTableUI {
 					.getCellRenderer(row, column);
 			Component rendererComponent = this.table.prepareRenderer(renderer,
 					row, column);
-			boolean isWatermarkBleed = SubstanceCoreUtilities
-					.toDrawWatermark(this.table);
+			boolean isWatermarkBleed = this.updateInfo.toDrawWatermark;
 			if (rendererComponent != null) {
 				if (!isWatermarkBleed) {
 					Color background = rendererComponent.getBackground();
 					// optimization - only render background if it's different
 					// from the table background
 					if ((background != null)
-							&& !table.getBackground().equals(background)) {
+							&& (!table.getBackground().equals(background) || this.updateInfo.isInDecorationArea)) {
 						// fill with the renderer background color
 						g2d.setColor(background);
 						g2d.fillRect(highlightCellRect.x, highlightCellRect.y,
@@ -1122,14 +1134,15 @@ public class SubstanceTableUI extends BasicTableUI {
 				// + currTheme.getDisplayName() + ":"
 				// + prevTheme.getDisplayName() + ":"
 				// + fadeCoef);
-				SubstanceHighlightUtils.paintHighlight(g2d, rendererComponent,
-						new Rectangle(highlightCellRect.x - (int) extraWidth,
-								highlightCellRect.y - (int) extraHeight,
-								highlightCellRect.width + (int) extraWidth,
-								highlightCellRect.height + (int) extraHeight),
-						highlightBorderAlpha, highlightOpenSides,
-						currHighlightScheme, prevHighlightScheme,
-						fadeHighlightCoef);
+				SubstanceHighlightUtils.paintHighlight(g2d, this.rendererPane,
+						rendererComponent, new Rectangle(highlightCellRect.x
+								- (int) extraWidth, highlightCellRect.y
+								- (int) extraHeight, highlightCellRect.width
+								+ (int) extraWidth, highlightCellRect.height
+								+ (int) extraHeight), highlightBorderAlpha,
+						highlightOpenSides, currHighlightScheme,
+						prevHighlightScheme, currBorderScheme,
+						prevBorderScheme, fadeHighlightCoef);
 				g2d.setComposite(TransitionLayout.getAlphaComposite(this.table,
 						g));
 			}
@@ -1145,12 +1158,12 @@ public class SubstanceTableUI extends BasicTableUI {
 				// the first frame uses full opacity, and the next frame
 				// starts the fade sequence. So, we use the UI delegate to
 				// compute the selection status.
-				boolean isSelected = this.hasSelectionAnimations() ? this.selectedIndices
+				boolean isSelected = updateInfo.hasSelectionAnimations ? this.selectedIndices
 						.containsKey(cellId)
 						: this.table.isCellSelected(row, column);
 				boolean newOpaque = !(isSelected || isRollover || (state != null));
 
-				if (SubstanceCoreUtilities.toDrawWatermark(this.table))
+				if (this.updateInfo.toDrawWatermark)
 					newOpaque = false;
 
 				Map<Component, Boolean> opacity = new HashMap<Component, Boolean>();
@@ -1168,6 +1181,8 @@ public class SubstanceTableUI extends BasicTableUI {
 				// + (newOpaque ? "opaque" : "transparent")
 				// + " with bg " + rendererComponent.getBackground());
 				// System.out.println(row + ":" + column + ":" + cellRect);
+				// SubstanceDecorationUtilities.setDecorationType(jRenderer,
+				// this.updateInfo.decorationAreaType);
 				this.rendererPane.paintComponent(g2d, rendererComponent,
 						this.table, cellRect.x, cellRect.y, cellRect.width,
 						cellRect.height, true);
@@ -1196,6 +1211,134 @@ public class SubstanceTableUI extends BasicTableUI {
 		g2d.dispose();
 	}
 
+	private void paintDropLines(Graphics g) {
+		JTable.DropLocation loc = table.getDropLocation();
+		if (loc == null) {
+			return;
+		}
+
+		Color color = UIManager.getColor("Table.dropLineColor");
+		Color shortColor = UIManager.getColor("Table.dropLineShortColor");
+		if (color == null && shortColor == null) {
+			return;
+		}
+
+		Rectangle rect;
+
+		rect = getHDropLineRect(loc);
+		if (rect != null) {
+			int x = rect.x;
+			int w = rect.width;
+			if (color != null) {
+				extendRect(rect, true);
+				g.setColor(color);
+				g.fillRect(rect.x, rect.y, rect.width, rect.height);
+			}
+			if (!loc.isInsertColumn() && shortColor != null) {
+				g.setColor(shortColor);
+				g.fillRect(x, rect.y, w, rect.height);
+			}
+		}
+
+		rect = getVDropLineRect(loc);
+		if (rect != null) {
+			int y = rect.y;
+			int h = rect.height;
+			if (color != null) {
+				extendRect(rect, false);
+				g.setColor(color);
+				g.fillRect(rect.x, rect.y, rect.width, rect.height);
+			}
+			if (!loc.isInsertRow() && shortColor != null) {
+				g.setColor(shortColor);
+				g.fillRect(rect.x, y, rect.width, h);
+			}
+		}
+	}
+
+	private Rectangle getHDropLineRect(JTable.DropLocation loc) {
+		if (!loc.isInsertRow()) {
+			return null;
+		}
+
+		int row = loc.getRow();
+		int col = loc.getColumn();
+		if (col >= table.getColumnCount()) {
+			col--;
+		}
+
+		Rectangle rect = table.getCellRect(row, col, true);
+
+		if (row >= table.getRowCount()) {
+			row--;
+			Rectangle prevRect = table.getCellRect(row, col, true);
+			rect.y = prevRect.y + prevRect.height;
+		}
+
+		if (rect.y == 0) {
+			rect.y = -1;
+		} else {
+			rect.y -= 2;
+		}
+
+		rect.height = 3;
+
+		return rect;
+	}
+
+	private Rectangle getVDropLineRect(JTable.DropLocation loc) {
+		if (!loc.isInsertColumn()) {
+			return null;
+		}
+
+		boolean ltr = table.getComponentOrientation().isLeftToRight();
+		int col = loc.getColumn();
+		Rectangle rect = table.getCellRect(loc.getRow(), col, true);
+
+		if (col >= table.getColumnCount()) {
+			col--;
+			rect = table.getCellRect(loc.getRow(), col, true);
+			if (ltr) {
+				rect.x = rect.x + rect.width;
+			}
+		} else if (!ltr) {
+			rect.x = rect.x + rect.width;
+		}
+
+		if (rect.x == 0) {
+			rect.x = -1;
+		} else {
+			rect.x -= 2;
+		}
+
+		rect.width = 3;
+
+		return rect;
+	}
+
+	private Rectangle extendRect(Rectangle rect, boolean horizontal) {
+		if (rect == null) {
+			return rect;
+		}
+
+		if (horizontal) {
+			rect.x = 0;
+			rect.width = table.getWidth();
+		} else {
+			rect.y = 0;
+
+			if (table.getRowCount() != 0) {
+				Rectangle lastRect = table.getCellRect(table.getRowCount() - 1,
+						0, true);
+				rect.height = lastRect.y + lastRect.height;
+			} else {
+				rect.height = table.getHeight();
+			}
+		}
+
+		return rect;
+	}
+
 	/**
 	 * Repaints a single cell during the fade animation cycle.
 	 * 
@@ -1691,13 +1834,17 @@ public class SubstanceTableUI extends BasicTableUI {
 		}
 	}
 
+	public static interface TableId extends Comparable {
+		public TableId cloneId();
+	}
+
 	/**
 	 * ID of a single table cell.
 	 * 
 	 * @author Kirill Grouchnikov
 	 */
 	@SuppressWarnings("unchecked")
-	public static class TableCellId implements Comparable {
+	public static class TableCellId implements TableId {
 		/**
 		 * Cell row.
 		 */
@@ -1800,6 +1947,11 @@ public class SubstanceTableUI extends BasicTableUI {
 		public String toString() {
 			return this.row + ":" + this.column;
 		}
+
+		@Override
+		public TableId cloneId() {
+			return new TableCellId(row, column);
+		}
 	}
 
 	/**
@@ -1808,7 +1960,7 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @author Kirill Grouchnikov
 	 */
 	@SuppressWarnings("unchecked")
-	protected static class TableColumnId implements Comparable {
+	protected static class TableColumnId implements TableId {
 		/**
 		 * Column.
 		 */
@@ -1864,6 +2016,11 @@ public class SubstanceTableUI extends BasicTableUI {
 		public int hashCode() {
 			return (this.column ^ (this.column >>> 32));
 		}
+
+		@Override
+		public TableId cloneId() {
+			return new TableColumnId(this.column);
+		}
 	}
 
 	/**
@@ -1872,7 +2029,7 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @author Kirill Grouchnikov
 	 */
 	@SuppressWarnings("unchecked")
-	protected static class TableRowId implements Comparable {
+	protected static class TableRowId implements TableId {
 		/**
 		 * Row.
 		 */
@@ -1928,6 +2085,11 @@ public class SubstanceTableUI extends BasicTableUI {
 		public int hashCode() {
 			return (this.row ^ (this.row >>> 32));
 		}
+
+		@Override
+		public TableId cloneId() {
+			return new TableRowId(this.row);
+		}
 	}
 
 	/**
@@ -2061,7 +2223,11 @@ public class SubstanceTableUI extends BasicTableUI {
 				SubstanceTableUI.this.rolledOverId = null;
 			} else {
 				// check if this is the same index
-				Comparable<?> newId = SubstanceTableUI.this.getId(row, column);
+
+				// need to clone ID since it can go to the fade tracker
+				// and be set as the rollover id
+				TableId newId = SubstanceTableUI.this.getId(row, column)
+						.cloneId();
 				// Comparable currId = (Comparable) table
 				// .getClientProperty(ROLLED_OVER_INDEX);
 				if ((SubstanceTableUI.this.rolledOverId != null)
@@ -2071,7 +2237,7 @@ public class SubstanceTableUI extends BasicTableUI {
 				this.fadeOut();
 				FadeTrackerCallback callback = SubstanceTableUI.this
 						.getCallback(row, column);
-				if (SubstanceTableUI.this.hasRolloverAnimations()) {
+				if (SubstanceTableUI.this._hasRolloverAnimations()) {
 					FadeTracker.getInstance()
 							.trackFadeIn(FadeKind.ROLLOVER,
 									SubstanceTableUI.this.table, newId, false,
@@ -2140,7 +2306,7 @@ public class SubstanceTableUI extends BasicTableUI {
 			if (SubstanceTableUI.this.rolledOverId != null) {
 				FadeTrackerCallback callback = SubstanceTableUI.this
 						.getCallback(SubstanceTableUI.this.rolledOverId);
-				if (SubstanceTableUI.this.hasRolloverAnimations()) {
+				if (SubstanceTableUI.this._hasRolloverAnimations()) {
 					FadeTracker.getInstance()
 							.trackFadeOut(FadeKind.ROLLOVER,
 									SubstanceTableUI.this.table,
@@ -2224,17 +2390,31 @@ public class SubstanceTableUI extends BasicTableUI {
 	 *            Column index.
 	 * @return Comparable ID for the specified location.
 	 */
-	public Comparable<?> getId(int row, int column) {
+	public TableId getId(int row, int column) {
 		boolean hasRowSelection = this.table.getRowSelectionAllowed();
 		boolean hasColumnSelection = this.table.getColumnSelectionAllowed();
 
-		if (hasRowSelection && !hasColumnSelection)
-			return new TableRowId(row);
-		if (!hasRowSelection && hasColumnSelection)
-			return new TableColumnId(column);
-		return new TableCellId(row, column);
+		// reuse the ID objects for optimizations - called a lot during
+		// painting and animations
+		if (hasRowSelection && !hasColumnSelection) {
+			rowId.row = row;
+			return rowId;
+		}
+		if (!hasRowSelection && hasColumnSelection) {
+			colId.column = column;
+			return colId;
+		}
+		cellId.column = column;
+		cellId.row = row;
+		return cellId;
 	}
 
+	TableRowId rowId;
+
+	TableColumnId colId;
+
+	TableCellId cellId;
+
 	/**
 	 * Synchronizes the current selection state.
 	 * 
@@ -2252,40 +2432,29 @@ public class SubstanceTableUI extends BasicTableUI {
 		int rows = this.table.getRowCount();
 		int cols = this.table.getColumnCount();
 
-		// fix for defect 209 - selection very slow on large tables with
-		// column selection set to true and row selection set to false.
-		// Solution - no selection animations on tables with more than 1000
-		// cells.
-		if (!this.hasSelectionAnimations()) {
-			// this.selectedIndices.clear();
-			// for (int i = 0; i < rows; i++) {
-			// for (int j = 0; j < cols; j++) {
-			// if (this.table.isCellSelected(i, j)) {
-			// TableCellId cellId = new TableCellId(i, j);
-			// this.selectedIndices.put(cellId, this.table.getModel()
-			// .getValueAt(i, j));
-			// }
-			// }
-			// }
-			this.prevStateMap.clear();
-			table.repaint();
-			return;
-		}
-		// Map<TableCellId, Object> currSelected = (Map<TableCellId, Object>)
-		// SubstanceTableUI.this.table
-		// .getClientProperty(SubstanceTableUI.SELECTED_INDICES);
-		// System.err.println("SyncSelection was " + currSelected.size());
-		// if (e != null) {
-		// System.err.println(e.getFirstIndex() + ":" + e.getLastIndex() + ":"
-		// + e.getValueIsAdjusting());
-		// }
-
 		int rowLeadIndex = this.table.getSelectionModel()
 				.getLeadSelectionIndex();
 		int colLeadIndex = this.table.getColumnModel().getSelectionModel()
 				.getLeadSelectionIndex();
 		boolean isFocusOwner = this.table.isFocusOwner();
 
+		// fix for defect 209 - selection very slow on large tables with
+		// column selection set to true and row selection set to false.
+		// Solution - no selection animations on tables with more than 1000
+		// cells.
+		if (!this._hasSelectionAnimations()) {
+			this.prevStateMap.clear();
+			table.repaint();
+
+			// fix for issue 414 - track focus on tables
+			// without selection animations
+			if (isFocusOwner) {
+				this.focusedRow = rowLeadIndex;
+				this.focusedColumn = colLeadIndex;
+			}
+			return;
+		}
+
 		Set<Long> initiatedFadeSequences = new HashSet<Long>();
 		boolean fadeCanceled = false;
 
@@ -2461,7 +2630,7 @@ public class SubstanceTableUI extends BasicTableUI {
 	public ComponentState getPrevCellState(TableCellId cellIndex) {
 		if (this.prevStateMap.containsKey(cellIndex))
 			return this.prevStateMap.get(cellIndex);
-		return ComponentState.DEFAULT;
+		return this.getCellState(cellIndex);
 	}
 
 	/**
@@ -2472,29 +2641,23 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @return The current state for the specified cell.
 	 */
 	public ComponentState getCellState(TableCellId cellIndex) {
-		ButtonModel synthModel = new DefaultButtonModel();
 		int row = cellIndex.row;
 		int column = cellIndex.column;
-		synthModel.setEnabled(this.table.isEnabled());
-		// Integer currRoIndex = (Integer) list
-		// .getClientProperty(ROLLED_OVER_INDEX);
 		Comparable<?> cellId = this.getId(row, column);
-		// synthModel
-		// .setRollover(((this.rolledOverId != null) && this.rolledOverId
-		// .equals(cellIndex)));
-		synthModel.setRollover(cellId.equals(this.rolledOverId));
-		if (this.hasSelectionAnimations()
+		boolean isEnabled = this.table.isEnabled();
+		boolean isRollover = cellId.equals(this.rolledOverId);
+		boolean isSelected = false;
+		boolean hasSelectionAnimations = (this.updateInfo != null) ? this.updateInfo.hasSelectionAnimations
+				: this._hasSelectionAnimations();
+		if (hasSelectionAnimations
 				&& FadeConfigurationManager.getInstance().fadeAllowed(
 						FadeKind.SELECTION, table))
-			synthModel.setSelected(this.selectedIndices.containsKey(cellId));
+			isSelected = this.selectedIndices.containsKey(cellId);
 		else {
-			// System.out.println(row + ":" + column + "-->"
-			// + this.table.isCellSelected(row, column));
-			synthModel.setSelected(this.table.isCellSelected(row, column));
+			isSelected = this.table.isCellSelected(row, column);
 		}
-		// System.out.println(row + ":" + column + "-->"
-		// + ComponentState.getState(synthModel, null));
-		return ComponentState.getState(synthModel, null);
+
+		return ComponentState.getState(isEnabled, isRollover, isSelected);
 	}
 
 	/**
@@ -2503,7 +2666,7 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @return <code>true</code> if the table has animations, <code>false</code>
 	 *         otherwise.
 	 */
-	protected boolean hasAnimations() {
+	protected boolean _hasAnimations() {
 		// fix for defects 164 and 209 - selection
 		// and deletion are very slow on large tables.
 		int rowCount = this.table.getRowCount();
@@ -2533,8 +2696,8 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @return <code>true</code> if the table has selection animations,
 	 *         <code>false</code> otherwise.
 	 */
-	public boolean hasSelectionAnimations() {
-		return this.hasAnimations()
+	protected boolean _hasSelectionAnimations() {
+		return this._hasAnimations()
 				&& !LafWidgetUtilities.hasNoFades(this.table,
 						FadeKind.SELECTION);
 	}
@@ -2545,8 +2708,8 @@ public class SubstanceTableUI extends BasicTableUI {
 	 * @return <code>true</code> if the table has rollover animations,
 	 *         <code>false</code> otherwise.
 	 */
-	public boolean hasRolloverAnimations() {
-		return this.hasAnimations()
+	protected boolean _hasRolloverAnimations() {
+		return this._hasAnimations()
 				&& !LafWidgetUtilities
 						.hasNoFades(this.table, FadeKind.ROLLOVER);
 	}
@@ -2589,6 +2752,7 @@ public class SubstanceTableUI extends BasicTableUI {
 		this.paint(g2d, c);
 		SubstanceStripingUtils.tearDown(c);
 		g2d.dispose();
+		this.updateInfo = null;
 	}
 
 	/**
@@ -2606,6 +2770,24 @@ public class SubstanceTableUI extends BasicTableUI {
 		return null;
 	}
 
+	public SubstanceColorScheme getHighlightColorScheme(ComponentState state) {
+		if (this.updateInfo != null)
+			return updateInfo.getHighlightColorScheme(state);
+		return null;
+	}
+
+	public boolean hasSelectionAnimations() {
+		if (this.updateInfo != null)
+			return this.updateInfo.hasSelectionAnimations;
+		return this._hasSelectionAnimations();
+	}
+
+	public boolean hasRolloverAnimations() {
+		if (this.updateInfo != null)
+			return this.updateInfo.hasRolloverAnimations;
+		return this._hasRolloverAnimations();
+	}
+
 	private UpdateOptimizationInfo updateInfo;
 
 	private class UpdateOptimizationInfo {
@@ -2613,10 +2795,22 @@ public class SubstanceTableUI extends BasicTableUI {
 
 		private Map<ComponentState, SubstanceColorScheme> highlightSchemeMap;
 
+		private Map<ComponentState, SubstanceColorScheme> borderSchemeMap;
+
 		private Map<ComponentState, Float> highlightAlphaMap;
 
 		public SubstanceColorScheme defaultScheme;
 
+		public boolean hasSelectionAnimations;
+
+		public boolean hasRolloverAnimations;
+
+		public Map<Component, Boolean> opacity;
+
+		public DecorationAreaType decorationAreaType;
+
+		public boolean isInDecorationArea;
+
 		public UpdateOptimizationInfo() {
 			this.toDrawWatermark = SubstanceCoreUtilities
 					.toDrawWatermark(table);
@@ -2626,17 +2820,40 @@ public class SubstanceTableUI extends BasicTableUI {
 					ComponentState.class);
 			this.highlightSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
 					ComponentState.class);
+			this.borderSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
+					ComponentState.class);
+			this.hasSelectionAnimations = _hasSelectionAnimations();
+			this.opacity = new HashMap<Component, Boolean>();
+			this.decorationAreaType = SubstanceDecorationUtilities
+					.getDecorationType(table);
+
+			SubstanceSkin skin = SubstanceCoreUtilities.getSkin(table);
+			this.isInDecorationArea = (this.decorationAreaType != null)
+					&& skin
+							.isRegisteredAsDecorationArea(this.decorationAreaType)
+					& TransitionLayout.isOpaque(table);
 		}
 
 		public SubstanceColorScheme getHighlightColorScheme(ComponentState state) {
 			if (!this.highlightSchemeMap.containsKey(state)) {
 				this.highlightSchemeMap.put(state,
-						SubstanceColorSchemeUtilities.getHighlightColorScheme(
-								table, state));
+						SubstanceColorSchemeUtilities.getColorScheme(table,
+								ColorSchemeAssociationKind.HIGHLIGHT, state));
 			}
 			return this.highlightSchemeMap.get(state);
 		}
 
+		public SubstanceColorScheme getHighlightBorderColorScheme(
+				ComponentState state) {
+			if (!this.borderSchemeMap.containsKey(state)) {
+				this.borderSchemeMap.put(state, SubstanceColorSchemeUtilities
+						.getColorScheme(table,
+								ColorSchemeAssociationKind.HIGHLIGHT_BORDER,
+								state));
+			}
+			return this.borderSchemeMap.get(state);
+		}
+
 		public float getHighlightAlpha(ComponentState state) {
 			if (!this.highlightAlphaMap.containsKey(state)) {
 				this.highlightAlphaMap.put(state, SubstanceColorSchemeUtilities
