@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -95,27 +95,16 @@ public class SubstanceTreeUI extends BasicTreeUI {
 	 */
 	protected MouseListener substanceRowSelectionListener;
 
-	// /**
-	// * If <code>true</code>, the mouse pointer is in the bounds of the
-	// * associated tree.
-	// */
-	// private boolean isInside = false;
-
 	/**
-	 * Map of previous fade states (for state-aware theme transitions).
+	 * Map of previous fade states (for state-aware color scheme transitions).
 	 */
 	private Map<TreePathId, ComponentState> prevStateMap;
 
 	/**
-	 * Map of next fade states (for state-aware theme transitions).
+	 * Map of next fade states (for state-aware color scheme transitions).
 	 */
 	private Map<TreePathId, ComponentState> nextStateMap;
 
-	// /**
-	// * Current hash color.
-	// */
-	// private Color currHashColor;
-
 	/**
 	 * The current default color scheme. Is computed in
 	 * {@link #update(Graphics, JComponent)} and reused in
@@ -209,8 +198,19 @@ public class SubstanceTreeUI extends BasicTreeUI {
 			Rectangle bounds, TreePath path, int row, boolean isExpanded,
 			boolean hasBeenExpanded, boolean isLeaf) {
 		// Don't paint the renderer if editing this row.
-		if ((this.editingComponent != null) && (this.editingRow == row))
-			return;
+		if ((this.editingComponent != null) && (this.editingRow == row)) {
+			// fix for issue 446 - paint the expand control
+			// on the editing row
+			if (shouldPaintExpandControl(path, row, isExpanded,
+					hasBeenExpanded, isLeaf)) {
+				if (!this.tree.getComponentOrientation().isLeftToRight()
+						&& LookUtils.IS_JAVA_5) {
+					bounds.x -= 4;
+				}
+				paintExpandControlEnforce(g, clipBounds, insets, bounds, path,
+						row, isExpanded, hasBeenExpanded, isLeaf);
+			}
+		}
 
 		int leadIndex;
 
@@ -267,11 +267,6 @@ public class SubstanceTreeUI extends BasicTreeUI {
 		ComponentState prevState = this.getPrevPathState(pathId);
 		ComponentState currState = this.getPathState(pathId);
 
-		SubstanceColorScheme prevScheme = SubstanceColorSchemeUtilities
-				.getHighlightColorScheme(tree, prevState);
-		SubstanceColorScheme currScheme = SubstanceColorSchemeUtilities
-				.getHighlightColorScheme(tree, currState);
-
 		// Compute the alpha values for the animation.
 		float startAlpha = SubstanceColorSchemeUtilities.getHighlightAlpha(
 				this.tree, prevState);
@@ -311,11 +306,13 @@ public class SubstanceTreeUI extends BasicTreeUI {
 			// Fix for defect 180 - painting the
 			// highlight beneath the entire row
 			SubstanceHighlightUtils
-					.paintHighlight(g2d, renderer, new Rectangle(this.tree
-							.getInsets().left, bounds.y, this.tree.getWidth()
-							- this.tree.getInsets().right
-							- this.tree.getInsets().left, bounds.height), 0.8f,
-							null, currScheme, prevScheme, fadeCoef);
+					.paintHighlight(g2d, this.rendererPane, renderer,
+							new Rectangle(this.tree.getInsets().left, bounds.y,
+									this.tree.getWidth()
+											- this.tree.getInsets().right
+											- this.tree.getInsets().left,
+									bounds.height), 0.8f, null, currState,
+							prevState, fadeCoef);
 			g2d.setComposite(TransitionLayout.getAlphaComposite(this.tree,
 					currFactor));
 		}
@@ -352,7 +349,7 @@ public class SubstanceTreeUI extends BasicTreeUI {
 
 		// Paint the expand control now after the row background has been
 		// overlayed by the highlight background on selected and rolled over
-		// rows. See comments on paintExpandControlEnforce().
+		// rows. See comments on paintExpandControl().
 		if (shouldPaintExpandControl(path, row, isExpanded, hasBeenExpanded,
 				isLeaf)) {
 			if (!this.tree.getComponentOrientation().isLeftToRight()
@@ -991,7 +988,7 @@ public class SubstanceTreeUI extends BasicTreeUI {
 	public ComponentState getPrevPathState(TreePathId pathId) {
 		if (this.prevStateMap.containsKey(pathId))
 			return this.prevStateMap.get(pathId);
-		return ComponentState.DEFAULT;
+		return this.getPathState(pathId);
 	}
 
 	/**
@@ -1002,13 +999,12 @@ public class SubstanceTreeUI extends BasicTreeUI {
 	 * @return The current state for the specified path.
 	 */
 	public ComponentState getPathState(TreePathId pathId) {
-		ButtonModel synthModel = new DefaultButtonModel();
-		synthModel.setEnabled(this.tree.isEnabled());
-		synthModel.setRollover((this.currRolloverPathId != null)
-				&& pathId.equals(this.currRolloverPathId));
 		int rowIndex = this.tree.getRowForPath(pathId.path);
-		synthModel.setSelected(this.tree.isRowSelected(rowIndex));
-		return ComponentState.getState(synthModel, null);
+		boolean isEnabled = this.tree.isEnabled();
+		boolean isRollover = (this.currRolloverPathId != null)
+				&& pathId.equals(this.currRolloverPathId);
+		boolean isSelected = this.tree.isRowSelected(rowIndex);
+		return ComponentState.getState(isEnabled, isRollover, isSelected);
 	}
 
 	/*
