@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -30,13 +30,11 @@
 package org.jvnet.substance.api;
 
 import java.awt.Component;
-import java.util.EnumMap;
-import java.util.Map;
+import java.util.*;
 
 import org.jvnet.lafwidget.animation.FadeKind;
 import org.jvnet.substance.colorscheme.BlendBiColorScheme;
 import org.jvnet.substance.painter.decoration.DecorationAreaType;
-import org.jvnet.substance.utils.SubstanceCoreUtilities;
 
 /**
  * Color scheme bundle. Defines the visual appearance of a single decoration
@@ -44,6 +42,7 @@ import org.jvnet.substance.utils.SubstanceCoreUtilities;
  * 
  * @author Kirill Grouchnikov
  * @see DecorationAreaType
+ * @see ColorSchemeAssociationKind
  * @see SubstanceSkin
  */
 @SubstanceApi
@@ -63,12 +62,6 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 */
 	protected SubstanceColorScheme disabledColorScheme;
 
-	/**
-	 * Maps from component state to the matching color schemes. This map doesn't
-	 * have to contain entries for all {@link ComponentState} instances.
-	 */
-	protected Map<ComponentState, SubstanceColorScheme> stateColorSchemeMap;
-
 	/**
 	 * Maps from component state to the alpha channel applied on color scheme.
 	 * This map doesn't have to contain entries for all {@link ComponentState}
@@ -76,27 +69,13 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 */
 	protected Map<ComponentState, Float> stateAlphaMap;
 
-	/**
-	 * Maps from component state to the matching color schemes for highlight
-	 * color schemes. This map doesn't have to contain entries for all {@link
-	 * ComponentState} instances.
-	 */
-	protected Map<ComponentState, SubstanceColorScheme> stateHighlightColorSchemeMap;
-
 	/**
 	 * Maps from component state to the alpha channel applied on highlight color
-	 * scheme. This map doesn't have to contain entries for all {@link
-	 * ComponentState} instances.
+	 * scheme. This map doesn't have to contain entries for all
+	 * {@link ComponentState} instances.
 	 */
 	protected Map<ComponentState, Float> stateHighlightSchemeAlphaMap;
 
-	/**
-	 * Maps from component state to the matching color schemes for border color
-	 * schemes. This map doesn't have to contain entries for all {@link
-	 * ComponentState} instances.
-	 */
-	protected Map<ComponentState, SubstanceColorScheme> stateBorderColorSchemeMap;
-
 	/**
 	 * If there is no explicitly registered color scheme for pressed component
 	 * states, this field will contain a synthesized color scheme for a pressed
@@ -111,15 +90,33 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 */
 	protected SubstanceColorScheme disabledSelectedScheme;
 
+	/**
+	 * Maps from color scheme association kinds to the map of color schemes.
+	 * Different visual parts of controls in the specific decoration are can be
+	 * painted with different color schemes. For example, a rollover button can
+	 * use a light orange scheme for the gradient fill and a dark gray scheme
+	 * for the border. In this case, this map will have:
+	 * 
+	 * <ul>
+	 * <li>An entry with key {@link ColorSchemeAssociationKind#FILL}. This entry
+	 * has a map entry with key {@link ComponentState#SELECTED} and value that
+	 * points to the light orange scheme.</li>
+	 * <li>An entry with key {@link ColorSchemeAssociationKind#BORDER}. This
+	 * entry has a map entry with key {@link ComponentState#SELECTED} and value
+	 * that points to the dark gray scheme.</li>
+	 * </ul>
+	 */
+	protected Map<ColorSchemeAssociationKind, Map<ComponentState, SubstanceColorScheme>> colorSchemeMap;
+
 	/**
 	 * Creates a new color scheme bundle.
 	 * 
 	 * @param activeColorScheme
-	 * 		The active color scheme of this bundle.
+	 *            The active color scheme of this bundle.
 	 * @param defaultColorScheme
-	 * 		The default color scheme of this bundle.
+	 *            The default color scheme of this bundle.
 	 * @param disabledColorScheme
-	 * 		The disabled color scheme of this bundle.
+	 *            The disabled color scheme of this bundle.
 	 */
 	public SubstanceColorSchemeBundle(SubstanceColorScheme activeColorScheme,
 			SubstanceColorScheme defaultColorScheme,
@@ -127,33 +124,36 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 		this.activeColorScheme = activeColorScheme;
 		this.defaultColorScheme = defaultColorScheme;
 		this.disabledColorScheme = disabledColorScheme;
-		this.stateColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-				ComponentState.class);
 		this.stateAlphaMap = new EnumMap<ComponentState, Float>(
 				ComponentState.class);
-		this.stateHighlightColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-				ComponentState.class);
 		this.stateHighlightSchemeAlphaMap = new EnumMap<ComponentState, Float>(
 				ComponentState.class);
-		this.stateBorderColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-				ComponentState.class);
+
+		this.colorSchemeMap = new HashMap<ColorSchemeAssociationKind, Map<ComponentState, SubstanceColorScheme>>();
+		for (ColorSchemeAssociationKind associationKind : ColorSchemeAssociationKind
+				.values()) {
+			this.colorSchemeMap.put(associationKind,
+					new EnumMap<ComponentState, SubstanceColorScheme>(
+							ComponentState.class));
+		}
 	}
 
 	/**
 	 * Registers a color scheme for the specific component state.
 	 * 
 	 * @param stateColorScheme
-	 * 		Color scheme for the specified component state.
+	 *            Color scheme for the specified component state.
 	 * @param alpha
-	 * 		Alpha channel for the color scheme.
+	 *            Alpha channel for the color scheme.
 	 * @param states
-	 * 		Component states.
+	 *            Component states.
 	 */
 	public void registerColorScheme(SubstanceColorScheme stateColorScheme,
 			float alpha, ComponentState... states) {
 		if (states != null) {
 			for (ComponentState state : states) {
-				this.stateColorSchemeMap.put(state, stateColorScheme);
+				this.colorSchemeMap.get(ColorSchemeAssociationKind.FILL).put(
+						state, stateColorScheme);
 				this.stateAlphaMap.put(state, alpha);
 			}
 		}
@@ -163,9 +163,9 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 * Registers a color scheme for the specific component state.
 	 * 
 	 * @param stateColorScheme
-	 * 		Color scheme for the specified component state.
+	 *            Color scheme for the specified component state.
 	 * @param states
-	 * 		Component states.
+	 *            Component states.
 	 */
 	public void registerColorScheme(SubstanceColorScheme stateColorScheme,
 			ComponentState... states) {
@@ -178,28 +178,32 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 * scheme otherwise.
 	 * 
 	 * @param stateHighlightScheme
-	 * 		Highlight color scheme for the specified component state.
+	 *            Highlight color scheme for the specified component state.
 	 * @param states
-	 * 		Component states. If <code>null</code>, the specified color scheme
-	 * 		will be applied for all states left unspecified.
+	 *            Component states. If <code>null</code>, the specified color
+	 *            scheme will be applied for all states left unspecified.
 	 */
 	public void registerHighlightColorScheme(
 			SubstanceColorScheme stateHighlightScheme, ComponentState... states) {
 		if ((states == null) || (states.length == 0)) {
 			for (ComponentState state : ComponentState.values()) {
-				if (this.stateHighlightColorSchemeMap.containsKey(state))
+				if (this.colorSchemeMap.get(
+						ColorSchemeAssociationKind.HIGHLIGHT)
+						.containsKey(state))
 					continue;
 				if (!state.isKindActive(FadeKind.ENABLE))
 					continue;
 				if (state == ComponentState.DEFAULT)
 					continue;
-				this.stateHighlightColorSchemeMap.put(state,
-						stateHighlightScheme);
+				// this.stateHighlightColorSchemeMap.put(state,
+				// stateHighlightScheme);
+				this.colorSchemeMap.get(ColorSchemeAssociationKind.HIGHLIGHT)
+						.put(state, stateHighlightScheme);
 			}
 		} else {
 			for (ComponentState state : states) {
-				this.stateHighlightColorSchemeMap.put(state,
-						stateHighlightScheme);
+				this.colorSchemeMap.get(ColorSchemeAssociationKind.HIGHLIGHT)
+						.put(state, stateHighlightScheme);
 			}
 		}
 	}
@@ -210,12 +214,12 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 * scheme otherwise.
 	 * 
 	 * @param stateHighlightScheme
-	 * 		Highlight color scheme for the specified component state.
+	 *            Highlight color scheme for the specified component state.
 	 * @param alpha
-	 * 		Alpha channel for the highlight color scheme.
+	 *            Alpha channel for the highlight color scheme.
 	 * @param states
-	 * 		Component states. If <code>null</code>, the specified color scheme
-	 * 		will be applied for all states left unspecified.
+	 *            Component states. If <code>null</code>, the specified color
+	 *            scheme will be applied for all states left unspecified.
 	 */
 	public void registerHighlightColorScheme(
 			SubstanceColorScheme stateHighlightScheme, float alpha,
@@ -226,16 +230,19 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 					continue;
 				if (state == ComponentState.DEFAULT)
 					continue;
-				if (!this.stateHighlightColorSchemeMap.containsKey(state))
-					this.stateHighlightColorSchemeMap.put(state,
+				if (!this.colorSchemeMap.get(
+						ColorSchemeAssociationKind.HIGHLIGHT)
+						.containsKey(state))
+					this.colorSchemeMap.get(
+							ColorSchemeAssociationKind.HIGHLIGHT).put(state,
 							stateHighlightScheme);
 				if (!this.stateHighlightSchemeAlphaMap.containsKey(state))
 					this.stateHighlightSchemeAlphaMap.put(state, alpha);
 			}
 		} else {
 			for (ComponentState state : states) {
-				this.stateHighlightColorSchemeMap.put(state,
-						stateHighlightScheme);
+				this.colorSchemeMap.get(ColorSchemeAssociationKind.HIGHLIGHT)
+						.put(state, stateHighlightScheme);
 				this.stateHighlightSchemeAlphaMap.put(state, alpha);
 			}
 		}
@@ -246,80 +253,74 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 * component state.
 	 * 
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param componentState
-	 * 		Component state.
-	 * @param toIgnoreHighlights
-	 * 		If <code>true</code>, the highlight scheme map will not be
-	 * 		consulted.
+	 *            Component state.
 	 * @return The color scheme of the component in the specified component
-	 * 	state.
+	 *         state.
 	 */
 	public SubstanceColorScheme getColorScheme(Component comp,
-			ComponentState componentState, boolean toIgnoreHighlights) {
-		if (!toIgnoreHighlights
-				&& SubstanceCoreUtilities.toUseHighlightColorScheme(comp)) {
-			SubstanceColorScheme registered = this.stateHighlightColorSchemeMap
-					.get(componentState);
-			if (registered != null)
-				return registered;
-		} else {
-			SubstanceColorScheme registered = this.stateColorSchemeMap
-					.get(componentState);
-			if (registered != null)
-				return registered;
+			ComponentState componentState) {
+		SubstanceColorScheme registered = this.colorSchemeMap.get(
+				ColorSchemeAssociationKind.FILL).get(componentState);
+		if (registered != null)
+			return registered;
 
-			if (componentState.isKindActive(FadeKind.PRESS)) {
-				if (this.pressedScheme == null) {
-					this.pressedScheme = this.activeColorScheme.shade(0.2)
-							.saturate(0.1);
-				}
-				return this.pressedScheme;
+		if (componentState.isKindActive(FadeKind.PRESS)) {
+			if (this.pressedScheme == null) {
+				this.pressedScheme = this.activeColorScheme.shade(0.2)
+						.saturate(0.1);
 			}
-			if (componentState == ComponentState.DISABLED_SELECTED) {
-				if (this.disabledSelectedScheme == null) {
-					this.disabledSelectedScheme = new BlendBiColorScheme(
-							this.activeColorScheme, this.disabledColorScheme,
-							0.25);
-				}
-				return this.disabledSelectedScheme;
-			}
-
-			if (componentState == ComponentState.DEFAULT)
-				return this.defaultColorScheme;
-			if (!componentState.isKindActive(FadeKind.ENABLE))
-				return this.disabledColorScheme;
-			return this.activeColorScheme;
+			return this.pressedScheme;
 		}
-		return null;
+		if (componentState == ComponentState.DISABLED_SELECTED) {
+			if (this.disabledSelectedScheme == null) {
+				this.disabledSelectedScheme = new BlendBiColorScheme(
+						this.activeColorScheme, this.disabledColorScheme, 0.25);
+			}
+			return this.disabledSelectedScheme;
+		}
+
+		if (componentState == ComponentState.DEFAULT)
+			return this.defaultColorScheme;
+		if (!componentState.isKindActive(FadeKind.ENABLE))
+			return this.disabledColorScheme;
+		return this.activeColorScheme;
 	}
 
 	/**
 	 * Returns the highlight color scheme of the component.
 	 * 
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param componentState
-	 * 		Component state.
+	 *            Component state.
 	 * @return Component highlight color scheme.
+	 * @deprecated Use
+	 *             {@link #getColorScheme(Component, ColorSchemeAssociationKind, ComponentState)}
+	 *             with {@link ColorSchemeAssociationKind#HIGHLIGHT} instead.
+	 *             This method will be removed in version 6.0.
 	 */
+	@Deprecated
 	public SubstanceColorScheme getHighlightColorScheme(Component comp,
 			ComponentState componentState) {
-		SubstanceColorScheme registered = this.stateHighlightColorSchemeMap
-				.get(componentState);
-		if (registered != null)
-			return registered;
-
-		return null;
+		return this.getColorScheme(comp, ColorSchemeAssociationKind.HIGHLIGHT,
+				componentState);
+		// SubstanceColorScheme registered = this.colorSchemeMap.get(
+		// ColorSchemeAssociationKind.HIGHLIGHT).get(componentState);
+		// if (registered != null)
+		// return registered;
+		//
+		// return null;
 	}
 
 	/**
 	 * Returns the alpha channel of the highlight color scheme of the component.
 	 * 
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param componentState
-	 * 		Component state.
+	 *            Component state.
 	 * @return Highlight color scheme alpha channel.
 	 */
 	public float getHighlightAlpha(Component comp, ComponentState componentState) {
@@ -335,9 +336,9 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 * Returns the alpha channel of the color scheme of the component.
 	 * 
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param componentState
-	 * 		Component state.
+	 *            Component state.
 	 * @return Color scheme alpha channel.
 	 */
 	public float getAlpha(Component comp, ComponentState componentState) {
@@ -376,74 +377,155 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	}
 
 	/**
-	 * Returns the color scheme for painting component border in the specified
-	 * state.
+	 * Registers the color scheme to be used for the specified visual area of
+	 * controls under the specified states. For example, if the light orange
+	 * scheme has to be used for gradient fill of rollover selected and rollover
+	 * controls, the parameters would be:
+	 * 
+	 * <ul>
+	 * <li><code>scheme</code>=light orange scheme</li>
+	 * <li>
+	 * <code>associationKind</code>={@link ColorSchemeAssociationKind#FILL}</li>
+	 * <li>
+	 * <code>states</code>={@link ComponentState#ROLLOVER_SELECTED}, {@link ComponentState#ROLLOVER_UNSELECTED}
+	 * </li>
+	 * </ul>
+	 * 
+	 * @param scheme
+	 *            Color scheme.
+	 * @param associationKind
+	 *            Color scheme association kind that specifies the visual areas
+	 *            of controls to be painted with this color scheme.
+	 * @param states
+	 *            Component states that further restrict the usage of the
+	 *            specified color scheme.
+	 * @since version 5.1
+	 */
+	public void registerColorScheme(SubstanceColorScheme scheme,
+			ColorSchemeAssociationKind associationKind,
+			ComponentState... states) {
+
+		if ((states == null) || (states.length == 0)) {
+			for (ComponentState state : ComponentState.values()) {
+				if (this.colorSchemeMap.get(associationKind).containsKey(state))
+					continue;
+				this.colorSchemeMap.get(associationKind).put(state, scheme);
+			}
+		} else {
+			for (ComponentState state : states) {
+				this.colorSchemeMap.get(associationKind).put(state, scheme);
+			}
+		}
+	}
+
+	/**
+	 * Returns the color scheme to be used for painting the specified visual
+	 * area of the component under the specified component state.
 	 * 
 	 * @param comp
-	 * 		Component.
+	 *            Component.
+	 * @param associationKind
+	 *            Color scheme association kind.
 	 * @param componentState
-	 * 		Component state.
-	 * @return Color scheme for painting component border in the specified
-	 * 	state.
+	 *            Component state.
+	 * @return Color scheme to be used for painting the specified visual area of
+	 *         the component under the specified component state.
+	 * @see #registerColorScheme(SubstanceColorScheme, ComponentState...)
+	 * @since version 5.1
 	 */
-	public SubstanceColorScheme getBorderColorScheme(Component comp,
+	public SubstanceColorScheme getColorScheme(Component comp,
+			ColorSchemeAssociationKind associationKind,
 			ComponentState componentState) {
-		SubstanceColorScheme registered = this.stateBorderColorSchemeMap
-				.get(componentState);
+		if (associationKind == ColorSchemeAssociationKind.FILL)
+			return this.getColorScheme(comp, componentState);
+
+		SubstanceColorScheme registered = this.colorSchemeMap.get(
+				associationKind).get(componentState);
 		if (registered != null)
 			return registered;
 
-		return this.getColorScheme(comp, componentState, true);
+		ColorSchemeAssociationKind fallback = associationKind.getFallback();
+		if (fallback == null)
+			return null;
+
+		return getColorScheme(comp, fallback, componentState);
 	}
 
 	/**
 	 * Registers a border color scheme for the specific component state if the
-	 * component state is not <code>null</code>, or a global border color scheme
-	 * otherwise.
+	 * component state is not <code>null</code>, or a global highlight color
+	 * scheme otherwise.
 	 * 
-	 * @param stateBorderScheme
-	 * 		Border color scheme for the specified component states.
+	 * @param scheme
+	 *            Border color scheme for the specified component state.
 	 * @param states
-	 * 		Component states. If <code>null</code>, the specified color scheme
-	 * 		will be applied for all states left unspecified.
+	 *            Component states. If <code>null</code>, the specified color
+	 *            scheme will be applied for all states left unspecified.
+	 * @deprecated Use
+	 *             {@link #registerColorScheme(SubstanceColorScheme, ColorSchemeAssociationKind, ComponentState...)}
+	 *             with {@link ColorSchemeAssociationKind#BORDER} instead. This
+	 *             method will be removed in version 6.0.
 	 */
-	public void registerBorderColorScheme(
-			SubstanceColorScheme stateBorderScheme, ComponentState... states) {
+	@Deprecated
+	public void registerBorderColorScheme(SubstanceColorScheme scheme,
+			ComponentState... states) {
 		if ((states == null) || (states.length == 0)) {
 			for (ComponentState state : ComponentState.values()) {
-				if (this.stateBorderColorSchemeMap.containsKey(state))
+				if (this.colorSchemeMap.get(ColorSchemeAssociationKind.BORDER)
+						.containsKey(state))
 					continue;
-				this.stateBorderColorSchemeMap.put(state, stateBorderScheme);
+				this.colorSchemeMap.get(ColorSchemeAssociationKind.BORDER).put(
+						state, scheme);
 			}
 		} else {
 			for (ComponentState state : states) {
-				this.stateBorderColorSchemeMap.put(state, stateBorderScheme);
+				this.colorSchemeMap.get(ColorSchemeAssociationKind.BORDER).put(
+						state, scheme);
 			}
 		}
 	}
 
+	/**
+	 * Returns the border color scheme of the component.
+	 * 
+	 * @param comp
+	 *            Component.
+	 * @param componentState
+	 *            Component state.
+	 * @return Component border color scheme.
+	 * @deprecated Use
+	 *             {@link #getColorScheme(Component, ColorSchemeAssociationKind, ComponentState)}
+	 *             with {@link ColorSchemeAssociationKind#BORDER} instead. This
+	 *             method will be removed in version 6.0.
+	 */
+	@Deprecated
+	public SubstanceColorScheme getBorderColorScheme(Component comp,
+			ComponentState componentState) {
+		return this.getColorScheme(comp, ColorSchemeAssociationKind.BORDER,
+				componentState);
+		// SubstanceColorScheme registered = this.colorSchemeMap.get(
+		// ColorSchemeAssociationKind.BORDER).get(componentState);
+		// if (registered != null)
+		// return registered;
+		//
+		// return this.getColorScheme(comp, componentState, true);
+	}
+
 	@Override
 	protected Object clone() throws CloneNotSupportedException {
 		SubstanceColorSchemeBundle clone = new SubstanceColorSchemeBundle(
 				this.activeColorScheme, this.defaultColorScheme,
 				this.disabledColorScheme);
 
-		if (this.stateColorSchemeMap != null) {
-			clone.stateColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-					this.stateColorSchemeMap);
+		for (Map.Entry<ColorSchemeAssociationKind, Map<ComponentState, SubstanceColorScheme>> entry : this.colorSchemeMap
+				.entrySet()) {
+			clone.colorSchemeMap.get(entry.getKey()).putAll(entry.getValue());
 		}
+
 		if (this.stateAlphaMap != null) {
 			clone.stateAlphaMap = new EnumMap<ComponentState, Float>(
 					this.stateAlphaMap);
 		}
-		if (this.stateBorderColorSchemeMap != null) {
-			clone.stateBorderColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-					this.stateBorderColorSchemeMap);
-		}
-		if (this.stateHighlightColorSchemeMap != null) {
-			clone.stateHighlightColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-					this.stateHighlightColorSchemeMap);
-		}
 		if (this.stateHighlightSchemeAlphaMap != null) {
 			clone.stateHighlightSchemeAlphaMap = new EnumMap<ComponentState, Float>(
 					this.stateHighlightSchemeAlphaMap);
@@ -457,7 +539,7 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 	 * scheme transformation on all the relevant color schemes
 	 * 
 	 * @param transform
-	 * 		Color scheme transformation.
+	 *            Color scheme transformation.
 	 * @return The new color scheme bundle.
 	 */
 	public SubstanceColorSchemeBundle transform(ColorSchemeTransform transform) {
@@ -467,14 +549,13 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 						.transform(this.defaultColorScheme), transform
 						.transform(this.disabledColorScheme));
 
-		// transform the state-specific schemes
-		if (this.stateColorSchemeMap != null) {
-			result.stateColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-					ComponentState.class);
-			for (Map.Entry<ComponentState, SubstanceColorScheme> entry : this.stateColorSchemeMap
-					.entrySet()) {
-				result.stateColorSchemeMap.put(entry.getKey(), transform
-						.transform(entry.getValue()));
+		for (Map.Entry<ColorSchemeAssociationKind, Map<ComponentState, SubstanceColorScheme>> entry : this.colorSchemeMap
+				.entrySet()) {
+			for (Map.Entry<ComponentState, SubstanceColorScheme> subEntry : entry
+					.getValue().entrySet()) {
+				result.colorSchemeMap.get(entry.getKey()).put(
+						subEntry.getKey(),
+						transform.transform(subEntry.getValue()));
 			}
 		}
 
@@ -484,28 +565,6 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 					this.stateAlphaMap);
 		}
 
-		// transform the state-specific border schemes
-		if (this.stateBorderColorSchemeMap != null) {
-			result.stateBorderColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-					ComponentState.class);
-			for (Map.Entry<ComponentState, SubstanceColorScheme> entry : this.stateBorderColorSchemeMap
-					.entrySet()) {
-				result.stateBorderColorSchemeMap.put(entry.getKey(), transform
-						.transform(entry.getValue()));
-			}
-		}
-
-		// transform the state-specific highlight schemes
-		if (this.stateHighlightColorSchemeMap != null) {
-			result.stateHighlightColorSchemeMap = new EnumMap<ComponentState, SubstanceColorScheme>(
-					ComponentState.class);
-			for (Map.Entry<ComponentState, SubstanceColorScheme> entry : this.stateHighlightColorSchemeMap
-					.entrySet()) {
-				result.stateHighlightColorSchemeMap.put(entry.getKey(),
-						transform.transform(entry.getValue()));
-			}
-		}
-
 		// highlight alphas are the same
 		if (this.stateHighlightSchemeAlphaMap != null) {
 			result.stateHighlightSchemeAlphaMap = new EnumMap<ComponentState, Float>(
@@ -513,4 +572,23 @@ public class SubstanceColorSchemeBundle implements Cloneable {
 		}
 		return result;
 	}
+
+	/**
+	 * Returns the set of all component states that have non-trivial alpha
+	 * associated with them. Non-trivial alpha is a value that is strictly less
+	 * than 1.0.
+	 * 
+	 * @return All component states that have associated non-trivial alpha
+	 *         values.
+	 */
+	Set<ComponentState> getStatesWithAlpha() {
+		Set<ComponentState> result = EnumSet.noneOf(ComponentState.class);
+		for (Map.Entry<ComponentState, Float> alphaEntry : this.stateAlphaMap
+				.entrySet()) {
+			if (alphaEntry.getValue() < 1.0f) {
+				result.add(alphaEntry.getKey());
+			}
+		}
+		return result;
+	}
 }
