@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -31,6 +31,7 @@ package org.jvnet.substance.api;
 
 import java.awt.*;
 import java.awt.event.KeyEvent;
+import java.net.URL;
 import java.util.*;
 
 import javax.swing.*;
@@ -40,6 +41,7 @@ import javax.swing.plaf.basic.BasicBorders;
 import javax.swing.plaf.basic.BasicBorders.MarginBorder;
 import javax.swing.table.JTableHeader;
 import javax.swing.text.DefaultEditorKit;
+import javax.swing.text.JTextComponent;
 
 import org.jvnet.lafwidget.animation.FadeKind;
 import org.jvnet.substance.SubstanceLookAndFeel;
@@ -99,6 +101,12 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 	 */
 	protected SubstanceBorderPainter borderPainter;
 
+	/**
+	 * The highlight border painter of <code>this</code> skin. Can be
+	 * <code>null</code>.
+	 */
+	protected SubstanceBorderPainter highlightBorderPainter;
+
 	/**
 	 * The highlight painter of <code>this</code> skin. Must be non-
 	 * <code>null</code>.
@@ -146,6 +154,16 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 	 */
 	protected SubstanceColorScheme watermarkScheme;
 
+	/**
+	 * All component states that have associated non-trivial alpha values.
+	 */
+	Set<ComponentState> statesWithAlpha;
+
+	// /**
+	// * The default color scheme of this skin.
+	// */
+	// SubstanceColorScheme defaultColorScheme;
+
 	/**
 	 * Constructs the basic data structures for a skin.
 	 */
@@ -163,6 +181,8 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 
 		this.selectedTabFadeStart = 0.1;
 		this.selectedTabFadeEnd = 0.3;
+
+		this.statesWithAlpha = EnumSet.noneOf(ComponentState.class);
 	}
 
 	/**
@@ -236,6 +256,17 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 		return this.borderPainter;
 	}
 
+	/**
+	 * Returns the highlight border painter of this skin.
+	 * 
+	 * @return The highlight border painter of this skin. The return value of
+	 *         this method may be <code>null</code>. In this case, call
+	 *         {@link #getBorderPainter()}.
+	 */
+	public SubstanceBorderPainter getHighlightBorderPainter() {
+		return this.highlightBorderPainter;
+	}
+
 	/**
 	 * Returns the button shaper of this skin.
 	 * 
@@ -330,8 +361,20 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				.getBackgroundFillColor());
 		Color backgroundLightColor = new ColorUIResource(mainActiveScheme
 				.getLightBackgroundFillColor());
+
 		Color disabledForegroundColor = SubstanceColorUtilities
 				.getForegroundColor(mainDisabledScheme);
+		Color disabledTextComponentForegroundColor = disabledForegroundColor;
+		float alpha = this.getAlpha(null, ComponentState.DISABLED_UNSELECTED);
+		if (alpha < 1.0f) {
+			ColorUIResource defaultTextBackgroundColor = SubstanceColorUtilities
+					.getDefaultBackgroundColor(JTextComponent.class, this,
+							false);
+			disabledTextComponentForegroundColor = new ColorUIResource(
+					SubstanceColorUtilities.getInterpolatedColor(
+							disabledTextComponentForegroundColor,
+							defaultTextBackgroundColor, alpha));
+		}
 
 		Color lineColor = new ColorUIResource(mainActiveScheme.getLineColor());
 
@@ -342,20 +385,29 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				.getColorBrightness(lineColor.getRGB());
 		Color lineBwColor = new ColorUIResource(new Color(lcb, lcb, lcb));
 
-		Color selectionBackgroundColor = new ColorUIResource(this
-				.getHighlightColorScheme(null, ComponentState.ACTIVE)
-				.getSelectionBackgroundColor());
+		SubstanceColorScheme textHighlightColorScheme = this.getColorScheme(
+				null, ColorSchemeAssociationKind.TEXT_HIGHLIGHT,
+				ComponentState.SELECTED);
+		if (textHighlightColorScheme == null) {
+			textHighlightColorScheme = this.getColorScheme(null,
+					ComponentState.ROLLOVER_SELECTED);
+		}
+		Color selectionTextBackgroundColor = new ColorUIResource(
+				textHighlightColorScheme.getSelectionBackgroundColor());
+		Color selectionTextForegroundColor = new ColorUIResource(
+				textHighlightColorScheme.getSelectionForegroundColor());
 
-		Color selectionForegroundColor = new ColorUIResource(mainActiveScheme
-				.getSelectionForegroundColor());
-
-		Color selectionCellForegroundColor = new ColorUIResource(this
-				.getHighlightColorScheme(null, ComponentState.SELECTED)
-				.getForegroundColor());
-
-		Color selectionCellBackgroundColor = new ColorUIResource(this
-				.getHighlightColorScheme(null, ComponentState.SELECTED)
-				.getBackgroundFillColor());
+		// SubstanceColorScheme textHighlightColorScheme =
+		// this.getColorScheme(null,
+		// ColorSchemeAssociationKind.HIGHLIGHT, ComponentState.SELECTED);
+		// if (textHighlightColorScheme == null) {
+		// textHighlightColorScheme = this.getColorScheme(null,
+		// ComponentState.ROLLOVER_SELECTED);
+		// }
+		Color selectionCellForegroundColor = new ColorUIResource(
+				textHighlightColorScheme.getForegroundColor());
+		Color selectionCellBackgroundColor = new ColorUIResource(
+				textHighlightColorScheme.getBackgroundFillColor());
 
 		Object regularMarginBorder = new UIDefaults.LazyValue() {
 			@Override
@@ -376,6 +428,13 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 			}
 		};
 
+		Object textMarginBorder = new UIDefaults.LazyValue() {
+			@Override
+			public Object createValue(UIDefaults table) {
+				return new BasicBorders.MarginBorder();
+			}
+		};
+
 		Object tooltipBorder = new UIDefaults.LazyValue() {
 			@Override
 			public Object createValue(UIDefaults table) {
@@ -562,14 +621,9 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 
 				"ENTER", JTextField.notifyAction,
 
-				"ctrl BACK_SLASH",
-				"unselect"/* DefaultEditorKit.unselectAction */,
+				"ctrl BACK_SLASH", "unselect",
 
-				"control shift O", "toggle-componentOrientation"/*
-																 * DefaultEditorKit.
-																 * toggleComponentOrientation
-																 */
-		});
+				"control shift O", "toggle-componentOrientation" });
 
 		Object multilineInputMap = new UIDefaults.LazyInputMap(new Object[] {
 				"ctrl C", DefaultEditorKit.copyAction,
@@ -667,8 +721,7 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 
 				"TAB", DefaultEditorKit.insertTabAction,
 
-				"ctrl BACK_SLASH",
-				"unselect"/* DefaultEditorKit.unselectAction */,
+				"ctrl BACK_SLASH", "unselect",
 
 				"ctrl HOME", DefaultEditorKit.beginAction,
 
@@ -684,18 +737,18 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 
 				"ctrl SPACE", "activate-link-action",
 
-				"control shift O", "toggle-componentOrientation"/*
-																 * DefaultEditorKit.
-																 * toggleComponentOrientation
-																 */
-		});
+				"control shift O", "toggle-componentOrientation" });
 
 		Object emptyIcon = new UIDefaults.LazyValue() {
 			@Override
 			public Object createValue(UIDefaults table) {
 				return new IconUIResource(new Icon() {
 					public int getIconHeight() {
-						return 2;
+						// return the value that matches the core height, so
+						// that the DefaultTreeCellEditor.EditorContainer
+						// returns the correct value in its getPreferredSize
+						// when it consults the "editingIcon" height.
+						return 16;
 					}
 
 					public int getIconWidth() {
@@ -727,9 +780,6 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				"EditorPane.focusInputMap",
 				multilineInputMap,
 
-				// "MenuBarUI",
-				// "org.jvnet.substance.SubstanceMenuBarUI",
-
 				"Button.defaultButtonFollowsFocus",
 				Boolean.FALSE,
 
@@ -774,13 +824,6 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				new CheckBoxMenuItemIcon(null, 1 + SubstanceSizeUtils
 						.getMenuCheckMarkSize(SubstanceSizeUtils
 								.getControlFontSize())),
-				// new UIDefaults.LazyValue() {
-				// public Object createValue(UIDefaults table) {
-				// return new IconUIResource(SubstanceIconFactory
-				// .getCheckBoxMenuItemIcon(1 + SubstanceSizeUtils
-				// .getMenuCheckMarkSize()));
-				// }
-				// },
 
 				"CheckBoxMenuItem.disabledForeground",
 				disabledForegroundColor,
@@ -850,10 +893,10 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				foregroundColor,
 
 				"ComboBox.selectionBackground",
-				backgroundLightColor,
+				selectionCellBackgroundColor,
 
 				"ComboBox.selectionForeground",
-				foregroundColor,
+				selectionCellForegroundColor,
 
 				"DesktopIcon.border",
 				regularMarginBorder,
@@ -876,7 +919,7 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JEditorPane.class, this, false),
 
 				"EditorPane.border",
-				textBorder,
+				textMarginBorder,
 
 				"EditorPane.foreground",
 				foregroundColor,
@@ -893,13 +936,13 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JEditorPane.class, this, true),
 
 				"EditorPane.inactiveForeground",
-				disabledForegroundColor,
+				disabledTextComponentForegroundColor,
 
 				"EditorPane.selectionBackground",
-				selectionBackgroundColor,
+				selectionTextBackgroundColor,
 
 				"EditorPane.selectionForeground",
-				selectionForegroundColor,
+				selectionTextForegroundColor,
 
 				"FileChooser.upFolderIcon",
 				new UIDefaults.LazyValue() {
@@ -1036,68 +1079,60 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JFormattedTextField.class, this, true),
 
 				"FormattedTextField.inactiveForeground",
-				disabledForegroundColor,
+				disabledTextComponentForegroundColor,
 
 				"FormattedTextField.selectionBackground",
-				selectionBackgroundColor,
+				selectionTextBackgroundColor,
 
 				"FormattedTextField.selectionForeground",
-				selectionForegroundColor,
+				selectionTextForegroundColor,
 
 				"InternalFrame.activeTitleBackground",
-				selectionForegroundColor,
+				selectionTextForegroundColor,
 
 				"InternalFrame.inactiveTitleBackground",
 				foregroundColor,
 
-				// "InternalFrame.activeTitleGradient",
-				// selectionBackgroundColor,
-				//
-				// "InternalFrame.inactiveTitleGradient",
-				// backgroundColor,
-
 				"InternalFrame.border",
 				new BorderUIResource(new SubstancePaneBorder()),
 
 				"InternalFrame.closeIcon",
 				new UIDefaults.LazyValue() {
 					public Object createValue(UIDefaults table) {
-						return SubstanceImageCreator
-								.getCloseIcon(titlePaneScheme);
+						return SubstanceImageCreator.getCloseIcon(
+								titlePaneScheme, titlePaneScheme);
 					}
 				},
 
 				"InternalFrame.iconifyIcon",
 				new UIDefaults.LazyValue() {
 					public Object createValue(UIDefaults table) {
-						return SubstanceImageCreator
-								.getMinimizeIcon(titlePaneScheme);
+						return SubstanceImageCreator.getMinimizeIcon(
+								titlePaneScheme, titlePaneScheme);
 					}
 				},
 
-				// maximize2
 				"InternalFrame.maximizeIcon",
 				new UIDefaults.LazyValue() {
 					public Object createValue(UIDefaults table) {
-						return SubstanceImageCreator
-								.getMaximizeIcon(titlePaneScheme);
+						return SubstanceImageCreator.getMaximizeIcon(
+								titlePaneScheme, titlePaneScheme);
 					}
 				},
 
-				// restore
 				"InternalFrame.minimizeIcon",
 				new UIDefaults.LazyValue() {
 					public Object createValue(UIDefaults table) {
-						return SubstanceImageCreator
-								.getRestoreIcon(titlePaneScheme);
+						return SubstanceImageCreator.getRestoreIcon(
+								titlePaneScheme, titlePaneScheme);
 					}
 				},
 
 				"InternalFrame.paletteCloseIcon",
 				new UIDefaults.LazyValue() {
 					public Object createValue(UIDefaults table) {
-						return SubstanceImageCreator
-								.getCloseIcon(titlePaneScheme);
+						return SubstanceImageCreator.getCloseIcon(
+								titlePaneScheme, titlePaneScheme);
 					}
 				},
 
@@ -1126,7 +1161,6 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 
 				"List.focusSelectedCellHighlightBorder",
 				new BorderUIResource.EmptyBorderUIResource(1, 1, 1, 1),
-				// new SubstanceBorder(new Insets(1, 1, 1, 1)),
 
 				"List.foreground",
 				foregroundColor,
@@ -1277,13 +1311,13 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JPasswordField.class, this, true),
 
 				"PasswordField.inactiveForeground",
-				disabledForegroundColor,
+				disabledTextComponentForegroundColor,
 
 				"PasswordField.selectionBackground",
-				selectionBackgroundColor,
+				selectionTextBackgroundColor,
 
 				"PasswordField.selectionForeground",
-				selectionForegroundColor,
+				selectionTextForegroundColor,
 
 				"PopupMenu.background",
 				SubstanceColorUtilities.getDefaultBackgroundColor(
@@ -1423,28 +1457,12 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				"Slider.highlight",
 				backgroundLightColor,
 
-				// "Slider.horizontalThumbIcon",
-				// new UIDefaults.LazyValue() {
-				// public Object createValue(UIDefaults table) {
-				// return SubstanceIconFactory.getSliderHorizontalIcon(16,
-				// false);
-				// }
-				// },
-				//
 				"Slider.shadow",
 				lineColor,
 
 				"Slider.tickColor",
 				foregroundColor,
 
-				// "Slider.verticalThumbIcon",
-				// new UIDefaults.LazyValue() {
-				// public Object createValue(UIDefaults table) {
-				// return SubstanceIconFactory.getSliderVerticalIcon(16,
-				// false);
-				// }
-				// },
-
 				"Spinner.arrowButtonInsets",
 				SubstanceSizeUtils
 						.getSpinnerArrowButtonInsets(SubstanceSizeUtils
@@ -1466,9 +1484,6 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				"Spinner.editorBorderPainted",
 				Boolean.TRUE,
 
-				// "Spinner.arrowButtonInsets",
-				// SubstanceSizeUtils.getSpinnerArrowButtonInsets(),
-
 				"SplitPane.background",
 				SubstanceColorUtilities.getDefaultBackgroundColor(
 						JSplitPane.class, this, false),
@@ -1509,7 +1524,6 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 
 				"TabbedPane.contentAreaColor",
 				null,
-				// new ColorUIResource(mainActiveScheme.getUltraLightColor()),
 
 				"TabbedPane.contentBorderInsets",
 				new InsetsUIResource(4, 4, 4, 4),
@@ -1518,7 +1532,8 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 				Boolean.FALSE,
 
 				"TabbedPane.darkShadow",
-				new ColorUIResource(getBorderColorScheme(null,
+				new ColorUIResource(getColorScheme(null,
+						ColorSchemeAssociationKind.BORDER,
 						ComponentState.SELECTED).getLineColor()),
 
 				"TabbedPane.focus",
@@ -1566,11 +1581,6 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 
 				"Table.focusCellHighlightBorder",
 				new SubstanceBorder(),
-				// new UIDefaults.LazyValue() {
-				// public Object createValue(UIDefaults table) {
-				// return new BorderUIResource(new SubstanceBorder());
-				// }
-				// },
 
 				"Table.foreground",
 				foregroundColor,
@@ -1602,7 +1612,7 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JTextArea.class, this, false),
 
 				"TextArea.border",
-				textBorder,
+				textMarginBorder,
 
 				"TextArea.caretForeground",
 				foregroundColor,
@@ -1619,13 +1629,13 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JTextArea.class, this, true),
 
 				"TextArea.inactiveForeground",
-				disabledForegroundColor,
+				disabledTextComponentForegroundColor,
 
 				"TextArea.selectionBackground",
-				selectionBackgroundColor,
+				selectionTextBackgroundColor,
 
 				"TextArea.selectionForeground",
-				selectionForegroundColor,
+				selectionTextForegroundColor,
 
 				"TextField.background",
 				SubstanceColorUtilities.getDefaultBackgroundColor(
@@ -1649,20 +1659,20 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JTextField.class, this, true),
 
 				"TextField.inactiveForeground",
-				disabledForegroundColor,
+				disabledTextComponentForegroundColor,
 
 				"TextField.selectionBackground",
-				selectionBackgroundColor,
+				selectionTextBackgroundColor,
 
 				"TextField.selectionForeground",
-				selectionForegroundColor,
+				selectionTextForegroundColor,
 
 				"TextPane.background",
 				SubstanceColorUtilities.getDefaultBackgroundColor(
 						JTextPane.class, this, false),
 
 				"TextPane.border",
-				textBorder,
+				textMarginBorder,
 
 				"TextPane.disabledBackground",
 				SubstanceColorUtilities.getDefaultBackgroundColor(
@@ -1679,13 +1689,13 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 						JTextPane.class, this, true),
 
 				"TextPane.inactiveForeground",
-				disabledForegroundColor,
+				disabledTextComponentForegroundColor,
 
 				"TextPane.selectionBackground",
-				selectionBackgroundColor,
+				selectionTextBackgroundColor,
 
 				"TextPane.selectionForeground",
-				selectionForegroundColor,
+				selectionTextForegroundColor,
 
 				"TitledBorder.titleColor",
 				foregroundColor,
@@ -1811,42 +1821,29 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 	 */
 	public SubstanceColorScheme getColorScheme(Component comp,
 			ComponentState componentState) {
-		return this.getColorScheme(comp, componentState, false);
-	}
-
-	/**
-	 * Returns the color scheme of the specified component in the specified
-	 * component state.
-	 * 
-	 * @param comp
-	 *            Component.
-	 * @param componentState
-	 *            Component state.
-	 * @param toIgnoreHighlights
-	 *            If <code>true</code>, the highlight scheme map will not be
-	 *            consulted.
-	 * @return The color scheme of the component in the specified component
-	 *         state.
-	 */
-	public SubstanceColorScheme getColorScheme(Component comp,
-			ComponentState componentState, boolean toIgnoreHighlights) {
 		// small optimization - lookup the decoration area only if there
 		// are decoration-specific scheme bundles.
 		if (this.colorSchemeBundleMap.size() > 1) {
 			DecorationAreaType decorationAreaType = SubstanceDecorationUtilities
 					.getDecorationType(comp);
+			// if ((decorationAreaType == DecorationAreaType.NONE)
+			// && (componentState == ComponentState.DEFAULT)) {
+			// return this.defaultColorScheme;
+			// }
 			if (this.colorSchemeBundleMap.containsKey(decorationAreaType)) {
 				SubstanceColorScheme registered = this.colorSchemeBundleMap
 						.get(decorationAreaType).getColorScheme(comp,
-								componentState, toIgnoreHighlights);
+								componentState);
 				if (registered != null)
 					return registered;
 			}
 		}
 
+		// if (componentState == ComponentState.DEFAULT)
+		// return this.defaultColorScheme;
+
 		SubstanceColorScheme registered = this.colorSchemeBundleMap.get(
-				DecorationAreaType.NONE).getColorScheme(comp, componentState,
-				toIgnoreHighlights);
+				DecorationAreaType.NONE).getColorScheme(comp, componentState);
 		if (registered != null)
 			return registered;
 
@@ -1865,7 +1862,12 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 	 * @param componentState
 	 *            Component state.
 	 * @return Component highlight color scheme.
+	 * @deprecated Use
+	 *             {@link #getColorScheme(Component, ColorSchemeAssociationKind, ComponentState)}
+	 *             with {@link ColorSchemeAssociationKind#HIGHLIGHT} instead.
+	 *             This method will be removed in version 6.0.
 	 */
+	@Deprecated
 	public SubstanceColorScheme getHighlightColorScheme(Component comp,
 			ComponentState componentState) {
 		// small optimization - lookup the decoration area only if there
@@ -1875,7 +1877,8 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 					.getDecorationType(comp);
 			if (this.colorSchemeBundleMap.containsKey(decorationAreaType)) {
 				SubstanceColorScheme registered = this.colorSchemeBundleMap
-						.get(decorationAreaType).getHighlightColorScheme(comp,
+						.get(decorationAreaType).getColorScheme(comp,
+								ColorSchemeAssociationKind.HIGHLIGHT,
 								componentState);
 				if (registered != null)
 					return registered;
@@ -1883,8 +1886,8 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 		}
 
 		SubstanceColorScheme registered = this.colorSchemeBundleMap.get(
-				DecorationAreaType.NONE).getHighlightColorScheme(comp,
-				componentState);
+				DecorationAreaType.NONE).getColorScheme(comp,
+				ColorSchemeAssociationKind.HIGHLIGHT, componentState);
 		if (registered != null)
 			return registered;
 
@@ -1952,6 +1955,11 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 	 * @return Color scheme alpha channel.
 	 */
 	public float getAlpha(Component comp, ComponentState componentState) {
+		// optimization - if the state is not registered in any
+		// scheme bundle with custom alpha, return 1.0
+		if (!this.statesWithAlpha.contains(componentState))
+			return 1.0f;
+
 		// small optimization - lookup the decoration area only if there
 		// are decoration-specific scheme bundles.
 		if (this.colorSchemeBundleMap.size() > 1) {
@@ -1997,7 +2005,12 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 			this.decoratedAreaSet.add(areaType);
 			this.colorSchemeBundleMap.put(areaType, bundle);
 			this.backgroundColorSchemeMap.put(areaType, backgroundColorScheme);
+
+			// if (areaType == DecorationAreaType.NONE) {
+			// this.defaultColorScheme = bundle.getDefaultColorScheme();
+			// }
 		}
+		this.statesWithAlpha.addAll(bundle.getStatesWithAlpha());
 	}
 
 	/**
@@ -2241,16 +2254,50 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 	}
 
 	/**
-	 * Returns the color scheme for painting component border in the specified
-	 * state.
+	 * Returns the color scheme to be used for painting the specified visual
+	 * area of the component under the specified component state.
+	 * 
+	 * @param comp
+	 *            Component.
+	 * @param associationKind
+	 *            Color scheme association kind.
+	 * @param componentState
+	 *            Component state.
+	 * @return Color scheme to be used for painting the specified visual area of
+	 *         the component under the specified component state.
+	 * @since version 5.1
+	 */
+	public SubstanceColorScheme getColorScheme(Component comp,
+			ColorSchemeAssociationKind associationKind,
+			ComponentState componentState) {
+		// small optimization - lookup the decoration area only if there
+		// are decoration-specific scheme bundles.
+		if (this.colorSchemeBundleMap.size() > 1) {
+			DecorationAreaType decorationAreaType = SubstanceDecorationUtilities
+					.getDecorationType(comp);
+			if (this.colorSchemeBundleMap.containsKey(decorationAreaType)) {
+				return this.colorSchemeBundleMap.get(decorationAreaType)
+						.getColorScheme(comp, associationKind, componentState);
+			}
+		}
+		return this.colorSchemeBundleMap.get(DecorationAreaType.NONE)
+				.getColorScheme(comp, associationKind, componentState);
+	}
+
+	/**
+	 * Returns the border color scheme of the component.
 	 * 
 	 * @param comp
 	 *            Component.
 	 * @param componentState
 	 *            Component state.
-	 * @return Color scheme for painting component border in the specified
-	 *         state.
+	 * @return Component border color scheme.
+	 * @deprecated Use
+	 *             {@link #getColorScheme(Component, ColorSchemeAssociationKind, ComponentState)}
+	 *             with {@link ColorSchemeAssociationKind#BORDER} instead. This
+	 *             method will be removed in version 6.0.
 	 */
+	@Deprecated
 	public SubstanceColorScheme getBorderColorScheme(Component comp,
 			ComponentState componentState) {
 		// small optimization - lookup the decoration area only if there
@@ -2260,11 +2307,14 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 					.getDecorationType(comp);
 			if (this.colorSchemeBundleMap.containsKey(decorationAreaType)) {
 				return this.colorSchemeBundleMap.get(decorationAreaType)
-						.getBorderColorScheme(comp, componentState);
+						.getColorScheme(comp,
+								ColorSchemeAssociationKind.BORDER,
+								componentState);
 			}
 		}
 		return this.colorSchemeBundleMap.get(DecorationAreaType.NONE)
-				.getBorderColorScheme(comp, componentState);
+				.getColorScheme(comp, ColorSchemeAssociationKind.BORDER,
+						componentState);
 	}
 
 	@Override
@@ -2280,9 +2330,11 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 		clone.buttonShaper = this.buttonShaper;
 		clone.decorationPainter = this.decorationPainter;
 		clone.gradientPainter = this.gradientPainter;
+		clone.highlightBorderPainter = this.highlightBorderPainter;
 		clone.highlightPainter = this.highlightPainter;
 		clone.watermark = this.watermark;
 		clone.watermarkScheme = this.watermarkScheme;
+		// clone.defaultColorScheme = this.defaultColorScheme;
 		clone.selectedTabFadeEnd = this.selectedTabFadeEnd;
 		clone.selectedTabFadeStart = this.selectedTabFadeStart;
 
@@ -2315,7 +2367,7 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 	/**
 	 * Creates a new skin that has the same settings as this skin with the
 	 * addition of applying the specified color scheme transformation on all the
-	 * relevant color schemes
+	 * relevant color schemes.
 	 * 
 	 * @param transform
 	 *            Color scheme transformation.
@@ -2338,10 +2390,15 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 		result.decorationPainter = this.decorationPainter;
 		result.gradientPainter = this.gradientPainter;
 		result.highlightPainter = this.highlightPainter;
+		result.highlightBorderPainter = this.highlightBorderPainter;
 		// same watermark and transformed scheme
 		result.watermark = this.watermark;
 		if (this.watermarkScheme != null)
 			result.watermarkScheme = transform.transform(this.watermarkScheme);
+		// issue 428 - transform the default color scheme
+		// result.defaultColorScheme = transform
+		// .transform(this.defaultColorScheme);
+
 		// same misc settings
 		result.selectedTabFadeEnd = this.selectedTabFadeEnd;
 		result.selectedTabFadeStart = this.selectedTabFadeStart;
@@ -2462,4 +2519,17 @@ public abstract class SubstanceSkin implements SubstanceTrait, Cloneable {
 		}
 		return true;
 	}
+
+	/**
+	 * Returns the collection of color schemes in the specified URL.
+	 * 
+	 * @param url
+	 *            URL that points to a resource containing the description of
+	 *            Substance color schemes.
+	 * @return The collection of color schemes in the specified URL.
+	 * @since version 5.2
+	 */
+	public static Map<String, SubstanceColorScheme> getColorSchemes(URL url) {
+		return SubstanceColorSchemeUtilities.getColorSchemes(url);
+	}
 }
