@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -76,9 +76,9 @@ public abstract class ImageWrapperDecorationPainter implements
 
 	/**
 	 * Indication whether this painter paints horizontal separators on the
-	 * bottom of each component passed to {@link
-	 * #paintDecorationArea(Graphics2D, Component, DecorationAreaType, int, int,
-	 * SubstanceSkin)}.
+	 * bottom of each component passed to
+	 * {@link #paintDecorationArea(Graphics2D, Component, DecorationAreaType, int, int, SubstanceSkin)}
+	 * .
 	 */
 	protected boolean isPaintingSeparators;
 
@@ -103,7 +103,7 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * @seeorg.jvnet.substance.painter.decoration.SubstanceDecorationPainter#
 	 * paintDecorationArea(java.awt.Graphics2D, java.awt.Component,
 	 * org.jvnet.substance.painter.decoration.DecorationAreaType, int, int,
-	 * org.jvnet.substance.theme.SubstanceTheme)
+	 * org.jvnet.substance.api.SubstanceSkin)
 	 */
 	public void paintDecorationArea(Graphics2D graphics, Component comp,
 			DecorationAreaType decorationAreaType, int width, int height,
@@ -125,17 +125,17 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * Paints the title background.
 	 * 
 	 * @param graphics
-	 * 		Graphics context.
+	 *            Graphics context.
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param decorationAreaType
-	 * 		Decoration area type. Must not be <code>null</code>.
+	 *            Decoration area type. Must not be <code>null</code>.
 	 * @param width
-	 * 		Width.
+	 *            Width.
 	 * @param height
-	 * 		Height.
+	 *            Height.
 	 * @param skin
-	 * 		Skin for painting the title background.
+	 *            Skin for painting the title background.
 	 */
 	private void paintTitleBackground(Graphics2D graphics, Component comp,
 			DecorationAreaType decorationAreaType, int width, int height,
@@ -157,7 +157,7 @@ public abstract class ImageWrapperDecorationPainter implements
 		if (this.isPaintingSeparators) {
 			temp.translate(0, height - 1);
 			SeparatorPainterUtils.paintSeparator(comp, temp, tileScheme, width,
-					0, JSeparator.HORIZONTAL, false, 0);
+					0, JSeparator.HORIZONTAL, false, 0, 0, false);
 			temp.dispose();
 		}
 	}
@@ -166,20 +166,21 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * Paints the background of non-title decoration areas.
 	 * 
 	 * @param graphics
-	 * 		Graphics context.
+	 *            Graphics context.
 	 * @param parent
-	 * 		Component ancestor for computing the correct offset of the
-	 * 		background painting.
+	 *            Component ancestor for computing the correct offset of the
+	 *            background painting.
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param decorationAreaType
-	 * 		Decoration area type. Must not be <code>null</code>.
+	 *            Decoration area type. Must not be <code>null</code>.
 	 * @param width
-	 * 		Width.
+	 *            Width.
 	 * @param height
-	 * 		Height.
+	 *            Height.
 	 * @param skin
-	 * 		Skin for painting the background of non-title decoration areas.
+	 *            Skin for painting the background of non-title decoration
+	 *            areas.
 	 */
 	private void paintExtraBackground(Graphics2D graphics, Container parent,
 			Component comp, DecorationAreaType decorationAreaType, int width,
@@ -251,8 +252,8 @@ public abstract class ImageWrapperDecorationPainter implements
 					.getWidth();
 
 			temp.translate(-dx, totalOffsetY);
-			SeparatorPainterUtils.paintSeparator(comp, temp, tileScheme, pWidth,
-					0, JSeparator.HORIZONTAL, false, 0);
+			SeparatorPainterUtils.paintSeparator(comp, temp, tileScheme,
+					pWidth, 0, JSeparator.HORIZONTAL, false, 0, 0, false);
 		}
 
 		temp.dispose();
@@ -264,23 +265,23 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * This method should respect the current {@link #textureAlpha} value.
 	 * 
 	 * @param g
-	 * 		Graphic context.
+	 *            Graphic context.
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param tileScheme
-	 * 		Theme for the tile colorization.
+	 *            Scheme for the tile colorization.
 	 * @param offsetTextureX
-	 * 		X offset for the tiling.
+	 *            X offset for the tiling.
 	 * @param offsetTextureY
-	 * 		Y offset for the tiling.
+	 *            Y offset for the tiling.
 	 * @param x
-	 * 		X coordinate of the tiling region.
+	 *            X coordinate of the tiling region.
 	 * @param y
-	 * 		Y coordinate of the tiling region.
+	 *            Y coordinate of the tiling region.
 	 * @param width
-	 * 		Width of the tiling region.
+	 *            Width of the tiling region.
 	 * @param height
-	 * 		Height of the tiling region.
+	 *            Height of the tiling region.
 	 */
 	protected void tileArea(Graphics2D g, Component comp,
 			SubstanceColorScheme tileScheme, int offsetTextureX,
@@ -314,7 +315,7 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * Sets the base decoration painter.
 	 * 
 	 * @param baseDecorationPainter
-	 * 		Base decoration painter.
+	 *            Base decoration painter.
 	 */
 	public void setBaseDecorationPainter(
 			SubstanceDecorationPainter baseDecorationPainter) {
@@ -325,7 +326,7 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * Sets the alpha channel for the image texture.
 	 * 
 	 * @param textureAlpha
-	 * 		Alpha channel for the image texture.
+	 *            Alpha channel for the image texture.
 	 */
 	public void setTextureAlpha(float textureAlpha) {
 		this.textureAlpha = textureAlpha;
@@ -335,12 +336,12 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * Sets indication whether this painter paints separators.
 	 * 
 	 * @param isPaintingSeparators
-	 * 		If <code>true</code>, this painter will paint horizontal separators
-	 * 		on the bottom of each component passed to either {@link
-	 * 		#paintTitleBackground(Graphics2D, Component, DecorationAreaType,
-	 * 		int, int, SubstanceSkin)} or {@link
-	 * 		#paintExtraBackground(Graphics2D, Container, Component,
-	 * 		DecorationAreaType, int, int, SubstanceSkin)}.
+	 *            If <code>true</code>, this painter will paint horizontal
+	 *            separators on the bottom of each component passed to either
+	 *            {@link #paintTitleBackground(Graphics2D, Component, DecorationAreaType, int, int, SubstanceSkin)}
+	 *            or
+	 *            {@link #paintExtraBackground(Graphics2D, Container, Component, DecorationAreaType, int, int, SubstanceSkin)}
+	 *            .
 	 */
 	public void setPaintingSeparators(boolean isPaintingSeparators) {
 		this.isPaintingSeparators = isPaintingSeparators;
@@ -350,7 +351,7 @@ public abstract class ImageWrapperDecorationPainter implements
 	 * Returns a colorized image tile.
 	 * 
 	 * @param scheme
-	 * 		Color scheme for the colorization.
+	 *            Color scheme for the colorization.
 	 * @return Colorized tile.
 	 */
 	protected Image getColorizedTile(SubstanceColorScheme scheme) {
@@ -360,7 +361,8 @@ public abstract class ImageWrapperDecorationPainter implements
 					.getWidth(null), this.originalTile.getHeight(null),
 					BufferedImage.TYPE_INT_ARGB);
 			tileBi.getGraphics().drawImage(this.originalTile, 0, 0, null);
-			result = SubstanceImageCreator.getColorSchemeImage(tileBi, scheme);
+			result = SubstanceImageCreator.getColorSchemeImage(tileBi, scheme,
+					0.0f);
 			this.colorizedTileMap.put(scheme.getDisplayName(), result);
 		}
 		return result;
