@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -65,7 +65,7 @@ public class MatteDecorationPainter implements SubstanceDecorationPainter {
 	 * @seeorg.jvnet.substance.painter.decoration.SubstanceDecorationPainter#
 	 * paintDecorationArea(java.awt.Graphics2D, java.awt.Component,
 	 * org.jvnet.substance.painter.decoration.DecorationAreaType, int, int,
-	 * org.jvnet.substance.theme.SubstanceTheme)
+	 * org.jvnet.substance.api.SubstanceSkin)
 	 */
 	public void paintDecorationArea(Graphics2D graphics, Component comp,
 			DecorationAreaType decorationAreaType, int width, int height,
@@ -87,20 +87,20 @@ public class MatteDecorationPainter implements SubstanceDecorationPainter {
 	 * Paints the title background.
 	 * 
 	 * @param graphics
-	 * 		Graphics context.
+	 *            Graphics context.
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param width
-	 * 		Width.
+	 *            Width.
 	 * @param height
-	 * 		Height.
+	 *            Height.
 	 * @param scheme
-	 * 		Color scheme for painting the title background.
+	 *            Color scheme for painting the title background.
 	 */
 	private void paintTitleBackground(Graphics2D graphics, Component comp,
 			int width, int height, SubstanceColorScheme scheme) {
 		Graphics2D temp = (Graphics2D) graphics.create();
-		this.fill(temp, scheme, 0, 0, 0, width, height);
+		this.fill(temp, comp, scheme, 0, 0, 0, width, height);
 		temp.dispose();
 	}
 
@@ -108,18 +108,18 @@ public class MatteDecorationPainter implements SubstanceDecorationPainter {
 	 * Paints the background of non-title decoration areas.
 	 * 
 	 * @param graphics
-	 * 		Graphics context.
+	 *            Graphics context.
 	 * @param parent
-	 * 		Component ancestor for computing the correct offset of the
-	 * 		background painting.
+	 *            Component ancestor for computing the correct offset of the
+	 *            background painting.
 	 * @param comp
-	 * 		Component.
+	 *            Component.
 	 * @param width
-	 * 		Width.
+	 *            Width.
 	 * @param height
-	 * 		Height.
+	 *            Height.
 	 * @param scheme
-	 * 		Color scheme for painting the title background.
+	 *            Color scheme for painting the title background.
 	 */
 	private void paintExtraBackground(Graphics2D graphics, Container parent,
 			Component comp, int width, int height, SubstanceColorScheme scheme) {
@@ -164,7 +164,7 @@ public class MatteDecorationPainter implements SubstanceDecorationPainter {
 		// int pHeight = parent.getHeight();
 		//
 		Graphics2D temp = (Graphics2D) graphics.create();
-		this.fill(temp, scheme, dy, 0, 0, width, height);
+		this.fill(temp, comp, scheme, dy, 0, 0, width, height);
 		temp.dispose();
 	}
 
@@ -172,22 +172,28 @@ public class MatteDecorationPainter implements SubstanceDecorationPainter {
 	 * Fills the relevant part with the gradient fill.
 	 * 
 	 * @param graphics
-	 * 		Graphics.
+	 *            Graphics.
+	 * @param comp
+	 *            Component.
 	 * @param scheme
-	 * 		Color scheme to use.
+	 *            Color scheme to use.
 	 * @param offsetY
-	 * 		Vertical offset.
+	 *            Vertical offset.
 	 * @param x
-	 * 		X coordinate of the fill area.
+	 *            X coordinate of the fill area.
 	 * @param y
-	 * 		Y coordinate of the fill area.
+	 *            Y coordinate of the fill area.
 	 * @param width
-	 * 		Fill area width.
+	 *            Fill area width.
 	 * @param height
-	 * 		Fill area height.
+	 *            Fill area height.
 	 */
-	protected void fill(Graphics2D graphics, SubstanceColorScheme scheme,
-			int offsetY, int x, int y, int width, int height) {
+	protected void fill(Graphics2D graphics, Component comp,
+			SubstanceColorScheme scheme, int offsetY, int x, int y, int width,
+			int height) {
+		// System.out.println(comp.getClass().getName() + ":"
+		// + scheme.getDisplayName());
+
 		// 0 - 50 : light -> medium
 		// 50 - : medium fill
 		int flexPoint = 50;
