@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -126,17 +126,9 @@ public class SubstanceDecorationUtilities {
 				JComponent jc = (JComponent) c;
 				Object prop = jc.getClientProperty(DECORATION_AREA_TYPE);
 				if (prop instanceof DecorationAreaType) {
+					if (prop == DecorationAreaType.NONE)
+						return null;
 					return (DecorationAreaType) prop;
-					// // special case for ignoring the propagation
-					// if (c != comp) {
-					// Object toIgnore = jc
-					// .getClientProperty(DECORATION_AREA_PROPAGATES);
-					// if (toIgnore == null)
-					// result = (DecorationAreaType) prop;
-					// } else {
-					// // installed directly on the component
-					// result = (DecorationAreaType) prop;
-					// }
 				}
 			}
 			c = c.getParent();
