@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -36,8 +36,7 @@ import java.util.Collection;
 import javax.swing.JSeparator;
 import javax.swing.SwingConstants;
 
-import org.jvnet.substance.api.SubstanceColorScheme;
-import org.jvnet.substance.api.SubstanceSkin;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.painter.decoration.DecorationAreaType;
 import org.jvnet.substance.painter.decoration.SubstanceDecorationUtilities;
 import org.jvnet.substance.utils.*;
@@ -62,8 +61,6 @@ public class SeparatorPainterUtils {
 	 *            Component.
 	 * @param graphics
 	 *            Graphics context.
-	 * @param scheme
-	 *            Theme.
 	 * @param width
 	 *            Separator width.
 	 * @param height
@@ -72,9 +69,8 @@ public class SeparatorPainterUtils {
 	 *            Separator orientation.
 	 */
 	public static void paintSeparator(Component c, Graphics2D graphics,
-			SubstanceColorScheme scheme, int width, int height, int orientation) {
-		paintSeparator(c, graphics, scheme, width, height, orientation, true,
-				10);
+			int width, int height, int orientation) {
+		paintSeparator(c, graphics, width, height, orientation, true, 10);
 	}
 
 	/**
@@ -85,7 +81,7 @@ public class SeparatorPainterUtils {
 	 * @param graphics
 	 *            Graphics context.
 	 * @param scheme
-	 *            Theme.
+	 *            Color scheme.
 	 * @param width
 	 *            Separator width.
 	 * @param height
@@ -100,10 +96,51 @@ public class SeparatorPainterUtils {
 	 *            allow providing a faded appearance on those ends.
 	 */
 	public static void paintSeparator(Component c, Graphics2D graphics,
-			SubstanceColorScheme scheme, int width, int height,
-			int orientation, boolean hasShadow, int maxGradLength) {
-		paintSeparator(c, graphics, scheme, width, height, orientation,
-				hasShadow, maxGradLength, maxGradLength, false);
+			int width, int height, int orientation, boolean hasShadow,
+			int maxGradLength) {
+		paintSeparator(c, graphics, width, height, orientation, hasShadow,
+				maxGradLength, maxGradLength, false);
+	}
+
+	/**
+	 * Paints a separator.
+	 * 
+	 * @param c
+	 *            Component.
+	 * @param g2d
+	 *            Graphics context.
+	 * @param width
+	 *            Separator width.
+	 * @param height
+	 *            Separator height.
+	 * @param orientation
+	 *            Separator orientation.
+	 * @param hasShadow
+	 *            If <code>true</code>, the separator painting will have shadow.
+	 * @param maxGradLengthStart
+	 *            Specifies the maximum pixel length of the starting "ramp"
+	 *            portion of the separator. The starting ramp portion is located
+	 *            on top / left separator end and allows providing a faded
+	 *            appearance on that end.
+	 * @param maxGradLengthEnd
+	 *            Specifies the maximum pixel length of the ending "ramp"
+	 *            portion of the separator. The ending ramp portion is located
+	 *            on bottom / right separator end and allows providing a faded
+	 *            appearance on that end.
+	 * @param toEnforceAlphaColors
+	 *            If <code>true</code>, the fade sequences will always use alpha
+	 *            colors. This may affect the performance.
+	 */
+	public static void paintSeparator(Component c, Graphics2D g2d, int width,
+			int height, int orientation, boolean hasShadow,
+			int maxGradLengthStart, int maxGradLengthEnd,
+			boolean toEnforceAlphaColors) {
+		SubstanceColorScheme compScheme = SubstanceColorSchemeUtilities
+				.getColorScheme(c, ColorSchemeAssociationKind.SEPARATOR,
+						ComponentState.DEFAULT);
+		paintSeparator(c, g2d, compScheme, width, height, orientation,
+				hasShadow, maxGradLengthStart, maxGradLengthEnd,
+				toEnforceAlphaColors);
 	}
 
 	/**
@@ -114,7 +151,7 @@ public class SeparatorPainterUtils {
 	 * @param g2d
 	 *            Graphics context.
 	 * @param scheme
-	 *            Theme.
+	 *            Color scheme.
 	 * @param width
 	 *            Separator width.
 	 * @param height
@@ -287,11 +324,12 @@ public class SeparatorPainterUtils {
 				scheme.getDisplayName(), 0, height, SwingConstants.VERTICAL,
 				true, 0.0, fadeStartFraction, backgrFill.getRGB());
 
+		float borderStrokeWidth = SubstanceSizeUtils
+				.getBorderStrokeWidth(componentFontSize);
+		int offset = (int) borderStrokeWidth;
 		if (!cached.containsKey(key)) {
-			float borderStrokeWidth = SubstanceSizeUtils
-					.getBorderStrokeWidth(componentFontSize);
-			BufferedImage toCache = SubstanceCoreUtilities.getBlankImage(
-					(int) Math.ceil(2.0 * borderStrokeWidth), height);
+			BufferedImage toCache = SubstanceCoreUtilities.getBlankImage(Math
+					.max(2, (int) Math.ceil(2.0 * borderStrokeWidth)), height);
 			Graphics2D graphics = toCache.createGraphics();
 
 			boolean isDark = scheme.isDark();
@@ -320,13 +358,13 @@ public class SeparatorPainterUtils {
 					height, new float[] { 0.0f, fadeStartFraction, 1.0f },
 					new Color[] { foreDark95, foreDark95, foreLight12 });
 			graphics.setPaint(forePaint);
+			graphics.translate(borderStrokeWidth / 2, 0);
 			graphics.drawLine(0, 0, 0, height);
 
 			LinearGradientPaint backPaint = new LinearGradientPaint(0, 0, 0,
 					height, new float[] { 0.0f, fadeStartFraction, 1.0f },
 					new Color[] { back95, back95, back12 });
 			graphics.setPaint(backPaint);
-			int offset = (int) borderStrokeWidth;
 			graphics.drawLine(offset, 0, offset, height);
 
 			graphics.dispose();
@@ -336,7 +374,7 @@ public class SeparatorPainterUtils {
 		Image singleLine = cached.get(key);
 
 		for (int lineX : x) {
-			g2d.drawImage(singleLine, lineX, y, null);
+			g2d.drawImage(singleLine, lineX - offset / 2, y, null);
 		}
 	}
 
@@ -372,11 +410,12 @@ public class SeparatorPainterUtils {
 				scheme.getDisplayName(), width, 0, SwingConstants.VERTICAL,
 				true, 0.0, fadeStartFraction, isLtr, backgrFill.getRGB());
 
+		float borderStrokeWidth = SubstanceSizeUtils
+				.getBorderStrokeWidth(componentFontSize);
+		int offset = (int) borderStrokeWidth;
 		if (!cached.containsKey(key)) {
-			float borderStrokeWidth = SubstanceSizeUtils
-					.getBorderStrokeWidth(componentFontSize);
 			BufferedImage toCache = SubstanceCoreUtilities.getBlankImage(width,
-					(int) Math.ceil(2.0 * borderStrokeWidth));
+					Math.max(2, (int) Math.ceil(2.0 * borderStrokeWidth)));
 			Graphics2D graphics = toCache.createGraphics();
 
 			boolean isDark = scheme.isDark();
@@ -413,7 +452,6 @@ public class SeparatorPainterUtils {
 					new Color[] { isLtr ? back95 : back12, back95,
 							isLtr ? back12 : back95 });
 			graphics.setPaint(backPaint);
-			int offset = (int) borderStrokeWidth;
 			graphics.drawLine(0, offset, width, offset);
 
 			graphics.dispose();
@@ -423,7 +461,7 @@ public class SeparatorPainterUtils {
 		Image singleLine = cached.get(key);
 
 		for (int lineY : y) {
-			g2d.drawImage(singleLine, x, lineY, null);
+			g2d.drawImage(singleLine, x, lineY - offset / 2, null);
 		}
 	}
 }
