@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -40,6 +40,7 @@ import javax.swing.text.JTextComponent;
 import org.jvnet.lafwidget.animation.FadeKind;
 import org.jvnet.lafwidget.animation.FadeState;
 import org.jvnet.substance.api.*;
+import org.jvnet.substance.painter.decoration.SubstanceDecorationUtilities;
 
 /**
  * Various color-related utilities. This class is <b>for internal use only</b>.
@@ -289,10 +290,16 @@ public class SubstanceColorUtilities {
 		int db = color2.getBlue();
 		int da = color2.getAlpha();
 
-		int r = (int) (color1Likeness * lr + (1.0 - color1Likeness) * dr);
-		int g = (int) (color1Likeness * lg + (1.0 - color1Likeness) * dg);
-		int b = (int) (color1Likeness * lb + (1.0 - color1Likeness) * db);
-		int a = (int) (color1Likeness * la + (1.0 - color1Likeness) * da);
+		// using some interpolation values (such as 0.29 from issue 401)
+		// results in an incorrect final value without Math.round.
+		int r = (lr == dr) ? lr : (int) Math.round(color1Likeness * lr
+				+ (1.0 - color1Likeness) * dr);
+		int g = (lg == dg) ? lg : (int) Math.round(color1Likeness * lg
+				+ (1.0 - color1Likeness) * dg);
+		int b = (lb == db) ? lb : (int) Math.round(color1Likeness * lb
+				+ (1.0 - color1Likeness) * db);
+		int a = (la == da) ? la : (int) Math.round(color1Likeness * la
+				+ (1.0 - color1Likeness) * da);
 
 		return (a << 24) | (r << 16) | (g << 8) | b;
 	}
@@ -526,7 +533,8 @@ public class SubstanceColorUtilities {
 
 			ComponentState currState = ComponentState.getState(model, ab);
 			SubstanceColorScheme currScheme = SubstanceColorSchemeUtilities
-					.getColorScheme(comp, currState);
+					.getColorScheme(comp, ColorSchemeAssociationKind.MARK,
+							currState);
 			Color currColor = currScheme.getFocusRingColor();
 
 			FadeState fadeState = SubstanceFadeUtilities.getFadeState(comp,
@@ -537,7 +545,8 @@ public class SubstanceColorUtilities {
 						.getPrevComponentState(ab);
 
 				SubstanceColorScheme prevScheme = SubstanceColorSchemeUtilities
-						.getColorScheme(comp, prevState);
+						.getColorScheme(comp, ColorSchemeAssociationKind.MARK,
+								prevState);
 
 				Color prevColor = prevScheme.getFocusRingColor();
 
@@ -610,20 +619,39 @@ public class SubstanceColorUtilities {
 	 */
 	public static Color getForegroundColor(Component component,
 			ComponentState state, ComponentState prevState) {
+		boolean isMenuItem = (component instanceof JMenuItem);
+		ColorSchemeAssociationKind currAssocKind = ColorSchemeAssociationKind.FILL;
+		// use HIGHLIGHT on active menu items
+		if (isMenuItem && state.isKindActive(FadeKind.ENABLE)
+				&& (state != ComponentState.DEFAULT))
+			currAssocKind = ColorSchemeAssociationKind.HIGHLIGHT;
 		SubstanceColorScheme colorScheme = SubstanceColorSchemeUtilities
-				.getColorScheme(component, state);
+				.getColorScheme(component, currAssocKind, state);
 		if (!state.isKindActive(FadeKind.ENABLE)) {
 			return colorScheme.getForegroundColor();
 		}
-		if (component instanceof JMenuItem) {
+
+		SubstanceColorScheme prevColorScheme = colorScheme;
+		if (prevState != state) {
+			ColorSchemeAssociationKind prevAssocKind = ColorSchemeAssociationKind.FILL;
+			// use HIGHLIGHT on active menu items
+			if (isMenuItem && prevState.isKindActive(FadeKind.ENABLE)
+					&& (prevState != ComponentState.DEFAULT))
+				prevAssocKind = ColorSchemeAssociationKind.HIGHLIGHT;
+			prevColorScheme = SubstanceColorSchemeUtilities.getColorScheme(
+					component, prevAssocKind, prevState);
+		}
+
+		if (isMenuItem) {
 			return SubstanceColorUtilities.getInterpolatedForegroundColor(
-					component, null, colorScheme, state, prevState,
-					FadeKind.ROLLOVER, FadeKind.SELECTION, FadeKind.PRESS,
-					FadeKind.ARM);
+					component, null, colorScheme, state, prevColorScheme,
+					prevState, FadeKind.ROLLOVER, FadeKind.SELECTION,
+					FadeKind.PRESS, FadeKind.ARM);
 		} else {
 			return SubstanceColorUtilities.getInterpolatedForegroundColor(
-					component, null, colorScheme, state, prevState,
-					FadeKind.ROLLOVER, FadeKind.SELECTION, FadeKind.PRESS);
+					component, null, colorScheme, state, prevColorScheme,
+					prevState, FadeKind.ROLLOVER, FadeKind.SELECTION,
+					FadeKind.PRESS);
 		}
 	}
 
@@ -643,10 +671,15 @@ public class SubstanceColorUtilities {
 	public static Color getForegroundColor(JTabbedPane tabPane, int tabIndex,
 			ComponentState state, ComponentState prevState) {
 		SubstanceColorScheme colorScheme = SubstanceColorSchemeUtilities
-				.getColorScheme(tabPane, tabIndex, state);
+				.getColorScheme(tabPane, tabIndex,
+						ColorSchemeAssociationKind.TAB, state);
+		SubstanceColorScheme prevColorScheme = (prevState != state) ? SubstanceColorSchemeUtilities
+				.getColorScheme(tabPane, tabIndex,
+						ColorSchemeAssociationKind.TAB, prevState)
+				: colorScheme;
 		return SubstanceColorUtilities.getInterpolatedForegroundColor(tabPane,
-				tabIndex, colorScheme, state, prevState, FadeKind.ROLLOVER,
-				FadeKind.SELECTION, FadeKind.PRESS);
+				tabIndex, colorScheme, state, prevColorScheme, prevState,
+				FadeKind.ROLLOVER, FadeKind.SELECTION, FadeKind.PRESS);
 	}
 
 	/**
@@ -670,29 +703,23 @@ public class SubstanceColorUtilities {
 	 */
 	public static Color getInterpolatedForegroundColor(Component comp,
 			Comparable<?> componentId, SubstanceColorScheme colorScheme,
-			ComponentState state, ComponentState prevState, FadeKind... kinds) {
-		SubstanceColorScheme colorScheme2 = colorScheme;
+			ComponentState state, SubstanceColorScheme prevColorScheme,
+			ComponentState prevState, FadeKind... kinds) {
+		// SubstanceColorScheme colorScheme2 = colorScheme;
 		float cyclePos = state.getCyclePosition();
 
 		FadeState fadeState = SubstanceFadeUtilities.getFadeState(comp,
 				componentId, kinds);
 		if (fadeState != null) {
-			colorScheme = (comp instanceof JTabbedPane) ? SubstanceColorSchemeUtilities
-					.getColorScheme((JTabbedPane) comp, (Integer) componentId,
-							state)
-					: SubstanceColorSchemeUtilities.getColorScheme(comp, state);
-			colorScheme2 = (comp instanceof JTabbedPane) ? SubstanceColorSchemeUtilities
-					.getColorScheme((JTabbedPane) comp, (Integer) componentId,
-							prevState)
-					: SubstanceColorSchemeUtilities.getColorScheme(comp,
-							prevState);
 			cyclePos = fadeState.getFadePosition();
 			if (!fadeState.isFadingIn())
 				cyclePos = 1.0f - cyclePos;
 		}
 
+		// System.out.println(colorScheme.getDisplayName() + ":"
+		// + prevColorScheme.getDisplayName() + ":" + cyclePos);
 		Color c1 = colorScheme.getForegroundColor();
-		Color c2 = colorScheme2.getForegroundColor();
+		Color c2 = prevColorScheme.getForegroundColor();
 
 		return getInterpolatedColor(c1, c2, cyclePos);
 	}
@@ -729,6 +756,13 @@ public class SubstanceColorUtilities {
 		boolean isBackgroundUiResource = backgr instanceof UIResource;
 
 		if (!isBackgroundUiResource) {
+			// special case for issue 386 - if the colorization
+			// is 1.0, return the component background
+			if ((SubstanceCoreUtilities.getColorizationFactor(component) == 1.0f)
+					&& component.isEnabled()) {
+				return backgr;
+			}
+
 			SubstanceColorScheme scheme = SubstanceColorSchemeUtilities
 					.getColorScheme(component,
 							component.isEnabled() ? ComponentState.DEFAULT
@@ -738,8 +772,10 @@ public class SubstanceColorUtilities {
 			ComponentState state = component.isEnabled() ? ComponentState.DEFAULT
 					: ComponentState.DISABLED_UNSELECTED;
 			if (component instanceof JTextComponent) {
-				if (!((JTextComponent) component).isEditable()) {
-					state = ComponentState.DISABLED_UNSELECTED;
+				// special case - enabled uneditable text components
+				if (component.isEnabled()
+						&& !((JTextComponent) component).isEditable()) {
+					state = ComponentState.DISABLED_SELECTED;
 				}
 			}
 			// menu items always use the same background color so that the
@@ -749,6 +785,18 @@ public class SubstanceColorUtilities {
 			}
 			backgr = SubstanceColorUtilities.getDefaultBackgroundColor(
 					component, state);
+
+			if (!state.isKindActive(FadeKind.ENABLE)) {
+				float alpha = SubstanceColorSchemeUtilities.getAlpha(component,
+						state);
+				if (alpha < 1.0f) {
+					Color defaultColor = SubstanceColorUtilities
+							.getDefaultBackgroundColor(component,
+									ComponentState.DEFAULT);
+					backgr = SubstanceColorUtilities.getInterpolatedColor(
+							backgr, defaultColor, 1.0f - (1.0f - alpha) / 2.0f);
+				}
+			}
 		}
 		return backgr;
 	}
@@ -801,8 +849,12 @@ public class SubstanceColorUtilities {
 					.getColorScheme(comp, compState)
 					.getLightBackgroundFillColor());
 		}
-		return new ColorUIResource(SubstanceColorSchemeUtilities
-				.getColorScheme(comp, compState).getBackgroundFillColor());
+		return new ColorUIResource(SubstanceCoreUtilities.getSkin(comp)
+				.getBackgroundColorScheme(
+						SubstanceDecorationUtilities.getDecorationType(comp))
+				.getBackgroundFillColor());
+		// SubstanceColorSchemeUtilities
+		// .getColorScheme(comp, compState).getBackgroundFillColor());
 	}
 
 	/**
