@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -60,6 +60,7 @@ import org.jvnet.substance.painter.gradient.SubstanceGradientPainter;
 import org.jvnet.substance.shaper.SubstanceButtonShaper;
 import org.jvnet.substance.utils.border.SubstanceBorder;
 import org.jvnet.substance.utils.combo.SubstanceComboBoxButton;
+import org.jvnet.substance.utils.combo.SubstanceComboPopup;
 import org.jvnet.substance.utils.icon.*;
 import org.jvnet.substance.utils.menu.SubstanceMenu;
 import org.jvnet.substance.utils.scroll.SubstanceScrollButton;
@@ -276,6 +277,7 @@ public class SubstanceCoreUtilities {
 	 * @see SubstanceLookAndFeel#BUTTON_PAINT_NEVER_PROPERTY
 	 */
 	public static boolean isButtonNeverPainted(JComponent button) {
+		// small optimizations for checkboxes and radio buttons
 		if (button instanceof JCheckBox)
 			return false;
 		if (button instanceof JRadioButton)
@@ -709,6 +711,11 @@ public class SubstanceCoreUtilities {
 	 * @see SubstanceLookAndFeel#FLAT_PROPERTY
 	 */
 	public static boolean hasFlatAppearance(Component comp, boolean defaultValue) {
+		// small optimizations for checkboxes and radio buttons
+		if (comp instanceof JCheckBox)
+			return defaultValue;
+		if (comp instanceof JRadioButton)
+			return defaultValue;
 		Component c = comp;
 		while (c != null) {
 			if (c instanceof JComponent) {
@@ -743,6 +750,11 @@ public class SubstanceCoreUtilities {
 	 *         <code>false</code> otherwise.
 	 */
 	public static boolean hasFlatAppearance(AbstractButton button) {
+		// small optimizations for checkboxes and radio buttons
+		if (button instanceof JCheckBox)
+			return false;
+		if (button instanceof JRadioButton)
+			return false;
 		return ((SubstanceCoreUtilities.isToolBarButton(button) && SubstanceCoreUtilities
 				.hasFlatAppearance(button, true)) || SubstanceCoreUtilities
 				.hasFlatAppearance(button, false));
@@ -905,23 +917,9 @@ public class SubstanceCoreUtilities {
 	 *         otherwise.
 	 * @see SubstanceLookAndFeel#USE_THEMED_DEFAULT_ICONS
 	 */
-	public static boolean useThemedDefaultIcon(JComponent comp) {
-		if (comp instanceof JRadioButton)
-			return false;
-		if (comp instanceof JCheckBox)
-			return false;
-		if (comp != null) {
-			Object useThemedDefaultIconProperty = comp
-					.getClientProperty(SubstanceLookAndFeel.USE_THEMED_DEFAULT_ICONS);
-			if (Boolean.TRUE.equals(useThemedDefaultIconProperty))
-				return true;
-			if (Boolean.FALSE.equals(useThemedDefaultIconProperty))
-				return false;
-		}
-		if (Boolean.TRUE.equals(UIManager
-				.get(SubstanceLookAndFeel.USE_THEMED_DEFAULT_ICONS)))
-			return true;
-		return false;
+	public static boolean useThemedDefaultIcon() {
+		return Boolean.TRUE.equals(UIManager
+				.get(SubstanceLookAndFeel.USE_THEMED_DEFAULT_ICONS));
 	}
 
 	/**
@@ -1388,16 +1386,17 @@ public class SubstanceCoreUtilities {
 			return null;
 		ButtonModel model = button.getModel();
 		Icon result = origIcon;
-		if (SubstanceCoreUtilities.useThemedDefaultIcon(button)) {
-			boolean useThemedVersion = !model.isArmed() && !model.isPressed()
-					&& !model.isSelected();
+		if (!(button instanceof JRadioButton) && !(button instanceof JCheckBox)
+				&& SubstanceCoreUtilities.useThemedDefaultIcon()) {
+			boolean useThemedVersion = !model.isArmed()
+					&& !model.isPressed()
+					&& !model.isSelected()
+					&& !origIcon.getClass().isAnnotationPresent(
+							TransitionAware.class);
 			if (!ignoreRolloverSetting)
 				useThemedVersion = useThemedVersion && !model.isRollover();
 			if (useThemedVersion) {
-				result = new ImageIcon(SubstanceImageCreator
-						.getColorSchemeImage(button, result,
-								SubstanceColorSchemeUtilities.getColorScheme(
-										button, ComponentState.DEFAULT)));
+				result = getThemedIcon(button, result);
 			}
 		}
 		if (FadeConfigurationManager.getInstance().fadeAllowed(
@@ -1408,6 +1407,23 @@ public class SubstanceCoreUtilities {
 		return result;
 	}
 
+	public static Icon getThemedIcon(Component comp, Icon orig) {
+		SubstanceColorScheme colorScheme = SubstanceColorSchemeUtilities
+				.getColorScheme(comp, ComponentState.DEFAULT);
+		float brightnessFactor = colorScheme.isDark() ? 0.2f : 0.8f;
+		return new ImageIcon(SubstanceImageCreator.getColorSchemeImage(comp,
+				orig, colorScheme, brightnessFactor));
+	}
+
+	public static Icon getThemedIcon(JTabbedPane tab, int tabIndex, Icon orig) {
+		SubstanceColorScheme colorScheme = SubstanceColorSchemeUtilities
+				.getColorScheme(tab, tabIndex, ColorSchemeAssociationKind.TAB,
+						ComponentState.DEFAULT);
+		float brightnessFactor = colorScheme.isDark() ? 0.2f : 0.8f;
+		return new ImageIcon(SubstanceImageCreator.getColorSchemeImage(tab,
+				orig, colorScheme, brightnessFactor));
+	}
+
 	/**
 	 * Returns the current icon for the specified button.
 	 * 
@@ -1430,6 +1446,9 @@ public class SubstanceCoreUtilities {
 		if (icon == null)
 			return null;
 
+		if (icon.getClass().isAnnotationPresent(TransitionAware.class))
+			return icon;
+
 		Icon tmpIcon = null;
 
 		if (icon != null) {
@@ -2065,6 +2084,24 @@ public class SubstanceCoreUtilities {
 		return SubstanceCoreUtilities.getSkin(comp).getBorderPainter();
 	}
 
+	/**
+	 * Returns the highlight border painter for the specified component.
+	 * 
+	 * @param comp
+	 *            Component.
+	 * @return Highlight border painter for the specified component.
+	 * @see SubstanceSkin#getBorderPainter()
+	 * @see SubstanceSkin#getHighlightBorderPainter()
+	 */
+	public static SubstanceBorderPainter getHighlightBorderPainter(
+			Component comp) {
+		SubstanceBorderPainter result = SubstanceCoreUtilities.getSkin(comp)
+				.getHighlightBorderPainter();
+		if (result != null)
+			return result;
+		return getBorderPainter(comp);
+	}
+
 	/**
 	 * Returns the component hierarchy.
 	 * 
@@ -2138,14 +2175,6 @@ public class SubstanceCoreUtilities {
 		return ui.getTitlePane();
 	}
 
-	/**
-	 * Resets all image caches.
-	 */
-	public static void resetCaches() {
-		LazyResettableHashMap.reset();
-		SubstanceIconFactory.reset();
-	}
-
 	/**
 	 * Returns the arrow icon.
 	 * 
@@ -2181,10 +2210,10 @@ public class SubstanceCoreUtilities {
 					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
 						int fontSize = SubstanceSizeUtils
 								.getComponentFontSize(comp);
-						return SubstanceImageCreator.getDoubleArrowIcon(
+						return SubstanceImageCreator.getDoubleArrowIconDelta(
 								fontSize, 0, 2, 0, orientation, scheme);
 					}
-				});
+				}, "substance.general.doubleArrow");
 		return result;
 	}
 
@@ -2238,7 +2267,34 @@ public class SubstanceCoreUtilities {
 	public static SubstanceSkin getSkin(Component c) {
 		if (!SubstanceLookAndFeel.isCurrentLookAndFeel())
 			return null;
+
+		SubstanceComboPopup comboPopup = (SubstanceComboPopup) SwingUtilities
+				.getAncestorOfClass(SubstanceComboPopup.class, c);
+		if (comboPopup != null) {
+			// special case for combobox popup - take the skin
+			// of the combobox itself - issue 439
+			return getSkin(comboPopup.getCombobox());
+		}
+
 		JRootPane rootPane = SwingUtilities.getRootPane(c);
+
+		if (c instanceof SubstanceInternalFrameTitlePane) {
+			// use correct root pane for painting the
+			// title panes of internal frames
+			Component frame = c.getParent();
+			if ((frame != null) && (frame instanceof JInternalFrame)) {
+				rootPane = ((JInternalFrame) frame).getRootPane();
+			}
+		}
+		if ((c != null)
+				&& (c.getParent() instanceof SubstanceInternalFrameTitlePane)) {
+			// use correct root pane for painting the
+			// title buttons of internal frames
+			Component frame = c.getParent().getParent();
+			if ((frame != null) && (frame instanceof JInternalFrame)) {
+				rootPane = ((JInternalFrame) frame).getRootPane();
+			}
+		}
 		if (rootPane != null) {
 			Object skinProp = rootPane
 					.getClientProperty(SubstanceLookAndFeel.SKIN_PROPERTY);
@@ -2315,4 +2371,159 @@ public class SubstanceCoreUtilities {
 			throw uiThreadingViolationError;
 		}
 	}
+
+	/**
+	 * Tests UI threading violations on changing the state the specified
+	 * component.
+	 * 
+	 * @param comp
+	 *            Component.
+	 * @throws UiThreadingViolationException
+	 *             If the component is changing state off Event Dispatch Thread.
+	 */
+	public static void testComponentStateChangeThreadingViolation(Component comp) {
+		if (!SwingUtilities.isEventDispatchThread()) {
+			UiThreadingViolationException uiThreadingViolationError = new UiThreadingViolationException(
+					"Component state change must be done on Event Dispatch Thread");
+			uiThreadingViolationError.printStackTrace(System.err);
+			throw uiThreadingViolationError;
+		}
+	}
+
+	public static void traceSubstanceApiUsage(Component comp, String message) {
+		Window w = SwingUtilities.getWindowAncestor(comp);
+		String wTitle = null;
+		if (w instanceof Frame) {
+			wTitle = ((Frame) w).getTitle();
+		}
+		if (w instanceof Dialog) {
+			wTitle = ((Dialog) w).getTitle();
+		}
+		throw new IllegalArgumentException(message + " [component "
+				+ comp.getClass().getSimpleName() + " in window "
+				+ w.getClass().getSimpleName() + ":'" + wTitle + "' under "
+				+ UIManager.getLookAndFeel().getName() + "]");
+	}
+
+	/**
+	 * Scans {@code imageList} for best-looking image of specified dimensions.
+	 * Image can be scaled and/or padded with transparency.
+	 */
+	public static BufferedImage getScaledIconImage(
+			java.util.List<Image> imageList, int width, int height) {
+		if (width == 0 || height == 0) {
+			return null;
+		}
+		Image bestImage = null;
+		int bestWidth = 0;
+		int bestHeight = 0;
+		double bestSimilarity = 3; // Impossibly high value
+		for (Iterator<Image> i = imageList.iterator(); i.hasNext();) {
+			// Iterate imageList looking for best matching image.
+			// 'Similarity' measure is defined as good scale factor and small
+			// insets.
+			// best possible similarity is 0 (no scale, no insets).
+			// It's found while the experiments that good-looking result is
+			// achieved
+			// with scale factors x1, x3/4, x2/3, xN, x1/N.
+			Image im = i.next();
+			if (im == null) {
+				continue;
+			}
+			int iw;
+			int ih;
+			try {
+				iw = im.getWidth(null);
+				ih = im.getHeight(null);
+			} catch (Exception e) {
+				continue;
+			}
+			if (iw > 0 && ih > 0) {
+				// Calc scale factor
+				double scaleFactor = Math.min((double) width / (double) iw,
+						(double) height / (double) ih);
+				// Calculate scaled image dimensions
+				// adjusting scale factor to nearest "good" value
+				int adjw = 0;
+				int adjh = 0;
+				double scaleMeasure = 1; // 0 - best (no) scale, 1 - impossibly
+				// bad
+				if (scaleFactor >= 2) {
+					// Need to enlarge image more than twice
+					// Round down scale factor to multiply by integer value
+					scaleFactor = Math.floor(scaleFactor);
+					adjw = iw * (int) scaleFactor;
+					adjh = ih * (int) scaleFactor;
+					scaleMeasure = 1.0 - 0.5 / scaleFactor;
+				} else if (scaleFactor >= 1) {
+					// Don't scale
+					scaleFactor = 1.0;
+					adjw = iw;
+					adjh = ih;
+					scaleMeasure = 0;
+				} else if (scaleFactor >= 0.75) {
+					// Multiply by 3/4
+					scaleFactor = 0.75;
+					adjw = iw * 3 / 4;
+					adjh = ih * 3 / 4;
+					scaleMeasure = 0.3;
+				} else if (scaleFactor >= 0.6666) {
+					// Multiply by 2/3
+					scaleFactor = 0.6666;
+					adjw = iw * 2 / 3;
+					adjh = ih * 2 / 3;
+					scaleMeasure = 0.33;
+				} else {
+					// Multiply size by 1/scaleDivider
+					// where scaleDivider is minimum possible integer
+					// larger than 1/scaleFactor
+					double scaleDivider = Math.ceil(1.0 / scaleFactor);
+					scaleFactor = 1.0 / scaleDivider;
+					adjw = (int) Math.round(iw / scaleDivider);
+					adjh = (int) Math.round(ih / scaleDivider);
+					scaleMeasure = 1.0 - 1.0 / scaleDivider;
+				}
+				double similarity = ((double) width - (double) adjw) / width
+						+ ((double) height - (double) adjh) / height + // Large
+						// padding
+						// is
+						// bad
+						scaleMeasure; // Large rescale is bad
+				if (similarity < bestSimilarity) {
+					bestSimilarity = similarity;
+					bestImage = im;
+					bestWidth = adjw;
+					bestHeight = adjh;
+				}
+				if (similarity == 0)
+					break;
+			}
+		}
+		if (bestImage == null) {
+			// No images were found, possibly all are broken
+			return null;
+		}
+		BufferedImage bimage = new BufferedImage(width, height,
+				BufferedImage.TYPE_INT_ARGB);
+		Graphics2D g = bimage.createGraphics();
+		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
+				RenderingHints.VALUE_INTERPOLATION_BILINEAR);
+
+		// BufferedImage bestBuffered = getBlankImage(bestImage.getWidth(null),
+		// bestImage.getHeight(null));
+		// bestBuffered.getGraphics().drawImage(bestImage, 0, 0, null);
+		// BufferedImage scaled =
+		// LafWidgetUtilities.createThumbnail(bestBuffered,
+		// bestWidth);
+
+		try {
+			int x = (width - bestWidth) / 2;
+			int y = (height - bestHeight) / 2;
+			g.drawImage(bestImage, x, y, bestWidth, bestHeight, null);
+		} finally {
+			g.dispose();
+		}
+		return bimage;
+	}
+
 }
