@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -151,7 +151,8 @@ public final class SubstanceImageCreator {
 				.getBorderStrokeWidth(componentFontSize);
 		GeneralPath contourInner = (c instanceof JTextComponent) ? null
 				: SubstanceOutlineUtilities.getBaseOutline(width, height,
-						radius, null, borderThickness + borderDelta);
+						radius - borderThickness, null, borderThickness
+								+ borderDelta);
 		borderPainter.paintBorder(graphics, c, width, height, contour,
 				contourInner, borderScheme1, borderScheme2, cyclePos,
 				borderScheme1 != borderScheme2);
@@ -368,21 +369,18 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the arrow.
 	 * @return Double arrow icon.
 	 */
-	public static Icon getDoubleArrowIcon(int fontSize, float deltaWidth,
+	public static Icon getDoubleArrowIconDelta(int fontSize, float deltaWidth,
 			float deltaHeight, float deltaStrokeWidth, int direction,
 			SubstanceColorScheme colorScheme) {
-		float arrowWidth = SubstanceSizeUtils
-				.getArrowIconWidth(SubstanceSizeUtils.getControlFontSize())
+		float arrowWidth = SubstanceSizeUtils.getArrowIconWidth(fontSize)
 				+ deltaWidth;
-		float arrowHeight = SubstanceSizeUtils
-				.getArrowIconHeight(SubstanceSizeUtils.getControlFontSize())
+		float arrowHeight = SubstanceSizeUtils.getArrowIconHeight(fontSize)
 				+ deltaHeight;
 		float arrowStrokeWidth = SubstanceSizeUtils
-				.getDoubleArrowStrokeWidth(SubstanceSizeUtils
-						.getControlFontSize())
+				.getDoubleArrowStrokeWidth(fontSize)
 				+ deltaStrokeWidth;
-		return getDoubleArrowIcon(arrowWidth, arrowHeight, arrowStrokeWidth,
-				direction, colorScheme);
+		return getDoubleArrowIcon(fontSize, arrowWidth, arrowHeight,
+				arrowStrokeWidth, direction, colorScheme);
 	}
 
 	/**
@@ -404,15 +402,16 @@ public final class SubstanceImageCreator {
 	 * @see SwingConstants#SOUTH
 	 * @see SwingConstants#EAST
 	 */
-	public static Icon getDoubleArrowIcon(float width, float height,
-			float strokeWidth, int direction, SubstanceColorScheme colorScheme) {
-		int delta = 3 + SubstanceSizeUtils.getExtraPadding(SubstanceSizeUtils
-				.getControlFontSize());
+	public static Icon getDoubleArrowIcon(int fontSize, float width,
+			float height, float strokeWidth, int direction,
+			SubstanceColorScheme colorScheme) {
+		int delta = 3 + 2 * SubstanceSizeUtils.getExtraPadding(fontSize) / 3;
 		BufferedImage downArrowImage = SubstanceCoreUtilities.getBlankImage(
-				(int) width, (int) height + delta - 2);
+				(int) width, (int) height);
 
-		BufferedImage singleArrow = getArrow(width, Math.max(1, height - delta
-				+ 2), strokeWidth, SwingConstants.SOUTH, colorScheme);
+		BufferedImage singleArrow = getArrow(width,
+				Math.max(1, height - delta), strokeWidth, SwingConstants.SOUTH,
+				colorScheme);
 
 		// get graphics and set hints
 		Graphics2D graphics = (Graphics2D) downArrowImage.getGraphics();
@@ -422,6 +421,9 @@ public final class SubstanceImageCreator {
 		graphics.drawImage(singleArrow, 0, 0, null);
 		graphics.drawImage(singleArrow, 0, delta, null);
 
+		// graphics.setColor(Color.blue);
+		// graphics.drawRect(0, 0, (int) width - 1, (int) height - 1);
+
 		int quadrantCounterClockwise = 0;
 		switch (direction) {
 		case SwingConstants.NORTH:
@@ -604,24 +606,26 @@ public final class SubstanceImageCreator {
 	 *         parameters.
 	 */
 	public static BufferedImage getRadioButton(JComponent component,
-			int dimension, ComponentState componentState,
-			ComponentState prevState, int offsetX,
-			SubstanceColorScheme scheme1, SubstanceColorScheme scheme2,
+			SubstanceGradientPainter fillPainter,
+			SubstanceBorderPainter borderPainter, int dimension,
+			ComponentState componentState, ComponentState prevState,
+			int offsetX, SubstanceColorScheme currFillColorScheme,
+			SubstanceColorScheme prevFillColorScheme,
+			SubstanceColorScheme currMarkColorScheme,
+			SubstanceColorScheme prevMarkColorScheme,
+			SubstanceColorScheme currBorderColorScheme,
+			SubstanceColorScheme prevBorderColorScheme,
 			float interpolationCyclePos10, float checkMarkVisibility) {
 
 		// ComponentState.ColorSchemeKind kind = componentState
 		// .getColorSchemeKind();
 
-		float cyclePos = (scheme1 != scheme2) ? interpolationCyclePos10
+		float cyclePos = (currFillColorScheme != prevFillColorScheme) ? interpolationCyclePos10
 				: componentState.getCyclePosition();
 
-		SubstanceGradientPainter painter = SubstanceCoreUtilities
-				.getGradientPainter(component);
 		if (componentState.getColorSchemeKind() != ColorSchemeKind.CURRENT) {
-			painter = SimplisticSoftBorderReverseGradientPainter.INSTANCE;
+			fillPainter = SimplisticSoftBorderReverseGradientPainter.INSTANCE;
 		}
-		SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-				.getBorderPainter(component);
 
 		float borderThickness = SubstanceSizeUtils
 				.getBorderStrokeWidth(dimension);
@@ -641,19 +645,19 @@ public final class SubstanceImageCreator {
 				AlphaComposite.SRC_OVER, alpha));
 
 		graphics.translate(offsetX, 0);
-		painter.paintContourBackground(graphics, component, dimension,
-				dimension, contourBorder, false, scheme1, scheme2, cyclePos,
-				true, scheme1 != scheme2);
+		fillPainter.paintContourBackground(graphics, component, dimension,
+				dimension, contourBorder, false, currFillColorScheme,
+				prevFillColorScheme, cyclePos, true,
+				currFillColorScheme != prevFillColorScheme);
 
 		Shape contourInner = new Ellipse2D.Float(delta + borderThickness, delta
 				+ borderThickness, dimension - 2 * delta - 2 * borderThickness,
 				dimension - 2 * delta - 2 * borderThickness);
 
 		borderPainter.paintBorder(graphics, component, dimension, dimension,
-				contourBorder, contourInner, SubstanceColorSchemeUtilities
-						.getBorderColorScheme(component, componentState),
-				SubstanceColorSchemeUtilities.getBorderColorScheme(component,
-						prevState), cyclePos, scheme1 != scheme2);
+				contourBorder, contourInner, currBorderColorScheme,
+				prevBorderColorScheme, cyclePos,
+				currBorderColorScheme != prevBorderColorScheme);
 		graphics.setComposite(AlphaComposite.SrcOver);
 		// graphics.translate(-offsetX, 0);
 
@@ -672,28 +676,22 @@ public final class SubstanceImageCreator {
 
 			graphics.setComposite(AlphaComposite.getInstance(
 					AlphaComposite.SRC_OVER, alpha * checkMarkVisibility));
-			drawRadioMark(
-					graphics,
-					SubstanceColorUtilities
-							.getMarkColor(
-									scheme1,
-									componentState.getColorSchemeKind() != ColorSchemeKind.DISABLED),
+			drawRadioMark(graphics, SubstanceColorUtilities
+					.getMarkColor(currMarkColorScheme, componentState
+							.getColorSchemeKind() != ColorSchemeKind.DISABLED),
 					markOval);
 			graphics.setComposite(AlphaComposite.getInstance(
 					AlphaComposite.SRC_OVER, alpha * checkMarkVisibility
 							* cyclePos));
-			drawRadioMark(
-					graphics,
-					SubstanceColorUtilities
-							.getMarkColor(
-									scheme2,
-									componentState.getColorSchemeKind() != ColorSchemeKind.DISABLED),
+			drawRadioMark(graphics, SubstanceColorUtilities
+					.getMarkColor(prevMarkColorScheme, componentState
+							.getColorSchemeKind() != ColorSchemeKind.DISABLED),
 					markOval);
 		} else {
 			// draw ghost mark holder
 			graphics.setPaint(new GradientPaint(rc + radius, rc - radius,
-					scheme1.getDarkColor(), rc - radius, rc + radius, scheme1
-							.getLightColor()));
+					currFillColorScheme.getDarkColor(), rc - radius, rc
+							+ radius, currFillColorScheme.getLightColor()));
 			Shape markOval = new Ellipse2D.Double(rc - radius, rc - radius,
 					2 * radius, 2 * radius);
 			graphics.setComposite(AlphaComposite.getInstance(
@@ -749,11 +747,21 @@ public final class SubstanceImageCreator {
 	 *         component state.
 	 */
 	public static BufferedImage getCheckBox(AbstractButton button,
-			int dimension, ComponentState componentState,
-			ComponentState prevState, SubstanceColorScheme colorScheme1,
-			SubstanceColorScheme colorScheme2, float interpolationCyclePos,
-			float checkMarkVisibility, boolean isCheckMarkFadingOut) {
+			SubstanceGradientPainter fillPainter,
+			SubstanceBorderPainter borderPainter, int dimension,
+			ComponentState componentState, ComponentState prevState,
+			SubstanceColorScheme currFillColorScheme,
+			SubstanceColorScheme prevFillColorScheme,
+			SubstanceColorScheme currMarkColorScheme,
+			SubstanceColorScheme prevMarkColorScheme,
+			SubstanceColorScheme currBorderColorScheme,
+			SubstanceColorScheme prevBorderColorScheme,
+			float interpolationCyclePos, float checkMarkVisibility,
+			boolean isCheckMarkFadingOut) {
 
+		// int checkMarkSize = SubstanceSizeUtils
+		// .getCheckBoxMarkSize(SubstanceSizeUtils
+		// .getComponentFontSize(button));
 		int offset = SubstanceSizeUtils.getAdjustedSize(SubstanceSizeUtils
 				.getComponentFontSize(button), 3, 9, 1, false);
 		int delta = offset;
@@ -765,7 +773,7 @@ public final class SubstanceImageCreator {
 			cornerRadius = 2;
 		}
 
-		float cyclePos = (colorScheme1 != colorScheme2) ? interpolationCyclePos
+		float cyclePos = (currFillColorScheme != prevFillColorScheme) ? interpolationCyclePos
 				: componentState.getCyclePosition();
 
 		int contourDim = dimension - delta;
@@ -775,13 +783,9 @@ public final class SubstanceImageCreator {
 		GeneralPath contour = SubstanceOutlineUtilities.getBaseOutline(
 				contourDim, contourDim, cornerRadius, null, borderDelta);
 
-		SubstanceGradientPainter painter = SubstanceCoreUtilities
-				.getGradientPainter(button);
 		if (componentState.getColorSchemeKind() != ColorSchemeKind.CURRENT) {
-			painter = SimplisticSoftBorderReverseGradientPainter.INSTANCE;
+			fillPainter = SimplisticSoftBorderReverseGradientPainter.INSTANCE;
 		}
-		SubstanceBorderPainter borderPainter = SubstanceCoreUtilities
-				.getBorderPainter(button);
 
 		BufferedImage offBackground = SubstanceCoreUtilities.getBlankImage(
 				dimension, dimension);
@@ -792,21 +796,21 @@ public final class SubstanceImageCreator {
 				AlphaComposite.SRC_OVER, alpha));
 
 		graphics.translate(delta - 1, delta - 1);
-		painter.paintContourBackground(graphics, button, contourDim,
-				contourDim, contour, false, colorScheme1, colorScheme2,
-				cyclePos, true, colorScheme1 != colorScheme2);
+		fillPainter.paintContourBackground(graphics, button, contourDim,
+				contourDim, contour, false, currFillColorScheme,
+				prevFillColorScheme, cyclePos, true,
+				currFillColorScheme != prevFillColorScheme);
 
 		// graphics.drawImage(background, 0, 0, null);
 		int borderThickness = (int) SubstanceSizeUtils
 				.getBorderStrokeWidth(dimension);
 		GeneralPath contourInner = SubstanceOutlineUtilities.getBaseOutline(
-				contourDim, contourDim, cornerRadius, null, borderThickness
-						+ borderDelta);
+				contourDim, contourDim, cornerRadius - borderThickness, null,
+				borderThickness + borderDelta);
 		borderPainter.paintBorder(graphics, button, contourDim, contourDim,
-				contour, contourInner, SubstanceColorSchemeUtilities
-						.getBorderColorScheme(button, componentState),
-				SubstanceColorSchemeUtilities.getBorderColorScheme(button,
-						prevState), cyclePos, colorScheme1 != colorScheme2);
+				contour, contourInner, currBorderColorScheme,
+				prevBorderColorScheme, cyclePos,
+				currBorderColorScheme != prevBorderColorScheme);
 		graphics.translate(-delta, 1 - delta);
 		if (componentState.isKindActive(FadeKind.SELECTION)
 				|| (checkMarkVisibility > 0.0)) {
@@ -818,14 +822,14 @@ public final class SubstanceImageCreator {
 
 			BufferedImage checkMark1 = SubstanceImageCreator.getCheckMark(
 					button, dimension - 2 * offset / 3, componentState
-							.isKindActive(FadeKind.ENABLE), colorScheme1,
-					checkMarkVisibility);
+							.isKindActive(FadeKind.ENABLE),
+					currMarkColorScheme, checkMarkVisibility);
 			graphics.drawImage(checkMark1, 1 + 2 * offset / 3,
 					(dimension < 14) ? 0 : -1, null);
 			BufferedImage checkMark2 = SubstanceImageCreator.getCheckMark(
 					button, dimension - 2 * offset / 3, componentState
-							.isKindActive(FadeKind.ENABLE), colorScheme2,
-					checkMarkVisibility);
+							.isKindActive(FadeKind.ENABLE),
+					prevMarkColorScheme, checkMarkVisibility);
 			graphics.setComposite(AlphaComposite.getInstance(
 					AlphaComposite.SRC_OVER, alpha * cyclePos));
 			graphics.drawImage(checkMark2, 1 + 2 * offset / 3,
@@ -922,7 +926,7 @@ public final class SubstanceImageCreator {
 	 *            Y offset of the echo.
 	 * @return Image with overlayed echo.
 	 */
-	public static BufferedImage overlayEcho(BufferedImage image,
+	private static BufferedImage overlayEcho(BufferedImage image,
 			float echoAlpha, int offsetX, int offsetY) {
 		int width = image.getWidth();
 		int height = image.getHeight();
@@ -966,9 +970,10 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the icon.
 	 * @return <code>Minimize</code> icon.
 	 */
-	public static Icon getMinimizeIcon(SubstanceColorScheme scheme) {
+	public static Icon getMinimizeIcon(SubstanceColorScheme scheme,
+			SubstanceColorScheme backgroundScheme) {
 		int iSize = SubstanceSizeUtils.getTitlePaneIconSize();
-		return getMinimizeIcon(iSize, scheme);
+		return getMinimizeIcon(iSize, scheme, backgroundScheme);
 	}
 
 	/**
@@ -980,18 +985,31 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the icon.
 	 * @return <code>Minimize</code> icon.
 	 */
-	public static Icon getMinimizeIcon(int iSize, SubstanceColorScheme scheme) {
+	public static Icon getMinimizeIcon(int iSize, SubstanceColorScheme scheme,
+			SubstanceColorScheme backgroundScheme) {
 		BufferedImage image = SubstanceCoreUtilities
 				.getBlankImage(iSize, iSize);
 		Graphics2D graphics = (Graphics2D) image.getGraphics();
 		int start = (iSize / 4) - 2;
 		int end = (3 * iSize / 4);// - 1;
 		int size = end - start - 3;
-		Color color = scheme.getForegroundColor();
+		Color color = SubstanceColorUtilities.getMarkColor(scheme, true);
 		graphics.setColor(color);
 		graphics.fillRect(start + 2, end - 2, size, 3);
+
+		int fgStrength = SubstanceColorUtilities.getColorBrightness(color
+				.getRGB());
+		int fgNegativeStrength = SubstanceColorUtilities
+				.getColorBrightness(SubstanceColorUtilities
+						.getNegativeColor(color.getRGB()));
+		int bgStrength = SubstanceColorUtilities
+				.getColorBrightness(backgroundScheme.getLightColor().getRGB());
+		boolean noEcho = (fgStrength > fgNegativeStrength)
+				&& (fgStrength < bgStrength);
+
 		return new ImageIcon(SubstanceImageCreator.overlayEcho(image,
-				SubstanceColorUtilities.getColorStrength(color), 1, 1));
+				noEcho ? 0 : SubstanceColorUtilities.getColorStrength(color),
+				1, 1));
 	}
 
 	/**
@@ -1001,7 +1019,8 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the icon.
 	 * @return <code>Restore</code> icon.
 	 */
-	public static Icon getRestoreIcon(SubstanceColorScheme scheme) {
+	public static Icon getRestoreIcon(SubstanceColorScheme scheme,
+			SubstanceColorScheme backgroundScheme) {
 		int iSize = SubstanceSizeUtils.getTitlePaneIconSize();
 		BufferedImage image = SubstanceCoreUtilities
 				.getBlankImage(iSize, iSize);
@@ -1009,7 +1028,7 @@ public final class SubstanceImageCreator {
 		int start = (iSize / 4) - 2;
 		int end = (3 * iSize / 4) - 1;
 		int size = end - start - 3;
-		Color color = scheme.getForegroundColor();
+		Color color = SubstanceColorUtilities.getMarkColor(scheme, true);
 		graphics.setColor(color);
 		graphics.drawRect(start, end - size + 1, size, size);
 		graphics.drawLine(start, end - size + 2, start + size, end - size + 2);
@@ -1017,8 +1036,20 @@ public final class SubstanceImageCreator {
 		graphics.drawLine(end, start + 1, end, start + size + 1);
 		graphics.drawLine(start + size + 2, start + size + 1, end, start + size
 				+ 1);
+
+		int fgStrength = SubstanceColorUtilities.getColorBrightness(color
+				.getRGB());
+		int fgNegativeStrength = SubstanceColorUtilities
+				.getColorBrightness(SubstanceColorUtilities
+						.getNegativeColor(color.getRGB()));
+		int bgStrength = SubstanceColorUtilities
+				.getColorBrightness(backgroundScheme.getLightColor().getRGB());
+		boolean noEcho = (fgStrength > fgNegativeStrength)
+				&& (fgStrength < bgStrength);
+
 		return new ImageIcon(SubstanceImageCreator.overlayEcho(image,
-				SubstanceColorUtilities.getColorStrength(color), 1, 1));
+				noEcho ? 0 : SubstanceColorUtilities.getColorStrength(color),
+				1, 1));
 	}
 
 	/**
@@ -1028,9 +1059,10 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the icon.
 	 * @return <code>Maximize</code> icon.
 	 */
-	public static Icon getMaximizeIcon(SubstanceColorScheme scheme) {
+	public static Icon getMaximizeIcon(SubstanceColorScheme scheme,
+			SubstanceColorScheme backgroundScheme) {
 		int iSize = SubstanceSizeUtils.getTitlePaneIconSize();
-		return getMaximizeIcon(iSize, scheme);
+		return getMaximizeIcon(iSize, scheme, backgroundScheme);
 	}
 
 	/**
@@ -1042,18 +1074,31 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the icon.
 	 * @return <code>Maximize</code> icon.
 	 */
-	public static Icon getMaximizeIcon(int iSize, SubstanceColorScheme scheme) {
+	public static Icon getMaximizeIcon(int iSize, SubstanceColorScheme scheme,
+			SubstanceColorScheme backgroundScheme) {
 		BufferedImage image = SubstanceCoreUtilities
 				.getBlankImage(iSize, iSize);
 		Graphics2D graphics = (Graphics2D) image.getGraphics();
 		int start = (iSize / 4) - 1;
 		int end = iSize - start - 1;// (3 * iSize / 4);
-		Color color = scheme.getForegroundColor();
+		Color color = SubstanceColorUtilities.getMarkColor(scheme, true);
 		graphics.setColor(color);
 		graphics.drawRect(start, start, end - start, end - start);
 		graphics.drawLine(start, start + 1, end, start + 1);
+
+		int fgStrength = SubstanceColorUtilities.getColorBrightness(color
+				.getRGB());
+		int fgNegativeStrength = SubstanceColorUtilities
+				.getColorBrightness(SubstanceColorUtilities
+						.getNegativeColor(color.getRGB()));
+		int bgStrength = SubstanceColorUtilities
+				.getColorBrightness(backgroundScheme.getLightColor().getRGB());
+		boolean noEcho = (fgStrength > fgNegativeStrength)
+				&& (fgStrength < bgStrength);
+
 		return new ImageIcon(SubstanceImageCreator.overlayEcho(image,
-				SubstanceColorUtilities.getColorStrength(color), 1, 1));
+				noEcho ? 0 : SubstanceColorUtilities.getColorStrength(color),
+				1, 1));
 	}
 
 	/**
@@ -1063,9 +1108,10 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the icon.
 	 * @return <code>Close</code> icon.
 	 */
-	public static Icon getCloseIcon(SubstanceColorScheme scheme) {
+	public static Icon getCloseIcon(SubstanceColorScheme scheme,
+			SubstanceColorScheme backgroundScheme) {
 		return SubstanceImageCreator.getCloseIcon(SubstanceSizeUtils
-				.getTitlePaneIconSize(), scheme);
+				.getTitlePaneIconSize(), scheme, backgroundScheme);
 	}
 
 	/**
@@ -1077,7 +1123,9 @@ public final class SubstanceImageCreator {
 	 *            Color scheme for the icon.
 	 * @return <code>Close</code> icon.
 	 */
-	public static Icon getCloseIcon(int iSize, SubstanceColorScheme colorScheme) {
+	public static Icon getCloseIcon(int iSize,
+			SubstanceColorScheme colorScheme,
+			SubstanceColorScheme backgroundScheme) {
 		BufferedImage image = SubstanceCoreUtilities
 				.getBlankImage(iSize, iSize);
 		Graphics2D graphics = (Graphics2D) image.getGraphics();
@@ -1098,12 +1146,33 @@ public final class SubstanceImageCreator {
 				BasicStroke.JOIN_ROUND);
 
 		graphics.setStroke(stroke);
-		Color color = colorScheme.getForegroundColor();
+		Color color = SubstanceColorUtilities.getMarkColor(colorScheme, true);
 		graphics.setColor(color);
 		graphics.drawLine(start, start, end, end);
 		graphics.drawLine(start, end, end, start);
+
+		int fgStrength = SubstanceColorUtilities.getColorBrightness(color
+				.getRGB());
+		int fgNegativeStrength = SubstanceColorUtilities
+				.getColorBrightness(SubstanceColorUtilities
+						.getNegativeColor(color.getRGB()));
+		int bgStrength = SubstanceColorUtilities
+				.getColorBrightness(backgroundScheme.getLightColor().getRGB());
+		boolean noEcho = (fgStrength > fgNegativeStrength)
+				&& (fgStrength < bgStrength);
+		// System.out.println(SubstanceColorUtilities.getColorBrightness(color
+		// .getRGB())
+		// + ":"
+		// + SubstanceColorUtilities
+		// .getColorBrightness(SubstanceColorUtilities
+		// .getNegativeColor(color.getRGB()))
+		// + ":"
+		// + SubstanceColorUtilities.getColorBrightness(backgroundScheme
+		// .getLightColor().getRGB()));
+
 		return new ImageIcon(SubstanceImageCreator.overlayEcho(image,
-				SubstanceColorUtilities.getColorStrength(color), 1, 1));
+				noEcho ? 0 : SubstanceColorUtilities.getColorStrength(color),
+				1, 1));
 	}
 
 	/**
@@ -1127,8 +1196,8 @@ public final class SubstanceImageCreator {
 	 *            if <code>true</code>, the gradient will be vertical, if
 	 *            <code>false</code>, the gradient will be horizontal.
 	 */
-	public static void paintRectangularBackground(Graphics g, int startX,
-			int startY, int width, int height,
+	public static void paintRectangularBackground(Component c, Graphics g,
+			int startX, int startY, int width, int height,
 			SubstanceColorScheme colorScheme, float borderAlpha,
 			boolean isVertical) {
 		Graphics2D graphics = (Graphics2D) g.create();
@@ -1161,8 +1230,8 @@ public final class SubstanceImageCreator {
 			g2d.setComposite(TransitionLayout.getAlphaComposite(null,
 					borderAlpha, graphics));
 
-			paintSimpleBorder(g2d, width, height, colorScheme, colorScheme,
-					0.0f);
+			paintSimpleBorderAliased(c, g2d, width, height, colorScheme,
+					colorScheme, 0.0f);
 
 			g2d.dispose();
 		}
@@ -1185,14 +1254,16 @@ public final class SubstanceImageCreator {
 	 * @param cyclePos
 	 *            Interpolation cycle.
 	 */
-	public static void paintSimpleBorder(Graphics2D g2d, int width, int height,
-			SubstanceColorScheme colorScheme,
+	public static void paintSimpleBorder(Component c, Graphics2D g2d,
+			int width, int height, SubstanceColorScheme colorScheme,
 			SubstanceColorScheme colorScheme2, float cyclePos) {
-		int componentFontSize = SubstanceSizeUtils.getComponentFontSize(null);
-		int borderDelta = (int) Math.floor(SubstanceSizeUtils
-				.getBorderStrokeWidth(componentFontSize) / 2.0);
-		float borderThickness = SubstanceSizeUtils
-				.getBorderStrokeWidth(componentFontSize);
+		int componentFontSize = SubstanceSizeUtils.getComponentFontSize(c);
+		// int borderDelta = (int) Math.floor(SubstanceSizeUtils
+		// .getBorderStrokeWidth(componentFontSize) / 2.0);
+		// int borderDelta2 = (int) Math.floor(SubstanceSizeUtils
+		// .getBorderStrokeWidth(componentFontSize));
+		float borderThickness = (float) Math.floor(SubstanceSizeUtils
+				.getBorderStrokeWidth(componentFontSize));
 
 		g2d.setColor(SubstanceColorUtilities.getMidBorderColor(colorScheme,
 				colorScheme2, cyclePos));
@@ -1203,8 +1274,38 @@ public final class SubstanceImageCreator {
 		int joinKind = BasicStroke.JOIN_ROUND;
 		int capKind = BasicStroke.CAP_BUTT;
 		g2d.setStroke(new BasicStroke(borderThickness, capKind, joinKind));
-		g2d.drawRect(borderDelta, borderDelta, width - 1 - 2 * borderDelta,
-				height - 1 - 2 * borderDelta);
+		g2d.draw(new Rectangle2D.Float((borderThickness - 1.0f) / 2.0f,
+				(borderThickness - 1.0f) / 2.0f, width - 1
+						- (borderThickness - 1.5f), height - 1
+						- (borderThickness - 1.5f)));
+		// g2d.drawRect(borderDelta, borderDelta, width - 1 - borderDelta2,
+		// height
+		// - 1 - borderDelta2);
+	}
+
+	public static void paintSimpleBorderAliased(Component c, Graphics2D g2d,
+			int width, int height, SubstanceColorScheme colorScheme,
+			SubstanceColorScheme colorScheme2, float cyclePos) {
+		int componentFontSize = SubstanceSizeUtils.getComponentFontSize(c);
+		// int borderDelta = (int) Math.floor(SubstanceSizeUtils
+		// .getBorderStrokeWidth(componentFontSize) / 2.0);
+		// int borderDelta2 = (int) Math.floor(SubstanceSizeUtils
+		// .getBorderStrokeWidth(componentFontSize));
+		float borderThickness = (float) Math.floor(SubstanceSizeUtils
+				.getBorderStrokeWidth(componentFontSize));
+
+		g2d.setColor(SubstanceColorUtilities.getMidBorderColor(colorScheme,
+				colorScheme2, cyclePos));
+		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
+				RenderingHints.VALUE_ANTIALIAS_OFF);
+		g2d.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
+				RenderingHints.VALUE_STROKE_NORMALIZE);
+		int joinKind = BasicStroke.JOIN_MITER;
+		int capKind = BasicStroke.CAP_SQUARE;
+		g2d.setStroke(new BasicStroke(borderThickness, capKind, joinKind));
+		g2d.draw(new Rectangle2D.Float(borderThickness / 2.0f,
+				borderThickness / 2.0f, width - borderThickness, height
+						- borderThickness));
 	}
 
 	/**
@@ -1232,8 +1333,8 @@ public final class SubstanceImageCreator {
 	 * @param isVertical
 	 *            Indication of horizontal / vertical orientation.
 	 */
-	public static void paintRectangularStripedBackground(Graphics g,
-			int startX, int startY, int width, int height,
+	public static void paintRectangularStripedBackground(Component c,
+			Graphics g, int startX, int startY, int width, int height,
 			SubstanceColorScheme colorScheme, BufferedImage stripeImage,
 			int stripeOffset, float borderAlpha, boolean isVertical) {
 		Graphics2D graphics = (Graphics2D) g.create(startX, startY, width,
@@ -1287,8 +1388,8 @@ public final class SubstanceImageCreator {
 			g2d.setComposite(TransitionLayout.getAlphaComposite(null,
 					borderAlpha, graphics));
 
-			paintSimpleBorder(g2d, width, height, colorScheme, colorScheme,
-					0.0f);
+			paintSimpleBorderAliased(c, g2d, width, height, colorScheme,
+					colorScheme, 0.0f);
 			g2d.dispose();
 		}
 		graphics.dispose();
@@ -1484,8 +1585,8 @@ public final class SubstanceImageCreator {
 						bumpDotDiameter - 1), null, colorScheme1, colorScheme2,
 				interpolationCyclePos, colorScheme1 != colorScheme2);
 
-		graphics.setComposite(AlphaComposite.getInstance(
-				AlphaComposite.SRC_OVER, 0.8f));
+		graphics.setComposite(TransitionLayout.getAlphaComposite(divider, 0.8f,
+				g));
 		for (int col = 0; col < bumpColumns; col++) {
 			int cx = bumpColOffset + col * bumpCellSize;
 			for (int row = 0; row < bumpRows; row++) {
@@ -1564,14 +1665,17 @@ public final class SubstanceImageCreator {
 	 * 
 	 * @param tree
 	 *            Tree.
-	 * @param colorScheme
-	 *            Icon color scheme.
+	 * @param fillScheme
+	 *            Icon fill color scheme.
+	 * @param borderScheme
+	 *            Icon border color scheme.
 	 * @param isCollapsed
 	 *            Collapsed state.
 	 * @return Tree icon.
 	 */
 	public static BufferedImage getTreeIcon(JTree tree,
-			SubstanceColorScheme colorScheme, boolean isCollapsed) {
+			SubstanceColorScheme fillScheme, SubstanceColorScheme borderScheme,
+			boolean isCollapsed) {
 		int fontSize = SubstanceSizeUtils.getComponentFontSize(tree);
 		int dim = SubstanceSizeUtils.getTreeIconSize(fontSize);
 		BufferedImage result = SubstanceCoreUtilities.getBlankImage(dim + 2,
@@ -1595,25 +1699,24 @@ public final class SubstanceImageCreator {
 
 		g2.translate(0, 1);
 
-		boolean isDark = colorScheme.isDark();
+		boolean isDark = fillScheme.isDark();
 
-		SubstanceColorScheme fillScheme = new ShiftColorScheme(colorScheme,
-				colorScheme.getExtraLightColor(), 0.7);
+		fillScheme = new ShiftColorScheme(fillScheme, fillScheme
+				.getExtraLightColor(), 0.7);
 
 		gradPainter.paintContourBackground(g2, tree, dim - 1, dim - 1, contour,
 				false, fillScheme, fillScheme, 0, false, false);
-		SubstanceColorScheme borderScheme = new ShiftColorScheme(colorScheme,
-				isDark ? colorScheme.getUltraLightColor() : colorScheme
-						.getLightColor(), 0.95);
+		borderScheme = new ShiftColorScheme(borderScheme, isDark ? borderScheme
+				.getUltraLightColor() : borderScheme.getLightColor(), 0.5);
 		fbp.paintBorder(g2, tree, dim - 1, dim - 1, contour, null,
 				borderScheme, borderScheme, 0, false);
 
 		g2.translate(-1, -1);
 
-		Color signColor = isDark ? colorScheme.getUltraLightColor().brighter()
-				.brighter() : colorScheme.getUltraDarkColor();
+		Color signColor = isDark ? borderScheme.getUltraLightColor().brighter()
+				.brighter() : borderScheme.getUltraDarkColor();
 		if ((tree != null) && !tree.isEnabled())
-			signColor = colorScheme.getForegroundColor();
+			signColor = borderScheme.getForegroundColor();
 		g2.setColor(signColor);
 		int mid = dim / 2;
 		int length = 5 * dim / 12;
@@ -2091,13 +2194,15 @@ public final class SubstanceImageCreator {
 	 * @return Scheme-based version of the original icon.
 	 */
 	public static BufferedImage getColorSchemeImage(Component comp,
-			Icon original, SubstanceColorScheme colorScheme) {
+			Icon original, SubstanceColorScheme colorScheme,
+			float originalBrightnessFactor) {
 		int w = original.getIconWidth();
 		int h = original.getIconHeight();
 		BufferedImage origImage = SubstanceCoreUtilities.getBlankImage(w, h);
 		original.paintIcon(comp, origImage.getGraphics(), 0, 0);
 
-		return getColorSchemeImage(origImage, colorScheme);
+		return getColorSchemeImage(origImage, colorScheme,
+				originalBrightnessFactor);
 	}
 
 	/**
@@ -2113,8 +2218,8 @@ public final class SubstanceImageCreator {
 	 * @return Scheme-based version of the original icon.
 	 */
 	public static BufferedImage getColorSchemeImage(BufferedImage original,
-			SubstanceColorScheme colorScheme) {
-		return ColorSchemeFilter.getColorSchemeFilter(colorScheme).filter(
-				original, null);
+			SubstanceColorScheme colorScheme, float originalBrightnessFactor) {
+		return ColorSchemeFilter.getColorSchemeFilter(colorScheme,
+				originalBrightnessFactor).filter(original, null);
 	}
 }
