@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -447,32 +447,60 @@ public class SubstanceInternalFrameTitlePane extends
 
 		Icon restoreIcon = new TransitionAwareIcon(this.maxButton,
 				new TransitionAwareIcon.Delegate() {
-					public Icon getColorSchemeIcon(SubstanceColorScheme theme) {
-						return SubstanceIconFactory.getTitlePaneIcon(
-								SubstanceIconFactory.IconKind.RESTORE, theme);
+					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
+						return SubstanceIconFactory
+								.getTitlePaneIcon(
+										SubstanceIconFactory.IconKind.RESTORE,
+										scheme,
+										SubstanceCoreUtilities
+												.getSkin(
+														SubstanceInternalFrameTitlePane.this)
+												.getBackgroundColorScheme(
+														DecorationAreaType.SECONDARY_TITLE_PANE));
 					}
-				});
+				}, "substance.internalFrame.restoreIcon");
 		Icon maximizeIcon = new TransitionAwareIcon(this.maxButton,
 				new TransitionAwareIcon.Delegate() {
-					public Icon getColorSchemeIcon(SubstanceColorScheme theme) {
-						return SubstanceIconFactory.getTitlePaneIcon(
-								SubstanceIconFactory.IconKind.MAXIMIZE, theme);
+					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
+						return SubstanceIconFactory
+								.getTitlePaneIcon(
+										SubstanceIconFactory.IconKind.MAXIMIZE,
+										scheme,
+										SubstanceCoreUtilities
+												.getSkin(
+														SubstanceInternalFrameTitlePane.this)
+												.getBackgroundColorScheme(
+														DecorationAreaType.SECONDARY_TITLE_PANE));
 					}
-				});
+				}, "substance.internalFrame.maxIcon");
 		Icon minimizeIcon = new TransitionAwareIcon(this.iconButton,
 				new TransitionAwareIcon.Delegate() {
-					public Icon getColorSchemeIcon(SubstanceColorScheme theme) {
-						return SubstanceIconFactory.getTitlePaneIcon(
-								SubstanceIconFactory.IconKind.MINIMIZE, theme);
+					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
+						return SubstanceIconFactory
+								.getTitlePaneIcon(
+										SubstanceIconFactory.IconKind.MINIMIZE,
+										scheme,
+										SubstanceCoreUtilities
+												.getSkin(
+														SubstanceInternalFrameTitlePane.this)
+												.getBackgroundColorScheme(
+														DecorationAreaType.SECONDARY_TITLE_PANE));
 					}
-				});
+				}, "substance.internalFrame.minIcon");
 		Icon closeIcon = new TransitionAwareIcon(this.closeButton,
 				new TransitionAwareIcon.Delegate() {
-					public Icon getColorSchemeIcon(SubstanceColorScheme theme) {
-						return SubstanceIconFactory.getTitlePaneIcon(
-								SubstanceIconFactory.IconKind.CLOSE, theme);
+					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
+						return SubstanceIconFactory
+								.getTitlePaneIcon(
+										SubstanceIconFactory.IconKind.CLOSE,
+										scheme,
+										SubstanceCoreUtilities
+												.getSkin(
+														SubstanceInternalFrameTitlePane.this)
+												.getBackgroundColorScheme(
+														DecorationAreaType.SECONDARY_TITLE_PANE));
 					}
-				});
+				}, "substance.internalFrame.closeIcon");
 		if (this.frame.isIcon()) {
 			this.iconButton.setIcon(restoreIcon);
 			this.iconButton.setToolTipText(SubstanceCoreUtilities
@@ -727,6 +755,15 @@ public class SubstanceInternalFrameTitlePane extends
 
 			y = (getHeight() - buttonHeight) / 2;
 
+			// Icon icon = frame.getFrameIcon();
+			// int iconHeight = 0;
+			// if (icon != null) {
+			// iconHeight = icon.getIconHeight();
+			// }
+			// int xMenuBar = (leftToRight) ? 2 : w - 16 - 2;
+			// menuBar.setBounds(xMenuBar, (getHeight() - iconHeight) / 2, 16,
+			// 16);
+
 			if (frame.isClosable()) {
 				// if (isPalette) {
 				// spacing = 3;
