@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -74,7 +74,7 @@ public class SubstanceSizeUtils {
 	 * Sets the new value for the control font size.
 	 * 
 	 * @param size
-	 * 		Control font size.
+	 *            Control font size.
 	 */
 	public static void setControlFontSize(int size) {
 		controlFontSize = size;
@@ -87,7 +87,7 @@ public class SubstanceSizeUtils {
 	 * size from {@link #getControlFontSize()}.
 	 * 
 	 * @param c
-	 * 		Component.
+	 *            Component.
 	 * @return Font size for the specified component
 	 */
 	public static int getComponentFontSize(Component c) {
@@ -102,8 +102,7 @@ public class SubstanceSizeUtils {
 	 * <ul>
 	 * <li>The <code>baseSize</code> parameter specifies the base value</li>
 	 * <li>The <code>forEachBase</code> and <code>toAdjustBy</code> specify how
-	 * to adjust the resulting value based on the passed <code>fontSize</code>.
-	 * </li>
+	 * to adjust the resulting value based on the passed <code>fontSize</code>.</li>
 	 * </ul>
 	 * 
 	 * For example, if you want base value to be 1.2 pixels, and have it grow by
@@ -117,13 +116,13 @@ public class SubstanceSizeUtils {
 	 * </ul>
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @param baseSize
-	 * 		The base value.
+	 *            The base value.
 	 * @param forEachBase
-	 * 		Base units for computing the adjustment.
+	 *            Base units for computing the adjustment.
 	 * @param toAdjustBy
-	 * 		Adjustment amount for computing the adjustment.
+	 *            Adjustment amount for computing the adjustment.
 	 * @return Adjusted size.
 	 */
 	public static float getAdjustedSize(int fontSize, float baseSize,
@@ -142,8 +141,7 @@ public class SubstanceSizeUtils {
 	 * <ul>
 	 * <li>The <code>baseSize</code> parameter specifies the base value</li>
 	 * <li>The <code>forEachBase</code> and <code>toAdjustBy</code> specify how
-	 * to adjust the resulting value based on the passed <code>fontSize</code>.
-	 * </li>
+	 * to adjust the resulting value based on the passed <code>fontSize</code>.</li>
 	 * </ul>
 	 * 
 	 * For example, if you want base value to be 4 pixels, and have it grow by 1
@@ -157,16 +155,16 @@ public class SubstanceSizeUtils {
 	 * </ul>
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @param baseSize
-	 * 		The base value.
+	 *            The base value.
 	 * @param forEachBase
-	 * 		Base units for computing the adjustment.
+	 *            Base units for computing the adjustment.
 	 * @param toAdjustBy
-	 * 		Adjustment amount for computing the adjustment.
+	 *            Adjustment amount for computing the adjustment.
 	 * @param toRoundAsEven
-	 * 		If <code>true</code>, the final value will be rounded down to the
-	 * 		closest even value.
+	 *            If <code>true</code>, the final value will be rounded down to
+	 *            the closest even value.
 	 * @return Adjusted size.
 	 */
 	public static int getAdjustedSize(int fontSize, int baseSize,
@@ -184,7 +182,7 @@ public class SubstanceSizeUtils {
 	 * Returns the height of arrow icons under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Height of arrow icons under the specified font size.
 	 */
 	public static float getArrowIconHeight(int fontSize) {
@@ -197,7 +195,7 @@ public class SubstanceSizeUtils {
 	 * Returns the width of arrow icons under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Width of arrow icons under the specified font size.
 	 */
 	public static float getArrowIconWidth(int fontSize) {
@@ -211,7 +209,7 @@ public class SubstanceSizeUtils {
 	 * Returns the stroke width of arrow icons under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Stroke width of arrow icons under the specified font size.
 	 */
 	public static float getArrowStrokeWidth(int fontSize) {
@@ -222,7 +220,7 @@ public class SubstanceSizeUtils {
 	 * Returns the stroke width of borders under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Stroke width of borders under the specified font size.
 	 */
 	public static float getBorderStrokeWidth(int fontSize) {
@@ -233,7 +231,7 @@ public class SubstanceSizeUtils {
 	 * Returns the border for check boxes under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Border for check boxes under the specified font size.
 	 */
 	public static Border getCheckBoxBorder(int fontSize) {
@@ -250,7 +248,7 @@ public class SubstanceSizeUtils {
 	 * size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Check mark size for check boxes under the specified font size.
 	 */
 	public static int getCheckBoxMarkSize(int fontSize) {
@@ -262,9 +260,9 @@ public class SubstanceSizeUtils {
 	 * specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Corner radius for {@link ClassicButtonShaper} under the specified
-	 * 	font size.
+	 *         font size.
 	 */
 	public static float getClassicButtonCornerRadius(int fontSize) {
 		return getAdjustedSize(fontSize, 2, 6, 1, false);
@@ -274,7 +272,7 @@ public class SubstanceSizeUtils {
 	 * Returns the combo box border insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Combo box border insets under the specified font size.
 	 */
 	public static Insets getComboBorderInsets(int fontSize) {
@@ -289,7 +287,7 @@ public class SubstanceSizeUtils {
 	 * Returns the combo box text border insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Combo box text border insets under the specified font size.
 	 */
 	public static Insets getComboTextBorderInsets(int fontSize) {
@@ -309,7 +307,7 @@ public class SubstanceSizeUtils {
 	 * Returns the default border insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Default border insets under the specified font size.
 	 */
 	public static Insets getDefaultBorderInsets(int fontSize) {
@@ -324,7 +322,7 @@ public class SubstanceSizeUtils {
 	 * size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Stroke width of double arrow icons under the specified font size.
 	 */
 	public static float getDoubleArrowStrokeWidth(int fontSize) {
@@ -335,7 +333,7 @@ public class SubstanceSizeUtils {
 	 * Returns the diameter of a drag bump dot under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Diameter of a drag bump dot under the specified font size.
 	 */
 	public static int getDragBumpDiameter(int fontSize) {
@@ -347,7 +345,7 @@ public class SubstanceSizeUtils {
 	 * size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Diameter of a big drag bump dot under the specified font size.
 	 */
 	public static int getBigDragBumpDiameter(int fontSize) {
@@ -361,7 +359,7 @@ public class SubstanceSizeUtils {
 	 * Returns the extra padding amount under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Extra padding amount under the specified font size.
 	 */
 	public static int getExtraPadding(int fontSize) {
@@ -374,7 +372,7 @@ public class SubstanceSizeUtils {
 	 * Returns the focus ring padding amount under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Focus ring padding amount under the specified font size.
 	 */
 	public static int getFocusRingPadding(int fontSize) {
@@ -388,7 +386,7 @@ public class SubstanceSizeUtils {
 	 * Returns the stroke width of focus rings under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Stroke width of focus rings under the specified font size.
 	 */
 	public static float getFocusStrokeWidth(int fontSize) {
@@ -399,7 +397,7 @@ public class SubstanceSizeUtils {
 	 * Returns the list cell renderer insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return List cell renderer insets under the specified font size.
 	 */
 	public static Insets getListCellRendererInsets(int fontSize) {
@@ -423,9 +421,9 @@ public class SubstanceSizeUtils {
 	 * items under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Check mark size of check box menu items and radio button menu
-	 * 	items under the specified font size.
+	 *         items under the specified font size.
 	 */
 	public static int getMenuCheckMarkSize(int fontSize) {
 		int result = fontSize - 2;
@@ -438,7 +436,7 @@ public class SubstanceSizeUtils {
 	 * Returns the margin for menu items under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Margin for menu items under the specified font size.
 	 */
 	public static int getMenuItemMargin(int fontSize) {
@@ -450,9 +448,9 @@ public class SubstanceSizeUtils {
 	 * specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Gap between text and icon in button menu items under the
-	 * 	specified font size.
+	 *         specified font size.
 	 */
 	public static int getTextIconGap(int fontSize) {
 		return getAdjustedSize(fontSize, 4, 3, 1, false);
@@ -462,7 +460,7 @@ public class SubstanceSizeUtils {
 	 * Returns the minimum button height under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Minimum button height under the specified font size.
 	 */
 	public static int getMinButtonHeight(int fontSize) {
@@ -473,7 +471,7 @@ public class SubstanceSizeUtils {
 	 * Returns the maximum button height under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Maximum button height under the specified font size.
 	 */
 	public static int getMinButtonWidth(int fontSize) {
@@ -485,9 +483,9 @@ public class SubstanceSizeUtils {
 	 * font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Password dot diameter for password fields under the specified
-	 * 	font size.
+	 *         font size.
 	 */
 	public static int getPasswordDotDiameter(int fontSize) {
 		return getAdjustedSize(fontSize, 7, 2, 1, false);
@@ -498,9 +496,9 @@ public class SubstanceSizeUtils {
 	 * size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Password dot gap for password fields under the specified font
-	 * 	size.
+	 *         size.
 	 */
 	public static int getPasswordDotGap(int fontSize) {
 		return (fontSize - 6) / 3;
@@ -510,7 +508,7 @@ public class SubstanceSizeUtils {
 	 * Returns the border for radio buttons under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Border for radio buttons under the specified font size.
 	 */
 	public static Border getRadioButtonBorder(int fontSize) {
@@ -527,7 +525,7 @@ public class SubstanceSizeUtils {
 	 * size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Check mark size for radio buttons under the specified font size.
 	 */
 	public static int getRadioButtonMarkSize(int fontSize) {
@@ -541,11 +539,11 @@ public class SubstanceSizeUtils {
 	 * Returns the width of scroll bars under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Width of scroll bars under the specified font size.
 	 */
 	public static int getScrollBarWidth(int fontSize) {
-		int result = fontSize + 4;
+		int result = (int)(getArrowIconWidth(fontSize) * 3 / 2);
 		if (result % 2 == 0)
 			result++;
 		return result;
@@ -555,7 +553,7 @@ public class SubstanceSizeUtils {
 	 * Returns the slider thumb icon size under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Slider thumb icon size under the specified font size.
 	 */
 	public static int getSliderIconSize(int fontSize) {
@@ -569,7 +567,7 @@ public class SubstanceSizeUtils {
 	 * Returns the slider tick size under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Slider tick size under the specified font size.
 	 */
 	public static int getSliderTickSize(int fontSize) {
@@ -580,18 +578,40 @@ public class SubstanceSizeUtils {
 	 * Returns the slider track size under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Slider track size under the specified font size.
 	 */
 	public static int getSliderTrackSize(int fontSize) {
 		return Math.max(4, 5 + (fontSize - 11) / 2);
 	}
 
+	/**
+	 * Returns the height of small arrow icons under the specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Height of small arrow icons under the specified font size.
+	 */
+	public static float getSmallArrowIconHeight(int fontSize) {
+		return getArrowIconHeight(fontSize) - 1;
+	}
+
+	/**
+	 * Returns the width of small arrow icons under the specified font size.
+	 * 
+	 * @param fontSize
+	 *            Font size.
+	 * @return Width of small arrow icons under the specified font size.
+	 */
+	public static float getSmallArrowIconWidth(int fontSize) {
+		return getArrowIconWidth(fontSize) - 2;
+	}
+
 	/**
 	 * Returns the height of spinner arrow icons under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Height of spinner arrow icons under the specified font size.
 	 */
 	public static float getSpinnerArrowIconHeight(int fontSize) {
@@ -605,7 +625,7 @@ public class SubstanceSizeUtils {
 	 * Returns the width of spinner arrow icons under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Width of spinner arrow icons under the specified font size.
 	 */
 	public static float getSpinnerArrowIconWidth(int fontSize) {
@@ -620,7 +640,7 @@ public class SubstanceSizeUtils {
 	 * Returns the spinner border insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Spinner border insets under the specified font size.
 	 */
 	public static Insets getSpinnerBorderInsets(int fontSize) {
@@ -635,7 +655,7 @@ public class SubstanceSizeUtils {
 	 * Returns the spinner arrow button insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Spinner arrow button insets under the specified font size.
 	 */
 	public static Insets getSpinnerArrowButtonInsets(int fontSize) {
@@ -645,22 +665,22 @@ public class SubstanceSizeUtils {
 				borderStrokeWidth, borderStrokeWidth);
 	}
 
-	/**
-	 * Returns the spinner button width under the specified font size.
-	 * 
-	 * @param fontSize
-	 * 		Font size.
-	 * @return Spinner button width under the specified font size.
-	 */
-	public static float getSpinnerButtonWidth(int fontSize) {
-		return getArrowIconWidth(fontSize) * 3 / 2;
-	}
+//	/**
+//	 * Returns the spinner button width under the specified font size.
+//	 * 
+//	 * @param fontSize
+//	 *            Font size.
+//	 * @return Spinner button width under the specified font size.
+//	 */
+//	public static int getSpinnerButtonWidth(int fontSize) {
+//		return (int)(getArrowIconWidth(fontSize) * 3 / 2);
+//	}
 
 	/**
 	 * Returns the spinner text border insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Spinner text border insets under the specified font size.
 	 */
 	public static Insets getSpinnerTextBorderInsets(int fontSize) {
@@ -674,9 +694,9 @@ public class SubstanceSizeUtils {
 	 * font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Height of split pane divider arrow icons under the specified font
-	 * 	size.
+	 *         size.
 	 */
 	public static float getSplitPaneArrowIconHeight(int fontSize) {
 		float result = SubstanceSizeUtils.getArrowIconHeight(fontSize)
@@ -689,9 +709,9 @@ public class SubstanceSizeUtils {
 	 * font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Width of split pane divider arrow icons under the specified font
-	 * 	size.
+	 *         size.
 	 */
 	public static float getSplitPaneArrowIconWidth(int fontSize) {
 		float result = SubstanceSizeUtils.getArrowIconWidth(fontSize)
@@ -706,9 +726,9 @@ public class SubstanceSizeUtils {
 	 * specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Offset of the first split pane divider button under the specified
-	 * 	font size.
+	 *         font size.
 	 */
 	public static int getSplitPaneButtonOffset(int fontSize) {
 		return getAdjustedSize(fontSize, 2, 3, 1, false);
@@ -720,7 +740,7 @@ public class SubstanceSizeUtils {
 	 * assumed.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Tabbed pane content insets under the specified size.
 	 */
 	public static Insets getTabbedPaneContentInsets(int fontSize) {
@@ -734,7 +754,7 @@ public class SubstanceSizeUtils {
 	 * Returns the stroke width of tab close buttons under the specified size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Stroke width of tab close buttons under the specified size.
 	 */
 	public static float getTabCloseButtonStrokeWidth(int fontSize) {
@@ -745,7 +765,7 @@ public class SubstanceSizeUtils {
 	 * Returns the icon size of tab close buttons under the specified size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Icon size of tab close buttons under the specified size.
 	 */
 	public static int getTabCloseIconSize(int fontSize) {
@@ -756,7 +776,7 @@ public class SubstanceSizeUtils {
 	 * Returns the table cell renderer insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Table cell renderer insets under the specified font size.
 	 */
 	public static Insets getTableCellRendererInsets(int fontSize) {
@@ -769,7 +789,7 @@ public class SubstanceSizeUtils {
 	 * Returns the text border insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Text border insets under the specified font size.
 	 */
 	public static Insets getTextBorderInsets(int fontSize) {
@@ -785,9 +805,9 @@ public class SubstanceSizeUtils {
 	 * specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Text button padding amount on left and right sides under the
-	 * 	specified font size.
+	 *         specified font size.
 	 */
 	public static int getTextButtonLRPadding(int fontSize) {
 		return SubstanceSizeUtils.getAdjustedSize(fontSize, 3, 2, 1, false);
@@ -797,7 +817,7 @@ public class SubstanceSizeUtils {
 	 * Returns the icon size of title pane buttons under the specified size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Icon size of title pane buttons under the specified size.
 	 */
 	public static int getTitlePaneIconSize() {
@@ -808,7 +828,7 @@ public class SubstanceSizeUtils {
 	 * Returns the tool bar drag inset under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Tool bar drag inset under the specified font size.
 	 */
 	public static int getToolBarDragInset(int fontSize) {
@@ -819,7 +839,7 @@ public class SubstanceSizeUtils {
 	 * Returns the tool bar insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Tool bar insets under the specified font size.
 	 */
 	public static Insets getToolBarInsets(int fontSize) {
@@ -832,7 +852,7 @@ public class SubstanceSizeUtils {
 	 * Returns the tooltip border insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Tooltip border insets under the specified font size.
 	 */
 	public static Insets getToolTipBorderInsets(int fontSize) {
@@ -846,7 +866,7 @@ public class SubstanceSizeUtils {
 	 * Returns the tree cell renderer insets under the specified font size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Tree cell renderer insets under the specified font size.
 	 */
 	public static Insets getTreeCellRendererInsets(int fontSize) {
@@ -860,9 +880,9 @@ public class SubstanceSizeUtils {
 	 * size.
 	 * 
 	 * @param fontSize
-	 * 		Font size.
+	 *            Font size.
 	 * @return Icon size of tree expand / collapse icons under the specified
-	 * 	size.
+	 *         size.
 	 */
 	public static int getTreeIconSize(int fontSize) {
 		int extraPadding = SubstanceSizeUtils.getExtraPadding(fontSize);
@@ -883,7 +903,7 @@ public class SubstanceSizeUtils {
 	 * Resets the points to pixels ratio based on the specified font policy.
 	 * 
 	 * @param fontPolicy
-	 * 		Font policy.
+	 *            Font policy.
 	 */
 	public static void resetPointsToPixelsRatio(FontPolicy fontPolicy) {
 		if (fontPolicy instanceof DefaultGnomeFontPolicy) {
