@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -40,10 +40,10 @@ import javax.swing.plaf.basic.BasicSplitPaneDivider;
 import javax.swing.plaf.basic.BasicSplitPaneUI;
 
 import org.jvnet.lafwidget.animation.*;
+import org.jvnet.lafwidget.layout.TransitionLayout;
 import org.jvnet.substance.SubstanceLookAndFeel;
 import org.jvnet.substance.SubstanceSplitPaneUI;
-import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.painter.utils.SubstanceFillBackgroundDelegate;
 import org.jvnet.substance.utils.icon.TransitionAwareIcon;
 
@@ -75,7 +75,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 	 * Simple constructor.
 	 * 
 	 * @param ui
-	 * 		Associated UI.
+	 *            Associated UI.
 	 */
 	public SubstanceSplitPaneDivider(SubstanceSplitPaneUI ui) {
 		super(ui);
@@ -101,7 +101,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 			this.gripModel.setRollover(false);
 			this.gripModel.setEnabled(this.splitPane.isEnabled());
 
-			// fix for defect 109 - memory leak on changing theme
+			// fix for defect 109 - memory leak on changing skin
 			this.substanceRolloverListener = new RolloverControlListener(this,
 					this.gripModel);
 			this.addMouseListener(this.substanceRolloverListener);
@@ -141,7 +141,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 	 */
 	private void uninstall() {
 		// uninstalling
-		// fix for defect 109 - memory leak on changing theme
+		// fix for defect 109 - memory leak on changing skin
 		this.removeMouseListener(this.substanceRolloverListener);
 		this.removeMouseMotionListener(this.substanceRolloverListener);
 		this.substanceRolloverListener = null;
@@ -179,6 +179,14 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 				this.splitPane);
 		ComponentState prevState = SubstanceCoreUtilities
 				.getPrevComponentState(this.splitPane);
+		float alpha = SubstanceColorSchemeUtilities.getAlpha(this.splitPane,
+				currState);
+
+		if (alpha < 1.0f) {
+			graphics.setComposite(TransitionLayout.getAlphaComposite(
+					this.splitPane, alpha, g));
+		}
+
 		FadeState fadeState = SubstanceFadeUtilities.getFadeState(this,
 				FadeKind.ROLLOVER, FadeKind.SELECTION, FadeKind.PRESS);
 		float cyclePos = 1.0f;
@@ -208,9 +216,13 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 				// draw the grip bumps
 				SubstanceImageCreator.paintSplitDividerBumpImage(graphics,
 						this, gripX, gripY, thumbWidth, gripHeight, false,
-						currState, SubstanceColorSchemeUtilities.getColorScheme(this,
-								currState), SubstanceColorSchemeUtilities
-								.getColorScheme(this, prevState), cyclePos);
+						currState, SubstanceColorSchemeUtilities
+								.getColorScheme(this,
+										ColorSchemeAssociationKind.MARK,
+										currState),
+						SubstanceColorSchemeUtilities.getColorScheme(this,
+								ColorSchemeAssociationKind.MARK, prevState),
+						cyclePos);
 			}
 		} else {
 			int thumbWidth = this.getWidth();
@@ -227,9 +239,13 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 				// draw the grip bumps
 				SubstanceImageCreator.paintSplitDividerBumpImage(graphics,
 						this, gripX, gripY, gripWidth, thumbHeight, true,
-						currState, SubstanceColorSchemeUtilities.getColorScheme(this,
-								currState), SubstanceColorSchemeUtilities
-								.getColorScheme(this, prevState), cyclePos);
+						currState, SubstanceColorSchemeUtilities
+								.getColorScheme(this,
+										ColorSchemeAssociationKind.MARK,
+										currState),
+						SubstanceColorSchemeUtilities.getColorScheme(this,
+								ColorSchemeAssociationKind.MARK, prevState),
+						cyclePos);
 			}
 		}
 
@@ -267,7 +283,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 										.getArrowStrokeWidth(fontSize),
 								SwingConstants.NORTH, scheme);
 					}
-				});
+				}, "substance.splitPane.left.vertical");
 		Icon horizontalSplit = new TransitionAwareIcon(oneTouchButton,
 				new TransitionAwareIcon.Delegate() {
 					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
@@ -282,7 +298,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 										.getArrowStrokeWidth(fontSize),
 								SwingConstants.WEST, scheme);
 					}
-				});
+				}, "substance.splitPane.left.horizontal");
 		oneTouchButton
 				.setIcon(this.splitPane.getOrientation() == JSplitPane.VERTICAL_SPLIT ? verticalSplit
 						: horizontalSplit);
@@ -330,7 +346,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 										.getArrowStrokeWidth(fontSize),
 								SwingConstants.SOUTH, scheme);
 					}
-				});
+				}, "substance.splitPane.right.vertical");
 		Icon horizontalSplit = new TransitionAwareIcon(oneTouchButton,
 				new TransitionAwareIcon.Delegate() {
 					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
@@ -345,7 +361,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 										.getArrowStrokeWidth(fontSize),
 								SwingConstants.EAST, scheme);
 					}
-				});
+				}, "substance.splitPane.right.horizontal");
 		oneTouchButton
 				.setIcon(this.splitPane.getOrientation() == JSplitPane.VERTICAL_SPLIT ? verticalSplit
 						: horizontalSplit);
@@ -369,7 +385,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 	 * Updates the one-touch buttons.
 	 * 
 	 * @param orientation
-	 * 		Split pane orientation.
+	 *            Split pane orientation.
 	 */
 	public void updateOneTouchButtons(int orientation) {
 		if (orientation == JSplitPane.VERTICAL_SPLIT) {
@@ -390,7 +406,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 														.getArrowStrokeWidth(fontSize),
 												SwingConstants.NORTH, scheme);
 							}
-						}));
+						}, "substance.splitPane.left.vertical"));
 			}
 			if (this.rightButton != null) {
 				this.rightButton.setIcon(new TransitionAwareIcon(
@@ -409,7 +425,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 														.getArrowStrokeWidth(fontSize),
 												SwingConstants.SOUTH, scheme);
 							}
-						}));
+						}, "substance.splitPane.right.vertical"));
 			}
 		} else {
 			if (this.leftButton != null) {
@@ -429,7 +445,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 														.getArrowStrokeWidth(fontSize),
 												SwingConstants.WEST, scheme);
 							}
-						}));
+						}, "substance.splitPane.left.horizontal"));
 			}
 			if (this.rightButton != null) {
 				this.rightButton.setIcon(new TransitionAwareIcon(
@@ -448,7 +464,7 @@ public class SubstanceSplitPaneDivider extends BasicSplitPaneDivider implements
 														.getArrowStrokeWidth(fontSize),
 												SwingConstants.EAST, scheme);
 							}
-						}));
+						}, "substance.splitPane.right.horizontal"));
 			}
 		}
 	}
