@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -42,13 +42,11 @@ import java.util.*;
 import javax.swing.*;
 import javax.swing.plaf.UIResource;
 
-import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.animation.effects.GhostPaintingUtils;
 import org.jvnet.lafwidget.utils.RenderingUtils;
 import org.jvnet.lafwidget.utils.TrackableThread;
 import org.jvnet.substance.*;
-import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.api.SubstanceConstants.SubstanceWidgetType;
 import org.jvnet.substance.painter.decoration.DecorationAreaType;
 import org.jvnet.substance.painter.decoration.SubstanceDecorationUtilities;
@@ -165,6 +163,11 @@ public class SubstanceTitlePane extends JComponent {
 	 */
 	protected static final String EXTRA_COMPONENT_KIND = "substancelaf.internal.titlePane.extraComponentKind";
 
+	/**
+	 * The application icon to be displayed.
+	 */
+	protected Image appIcon;
+
 	/**
 	 * Enumerates the types of children components.
 	 * 
@@ -503,7 +506,7 @@ public class SubstanceTitlePane extends JComponent {
 
 		// Swing bug (?) - the updateComponentTree never gets to the
 		// system menu (and in our case we have radio menu items with
-		// rollover listeners). Fix for defect 109 - memory leak on theme
+		// rollover listeners). Fix for defect 109 - memory leak on skin
 		// switch
 		if ((this.menuBar != null) && (this.menuBar.getMenuCount() > 0)) {
 			this.menuBar.getUI().uninstallUI(this.menuBar);
@@ -536,7 +539,7 @@ public class SubstanceTitlePane extends JComponent {
 
 		// Property change listener for pulsating close button
 		// when window has been marked as changed.
-		// Fix for defect 109 - memory leak on theme change.
+		// Fix for defect 109 - memory leak on skin change.
 		this.propertyListener = new PropertyChangeListener() {
 			public void propertyChange(final PropertyChangeEvent evt) {
 				if (SubstanceLookAndFeel.WINDOW_MODIFIED.equals(evt
@@ -593,7 +596,7 @@ public class SubstanceTitlePane extends JComponent {
 			this.propertyChangeListener = null;
 		}
 
-		// Fix for defect 109 - memory leak on theme change.
+		// Fix for defect 109 - memory leak on skin change.
 		this.rootPane.removePropertyChangeListener(this.propertyListener);
 		if (this.getFrame() != null)
 			this.getFrame().removePropertyChangeListener(this.propertyListener);
@@ -646,6 +649,7 @@ public class SubstanceTitlePane extends JComponent {
 			this.installListeners();
 		}
 		this.setToolTipText(this.getTitle());
+		this.updateAppIcon();
 	}
 
 	/*
@@ -852,10 +856,16 @@ public class SubstanceTitlePane extends JComponent {
 		Icon closeIcon = new TransitionAwareIcon(closeButton,
 				new TransitionAwareIcon.Delegate() {
 					public Icon getColorSchemeIcon(SubstanceColorScheme scheme) {
-						return SubstanceIconFactory.getTitlePaneIcon(
-								SubstanceIconFactory.IconKind.CLOSE, scheme);
+						return SubstanceIconFactory
+								.getTitlePaneIcon(
+										SubstanceIconFactory.IconKind.CLOSE,
+										scheme,
+										SubstanceCoreUtilities
+												.getSkin(rootPane)
+												.getBackgroundColorScheme(
+														DecorationAreaType.PRIMARY_TITLE_PANE));
 					}
-				});
+				}, "substance.titlePane.closeIcon");
 		this.closeButton.setIcon(closeIcon);
 
 		this.closeButton.setFocusable(false);
@@ -876,11 +886,16 @@ public class SubstanceTitlePane extends JComponent {
 					new TransitionAwareIcon.Delegate() {
 						public Icon getColorSchemeIcon(
 								SubstanceColorScheme scheme) {
-							return SubstanceIconFactory.getTitlePaneIcon(
-									SubstanceIconFactory.IconKind.MINIMIZE,
-									scheme);
+							return SubstanceIconFactory
+									.getTitlePaneIcon(
+											SubstanceIconFactory.IconKind.MINIMIZE,
+											scheme,
+											SubstanceCoreUtilities
+													.getSkin(rootPane)
+													.getBackgroundColorScheme(
+															DecorationAreaType.PRIMARY_TITLE_PANE));
 						}
-					});
+					}, "substance.titlePane.minIcon");
 			this.minimizeButton.setIcon(minIcon);
 
 			this.minimizeButton.setFocusable(false);
@@ -900,11 +915,16 @@ public class SubstanceTitlePane extends JComponent {
 					new TransitionAwareIcon.Delegate() {
 						public Icon getColorSchemeIcon(
 								SubstanceColorScheme scheme) {
-							return SubstanceIconFactory.getTitlePaneIcon(
-									SubstanceIconFactory.IconKind.MAXIMIZE,
-									scheme);
+							return SubstanceIconFactory
+									.getTitlePaneIcon(
+											SubstanceIconFactory.IconKind.MAXIMIZE,
+											scheme,
+											SubstanceCoreUtilities
+													.getSkin(rootPane)
+													.getBackgroundColorScheme(
+															DecorationAreaType.PRIMARY_TITLE_PANE));
 						}
-					});
+					}, "substance.titlePane.maxIcon");
 			this.toggleButton.setIcon(maxIcon);
 
 			this.toggleButton.setToolTipText(SubstanceCoreUtilities
@@ -974,7 +994,7 @@ public class SubstanceTitlePane extends JComponent {
 			Frame frame = this.getFrame();
 
 			if (frame != null) {
-				JRootPane rootPane = this.getRootPane();
+				final JRootPane rootPane = this.getRootPane();
 
 				if (((state & Frame.MAXIMIZED_BOTH) != 0)
 						&& ((rootPane.getBorder() == null) || (rootPane
@@ -998,9 +1018,14 @@ public class SubstanceTitlePane extends JComponent {
 										return SubstanceIconFactory
 												.getTitlePaneIcon(
 														SubstanceIconFactory.IconKind.RESTORE,
-														scheme);
+														scheme,
+														SubstanceCoreUtilities
+																.getSkin(
+																		rootPane)
+																.getBackgroundColorScheme(
+																		DecorationAreaType.PRIMARY_TITLE_PANE));
 									}
-								});
+								}, "substance.titlePane.restoreIcon");
 						this
 								.updateToggleButton(this.restoreAction,
 										restoreIcon);
@@ -1018,9 +1043,14 @@ public class SubstanceTitlePane extends JComponent {
 										return SubstanceIconFactory
 												.getTitlePaneIcon(
 														SubstanceIconFactory.IconKind.MAXIMIZE,
-														scheme);
+														scheme,
+														SubstanceCoreUtilities
+																.getSkin(
+																		rootPane)
+																.getBackgroundColorScheme(
+																		DecorationAreaType.PRIMARY_TITLE_PANE));
 									}
-								});
+								}, "substance.titlePane.maxIcon");
 						this.updateToggleButton(this.maximizeAction, maxIcon);
 						this.toggleButton.setToolTipText(SubstanceCoreUtilities
 								.getResourceBundle(rootPane).getString(
@@ -1139,9 +1169,14 @@ public class SubstanceTitlePane extends JComponent {
 		int width = this.getWidth();
 		int height = this.getHeight();
 
-		SubstanceColorScheme scheme = SubstanceCoreUtilities.getSkin(rootPane)
-				.getMainDefaultColorScheme(
-						DecorationAreaType.PRIMARY_TITLE_PANE);
+		SubstanceSkin skin = SubstanceCoreUtilities.getSkin(rootPane);
+		if (skin == null) {
+			SubstanceCoreUtilities
+					.traceSubstanceApiUsage(this,
+							"Substance delegate used when Substance is not the current LAF");
+		}
+		SubstanceColorScheme scheme = skin
+				.getMainDefaultColorScheme(DecorationAreaType.PRIMARY_TITLE_PANE);
 
 		int xOffset = 0;
 		String theTitle = this.getTitle();
@@ -1287,9 +1322,12 @@ public class SubstanceTitlePane extends JComponent {
 		 */
 		public CloseAction() {
 			super(SubstanceCoreUtilities.getResourceBundle(rootPane).getString(
-					"SystemMenu.close"), SubstanceImageCreator
-					.getCloseIcon(SubstanceCoreUtilities.getSkin(rootPane)
+					"SystemMenu.close"), SubstanceImageCreator.getCloseIcon(
+					SubstanceCoreUtilities.getSkin(rootPane)
 							.getMainActiveColorScheme(
+									DecorationAreaType.PRIMARY_TITLE_PANE),
+					SubstanceCoreUtilities.getSkin(rootPane)
+							.getBackgroundColorScheme(
 									DecorationAreaType.PRIMARY_TITLE_PANE)));
 		}
 
@@ -1311,11 +1349,19 @@ public class SubstanceTitlePane extends JComponent {
 		 * Creates a new iconify action.
 		 */
 		public IconifyAction() {
-			super(SubstanceCoreUtilities.getResourceBundle(rootPane).getString(
-					"SystemMenu.iconify"), SubstanceImageCreator
-					.getMinimizeIcon(SubstanceCoreUtilities.getSkin(rootPane)
-							.getMainActiveColorScheme(
-									DecorationAreaType.PRIMARY_TITLE_PANE)));
+			super(
+					SubstanceCoreUtilities.getResourceBundle(rootPane)
+							.getString("SystemMenu.iconify"),
+					SubstanceImageCreator
+							.getMinimizeIcon(
+									SubstanceCoreUtilities
+											.getSkin(rootPane)
+											.getMainActiveColorScheme(
+													DecorationAreaType.PRIMARY_TITLE_PANE),
+									SubstanceCoreUtilities
+											.getSkin(rootPane)
+											.getBackgroundColorScheme(
+													DecorationAreaType.PRIMARY_TITLE_PANE)));
 		}
 
 		public void actionPerformed(ActionEvent e) {
@@ -1335,11 +1381,19 @@ public class SubstanceTitlePane extends JComponent {
 		 * Creates a new restore action.
 		 */
 		public RestoreAction() {
-			super(SubstanceCoreUtilities.getResourceBundle(rootPane).getString(
-					"SystemMenu.restore"), SubstanceImageCreator
-					.getRestoreIcon(SubstanceCoreUtilities.getSkin(rootPane)
-							.getMainActiveColorScheme(
-									DecorationAreaType.PRIMARY_TITLE_PANE)));
+			super(
+					SubstanceCoreUtilities.getResourceBundle(rootPane)
+							.getString("SystemMenu.restore"),
+					SubstanceImageCreator
+							.getRestoreIcon(
+									SubstanceCoreUtilities
+											.getSkin(rootPane)
+											.getMainActiveColorScheme(
+													DecorationAreaType.PRIMARY_TITLE_PANE),
+									SubstanceCoreUtilities
+											.getSkin(rootPane)
+											.getBackgroundColorScheme(
+													DecorationAreaType.PRIMARY_TITLE_PANE)));
 		}
 
 		public void actionPerformed(ActionEvent e) {
@@ -1367,11 +1421,19 @@ public class SubstanceTitlePane extends JComponent {
 		 * Creates a new maximize action.
 		 */
 		public MaximizeAction() {
-			super(SubstanceCoreUtilities.getResourceBundle(rootPane).getString(
-					"SystemMenu.maximize"), SubstanceImageCreator
-					.getMaximizeIcon(SubstanceCoreUtilities.getSkin(rootPane)
-							.getMainActiveColorScheme(
-									DecorationAreaType.PRIMARY_TITLE_PANE)));
+			super(
+					SubstanceCoreUtilities.getResourceBundle(rootPane)
+							.getString("SystemMenu.maximize"),
+					SubstanceImageCreator
+							.getMaximizeIcon(
+									SubstanceCoreUtilities
+											.getSkin(rootPane)
+											.getMainActiveColorScheme(
+													DecorationAreaType.PRIMARY_TITLE_PANE),
+									SubstanceCoreUtilities
+											.getSkin(rootPane)
+											.getMainDefaultColorScheme(
+													DecorationAreaType.PRIMARY_TITLE_PANE)));
 		}
 
 		public void actionPerformed(ActionEvent e) {
@@ -1395,32 +1457,10 @@ public class SubstanceTitlePane extends JComponent {
 	public class SubstanceMenuBar extends JMenuBar {
 		@Override
 		public void paint(Graphics g) {
-			Frame frame = SubstanceTitlePane.this.getFrame();
-
-			Image image = (frame != null) ? frame.getIconImage() : null;
-
-			if (image != null) {
-				int iSize = SubstanceSizeUtils.getTitlePaneIconSize();
-				double coef = Math.max((double) iSize
-						/ (double) image.getWidth(null), (double) iSize
-						/ (double) image.getHeight(null));
-				if (coef < 1.0) {
-					// fix for defect 255 - large icons need to be properly
-					// scaled down. While we can use
-					// RenderingHints.VALUE_INTERPOLATION_BILINEAR, it will not
-					// work good on large icons (such as 128*128), resulting in
-					// bad images. Here, we use multi-step scaling by Romain
-					// Guy.
-					BufferedImage bi = SubstanceCoreUtilities.getBlankImage(
-							image.getWidth(null), image.getHeight(null));
-					bi.getGraphics().drawImage(image, 0, 0, null);
-					g.drawImage(LafWidgetUtilities.createThumbnail(bi, iSize),
-							0, 0, null);
-				} else
-					g.drawImage(image, 0, 0, null);
+			if (appIcon != null) {
+				g.drawImage(appIcon, 0, 0, null);
 			} else {
 				Icon icon = UIManager.getIcon("InternalFrame.icon");
-
 				if (icon != null) {
 					icon.paintIcon(this, g, 0, 0);
 				}
@@ -1544,10 +1584,11 @@ public class SubstanceTitlePane extends JComponent {
 			if (SubstanceTitlePane.this.menuBar != null) {
 				SubstanceTitlePane.this.menuBar.setBounds(x, y, buttonWidth,
 						buttonHeight);
+				// System.out.println(menuBar.getBounds());
 			}
 
 			x = leftToRight ? w : 0;
-			spacing = 4;
+			spacing = 3;
 			x += leftToRight ? -spacing - buttonWidth : spacing;
 			if (SubstanceTitlePane.this.closeButton != null) {
 				SubstanceTitlePane.this.closeButton.setBounds(x, y,
@@ -1623,12 +1664,13 @@ public class SubstanceTitlePane extends JComponent {
 					SubstanceTitlePane.this.repaint();
 					SubstanceTitlePane.this.setToolTipText((String) pce
 							.getNewValue());
-				} else {
-					if ("componentOrientation".equals(name)
-							|| "iconImage".equals(name)) {
-						SubstanceTitlePane.this.revalidate();
-						SubstanceTitlePane.this.repaint();
-					}
+				} else if ("componentOrientation" == name) {
+					revalidate();
+					repaint();
+				} else if ("iconImage" == name) {
+					updateAppIcon();
+					revalidate();
+					repaint();
 				}
 			}
 		}
@@ -1692,4 +1734,25 @@ public class SubstanceTitlePane extends JComponent {
 	protected void markExtraComponent(JComponent comp, ExtraComponentKind kind) {
 		comp.putClientProperty(EXTRA_COMPONENT_KIND, kind);
 	}
+
+	/**
+	 * Updates the application icon.
+	 */
+	private void updateAppIcon() {
+		Window window = getWindow();
+		if (window == null) {
+			this.appIcon = null;
+			return;
+		}
+		java.util.List<Image> icons = window.getIconImages();
+
+		if (icons.size() == 0) {
+			this.appIcon = null;
+		} else {
+			int prefSize = SubstanceSizeUtils.getTitlePaneIconSize();
+			this.appIcon = SubstanceCoreUtilities.getScaledIconImage(icons,
+					prefSize, prefSize);
+		}
+	}
+
 }
