@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -44,8 +44,7 @@ import javax.swing.plaf.basic.BasicSliderUI;
 
 import org.jvnet.lafwidget.animation.*;
 import org.jvnet.substance.*;
-import org.jvnet.substance.api.ComponentState;
-import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.*;
 import org.jvnet.substance.painter.border.SubstanceBorderPainter;
 import org.jvnet.substance.painter.gradient.SubstanceGradientPainter;
 import org.jvnet.substance.utils.*;
@@ -81,19 +80,6 @@ public class SubstanceIconFactory {
 	private static LazyResettableHashMap<Icon> treeIcons = new LazyResettableHashMap<Icon>(
 			"SubstanceIconFactory.treeIcon");
 
-	/**
-	 * Resets image maps (used when setting new skin).
-	 * 
-	 * @see SubstanceLookAndFeel#setSkin(String)
-	 * @see SubstanceLookAndFeel#setSkin(org.jvnet.substance.skin.SubstanceSkin)
-	 */
-	public static synchronized void reset() {
-		SubstanceIconFactory.titlePaneIcons.get(IconKind.CLOSE).clear();
-		SubstanceIconFactory.titlePaneIcons.get(IconKind.MINIMIZE).clear();
-		SubstanceIconFactory.titlePaneIcons.get(IconKind.MAXIMIZE).clear();
-		SubstanceIconFactory.titlePaneIcons.get(IconKind.RESTORE).clear();
-	}
-
 	/**
 	 * Retrieves icon for horizontal slider in {@link SubstanceSliderUI}.
 	 * 
@@ -288,9 +274,11 @@ public class SubstanceIconFactory {
 			SubstanceColorScheme colorScheme2 = SubstanceColorSchemeUtilities
 					.getColorScheme(slider, prevState);
 			SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-					.getBorderColorScheme(slider, state);
+					.getColorScheme(slider, ColorSchemeAssociationKind.BORDER,
+							state);
 			SubstanceColorScheme borderScheme2 = SubstanceColorSchemeUtilities
-					.getBorderColorScheme(slider, prevState);
+					.getColorScheme(slider, ColorSchemeAssociationKind.BORDER,
+							prevState);
 
 			FadeTracker fadeTracker = FadeTracker.getInstance();
 			int width = 2 * sliderIcon.size / 3;
@@ -501,9 +489,11 @@ public class SubstanceIconFactory {
 			SubstanceColorScheme colorScheme2 = SubstanceColorSchemeUtilities
 					.getColorScheme(slider, prevState);
 			SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-					.getBorderColorScheme(slider, state);
+					.getColorScheme(slider, ColorSchemeAssociationKind.BORDER,
+							state);
 			SubstanceColorScheme borderScheme2 = SubstanceColorSchemeUtilities
-					.getBorderColorScheme(slider, prevState);
+					.getColorScheme(slider, ColorSchemeAssociationKind.BORDER,
+							prevState);
 
 			FadeTracker fadeTracker = FadeTracker.getInstance();
 			int width = 2 * sliderIcon.size / 3;
@@ -715,9 +705,11 @@ public class SubstanceIconFactory {
 			SubstanceColorScheme colorScheme2 = SubstanceColorSchemeUtilities
 					.getColorScheme(slider, prevState);
 			SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
-					.getBorderColorScheme(slider, state);
+					.getColorScheme(slider, ColorSchemeAssociationKind.BORDER,
+							state);
 			SubstanceColorScheme borderScheme2 = SubstanceColorSchemeUtilities
-					.getBorderColorScheme(slider, prevState);
+					.getColorScheme(slider, ColorSchemeAssociationKind.BORDER,
+							prevState);
 
 			FadeTracker fadeTracker = FadeTracker.getInstance();
 			int height = 2 * sliderIcon.size / 3;
@@ -919,20 +911,24 @@ public class SubstanceIconFactory {
 		private static Icon getIcon(JTree tree, boolean isCollapsed) {
 			ComponentState state = ((tree == null) || tree.isEnabled()) ? ComponentState.DEFAULT
 					: ComponentState.DISABLED_UNSELECTED;
-			SubstanceColorScheme scheme = SubstanceColorSchemeUtilities
+			SubstanceColorScheme fillScheme = SubstanceColorSchemeUtilities
 					.getColorScheme(tree, state);
+			SubstanceColorScheme borderScheme = SubstanceColorSchemeUtilities
+					.getColorScheme(tree, ColorSchemeAssociationKind.BORDER,
+							state);
 
 			int fontSize = SubstanceSizeUtils.getComponentFontSize(tree);
 
-			HashMapKey key = SubstanceCoreUtilities.getHashKey(fontSize, scheme
-					.getDisplayName(), isCollapsed);
+			HashMapKey key = SubstanceCoreUtilities.getHashKey(fontSize,
+					fillScheme.getDisplayName(), borderScheme.getDisplayName(),
+					isCollapsed);
 
 			Icon result = TreeIcon.icons.get(key);
 			if (result != null)
 				return result;
 
 			result = new ImageIcon(SubstanceImageCreator.getTreeIcon(tree,
-					scheme, isCollapsed));
+					fillScheme, borderScheme, isCollapsed));
 			TreeIcon.icons.put(key, result);
 
 			return result;
@@ -1008,7 +1004,7 @@ public class SubstanceIconFactory {
 	/**
 	 * Cache of title pane icons.
 	 */
-	private static final Map<IconKind, Map<SubstanceColorScheme, Icon>> titlePaneIcons = SubstanceIconFactory
+	private static final Map<IconKind, LazyResettableHashMap<Icon>> titlePaneIcons = SubstanceIconFactory
 			.createTitlePaneIcons();
 
 	/**
@@ -1016,17 +1012,17 @@ public class SubstanceIconFactory {
 	 * 
 	 * @return Empty map of title pane icons.
 	 */
-	private static Map<IconKind, Map<SubstanceColorScheme, Icon>> createTitlePaneIcons() {
-		Map<IconKind, Map<SubstanceColorScheme, Icon>> result = new HashMap<IconKind, Map<SubstanceColorScheme, Icon>>();
+	private static Map<IconKind, LazyResettableHashMap<Icon>> createTitlePaneIcons() {
+		Map<IconKind, LazyResettableHashMap<Icon>> result = new HashMap<IconKind, LazyResettableHashMap<Icon>>();
 
-		result.put(IconKind.CLOSE, new HashMap<SubstanceColorScheme, Icon>());
-		result
-				.put(IconKind.MINIMIZE,
-						new HashMap<SubstanceColorScheme, Icon>());
-		result
-				.put(IconKind.MAXIMIZE,
-						new HashMap<SubstanceColorScheme, Icon>());
-		result.put(IconKind.RESTORE, new HashMap<SubstanceColorScheme, Icon>());
+		result.put(IconKind.CLOSE, new LazyResettableHashMap<Icon>(
+				"Close title pane icons"));
+		result.put(IconKind.MINIMIZE, new LazyResettableHashMap<Icon>(
+				"Minimize title pane icons"));
+		result.put(IconKind.MAXIMIZE, new LazyResettableHashMap<Icon>(
+				"Maximize title pane icons"));
+		result.put(IconKind.RESTORE, new LazyResettableHashMap<Icon>(
+				"Restore title pane icons"));
 		return result;
 	}
 
@@ -1036,32 +1032,40 @@ public class SubstanceIconFactory {
 	 * @param iconKind
 	 *            Icon kind.
 	 * @param scheme
-	 *            Substance theme.
+	 *            Color scheme.
 	 * @return Title pane icon of the specified kind.
 	 */
 	public static Icon getTitlePaneIcon(IconKind iconKind,
-			SubstanceColorScheme scheme) {
-		Map<SubstanceColorScheme, Icon> kindMap = SubstanceIconFactory.titlePaneIcons
+			SubstanceColorScheme scheme, SubstanceColorScheme backgroundScheme) {
+
+		LazyResettableHashMap<Icon> kindMap = SubstanceIconFactory.titlePaneIcons
 				.get(iconKind);
-		Icon result = kindMap.get(scheme);
+
+		HashMapKey key = SubstanceCoreUtilities.getHashKey(scheme
+				.getDisplayName(), backgroundScheme.getDisplayName());
+		Icon result = kindMap.get(key);
 		if (result != null)
 			return result;
 
 		switch (iconKind) {
 		case CLOSE:
-			result = SubstanceImageCreator.getCloseIcon(scheme);
+			result = SubstanceImageCreator.getCloseIcon(scheme,
+					backgroundScheme);
 			break;
 		case MINIMIZE:
-			result = SubstanceImageCreator.getMinimizeIcon(scheme);
+			result = SubstanceImageCreator.getMinimizeIcon(scheme,
+					backgroundScheme);
 			break;
 		case MAXIMIZE:
-			result = SubstanceImageCreator.getMaximizeIcon(scheme);
+			result = SubstanceImageCreator.getMaximizeIcon(scheme,
+					backgroundScheme);
 			break;
 		case RESTORE:
-			result = SubstanceImageCreator.getRestoreIcon(scheme);
+			result = SubstanceImageCreator.getRestoreIcon(scheme,
+					backgroundScheme);
 			break;
 		}
-		kindMap.put(scheme, result);
+		kindMap.put(key, result);
 		return result;
 	}
 
