@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2008 Substance Kirill Grouchnikov. All Rights Reserved.
+ * Copyright (c) 2005-2009 Substance Kirill Grouchnikov. All Rights Reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
@@ -40,6 +40,8 @@ import javax.swing.plaf.ButtonUI;
 import javax.swing.plaf.basic.BasicHTML;
 import javax.swing.text.View;
 
+import org.jvnet.lafwidget.animation.FadeKind;
+import org.jvnet.lafwidget.animation.FadeState;
 import org.jvnet.lafwidget.layout.TransitionLayout;
 import org.jvnet.substance.SubstanceLookAndFeel;
 import org.jvnet.substance.api.ComponentState;
@@ -745,7 +747,46 @@ public class MenuUtilities {
 			}
 
 			if (icon != null) {
-				icon.paintIcon(menuItem, g2d, mli.iconRect.x, mli.iconRect.y);
+				boolean useThemed = SubstanceCoreUtilities
+						.useThemedDefaultIcon();
+
+				if (!useThemed) {
+					icon.paintIcon(menuItem, g2d, mli.iconRect.x,
+							mli.iconRect.y);
+				} else {
+					Icon themed = SubstanceCoreUtilities.getThemedIcon(
+							menuItem, icon);
+					boolean useRegularVersion = model.isPressed()
+							|| model.isSelected();
+					if (useRegularVersion) {
+						icon.paintIcon(menuItem, g2d, mli.iconRect.x,
+								mli.iconRect.y);
+					} else {
+						FadeState state = SubstanceFadeUtilities.getFadeState(
+								menuItem, FadeKind.ARM, FadeKind.ROLLOVER);
+						if (state != null) {
+							float alpha = state.getFadePosition();
+							themed.paintIcon(menuItem, g2d, mli.iconRect.x,
+									mli.iconRect.y);
+							g2d.setComposite(TransitionLayout
+									.getAlphaComposite(menuItem, alpha, g));
+							// System.out.println("Themed to regular at " +
+							// alpha);
+							icon.paintIcon(menuItem, g2d, mli.iconRect.x,
+									mli.iconRect.y);
+							g2d.setComposite(TransitionLayout
+									.getAlphaComposite(menuItem, g));
+						} else {
+							if (model.isRollover() || model.isArmed()) {
+								icon.paintIcon(menuItem, g2d, mli.iconRect.x,
+										mli.iconRect.y);
+							} else {
+								themed.paintIcon(menuItem, g2d, mli.iconRect.x,
+										mli.iconRect.y);
+							}
+						}
+					}
+				}
 			}
 		}
 
