@@ -41,18 +41,18 @@ import javax.swing.event.ChangeListener;
 import javax.swing.plaf.FontUIResource;
 
 import org.jdesktop.swingx.*;
+import org.jvnet.lafwidget.LAFAdapter;
 import org.jvnet.lafwidget.LafWidget;
 import org.jvnet.lafwidget.tabbed.DefaultTabPreviewPainter;
 import org.jvnet.lafwidget.utils.LafConstants.TabOverviewKind;
 import org.jvnet.substance.SubstanceLookAndFeel;
-import org.jvnet.substance.api.SubstanceConstants;
 import org.jvnet.substance.api.SubstanceConstants.Side;
 import org.jvnet.substance.api.SubstanceConstants.TabCloseKind;
 import org.jvnet.substance.api.tabbed.*;
 import org.jvnet.substance.fonts.FontSet;
 import org.jvnet.substance.painter.decoration.DecorationAreaType;
 import org.jvnet.substance.skin.SkinChangeListener;
-import org.jvnet.substance.skin.SubstanceBusinessBlackSteelLookAndFeel;
+import org.jvnet.substance.skin.SubstanceTwilightLookAndFeel;
 import org.jvnet.substance.utils.SubstanceImageCreator;
 
 import test.check.*;
@@ -134,7 +134,7 @@ public class Check extends JFrame {
 					new ImageIcon(cl
 							.getResource("test/resource/image-x-generic.png")),
 					SubstanceLookAndFeel.getCurrentSkin(this.getRootPane())
-							.getMainActiveColorScheme()));
+							.getMainActiveColorScheme(), 0.0f));
 		}
 		SubstanceLookAndFeel
 				.registerSkinChangeListener(new SkinChangeListener() {
@@ -151,7 +151,8 @@ public class Check extends JFrame {
 																.getResource("test/resource/image-x-generic.png")),
 												SubstanceLookAndFeel
 														.getCurrentSkin(null)
-														.getMainActiveColorScheme()));
+														.getMainActiveColorScheme(),
+												0.0f));
 							}
 						});
 					}
@@ -365,6 +366,8 @@ public class Check extends JFrame {
 
 		jtp.addTab("Colorized", new ColorizedControlsPanel());
 
+		jtp.addTab("Cells", new CellsPanel());
+
 		JPanel buttonStripPanel = new JPanel();
 		buttonStripPanel.setLayout(new FlowLayout());
 
@@ -627,11 +630,13 @@ public class Check extends JFrame {
 					// JNLP sandbox
 				}
 
+				LAFAdapter.startWidget();
+
 				try {
 					if (!hasLafSpecified) {
 						out(" CREATING LAF ");
 						long time0 = System.currentTimeMillis();
-						LookAndFeel laf = new SubstanceBusinessBlackSteelLookAndFeel();
+						LookAndFeel laf = new SubstanceTwilightLookAndFeel();
 						long time1 = System.currentTimeMillis();
 						out(" LAF CREATED " + (time1 - time0));
 						out(" SETTING LAF ");
@@ -645,7 +650,6 @@ public class Check extends JFrame {
 				}
 
 				SubstanceLookAndFeel.setToUseConstantThemesOnDialogs(true);
-
 				UIManager
 						.put(
 								SubstanceLookAndFeel.TABBED_PANE_CLOSE_BUTTONS_PROPERTY,
@@ -783,8 +787,8 @@ public class Check extends JFrame {
 			final JToggleButton buttonStyleStrikethrough = new JToggleButton(
 					getIcon(size + "/format-text-strikethrough"));
 			buttonStyleStrikethrough.putClientProperty(
-					SubstanceLookAndFeel.BUTTON_SIDE_PROPERTY,
-					SubstanceConstants.Side.LEFT.name());
+					SubstanceLookAndFeel.BUTTON_SIDE_PROPERTY, EnumSet
+							.of(Side.LEFT));
 			buttonStyleStrikethrough.putClientProperty(
 					SubstanceLookAndFeel.CORNER_RADIUS, Float.valueOf(3.0f));
 			buttonStyleBold.setSelected(true);
