@@ -12,12 +12,12 @@ import com.jgoodies.forms.layout.FormLayout;
 public class AlignmentPanel {
 
 	public static JComponent getAlignmentPanel() {
-		FormLayout lm = new FormLayout("right:pref, 4dlu, left:pref:grow", "");
+		FormLayout lm = new FormLayout("left:pref:grow", "");
 		DefaultFormBuilder builder = new DefaultFormBuilder(lm, new ScrollablePanel());
 		builder.setDefaultDialogBorder();
 
 		for (int fontSize = 11; fontSize < 25; fontSize++) {
-			builder.append("size " + fontSize, getSubPanel(fontSize));
+			builder.append(getSubPanel(fontSize));
 		}
 
 		return new JScrollPane(builder.getPanel());
@@ -28,13 +28,17 @@ public class AlignmentPanel {
 
 		Font font = new Font("Tahoma", Font.PLAIN, size);
 
+		JLabel label = new JLabel("Tahoma " + size);
+		label.setFont(font);
+		result.add(label);
+		
 		JTextField tf = new JTextField("sample");
 		tf.setFont(font);
 		result.add(tf);
 
-		JFormattedTextField ftf = new JFormattedTextField("sample");
-		ftf.setFont(font);
-		result.add(ftf);
+//		JFormattedTextField ftf = new JFormattedTextField("sample");
+//		ftf.setFont(font);
+//		result.add(ftf);
 
 		JPasswordField pf = new JPasswordField("sample");
 		pf.setFont(font);
