@@ -30,8 +30,6 @@
 package test.check;
 
 import java.awt.*;
-import java.awt.event.ActionEvent;
-import java.awt.event.ActionListener;
 import java.text.DecimalFormat;
 import java.util.Enumeration;
 
@@ -60,13 +58,13 @@ public class ColorizedControlsPanel extends JPanel {
 	public JButton defaultButton;
 
 	/**
-	 * Returns a row of buttons, consisting of {@link JButton},
-	 * {@link JToggleButton}, {@link JCheckBox} and {@link JRadioButton} in
-	 * default states.
+	 * Returns a row of buttons, consisting of {@link JButton}, {@link
+	 * JToggleButton}, {@link JCheckBox} and {@link JRadioButton} in default
+	 * states.
 	 * 
-	 * @return A row of buttons, consisting of {@link JButton},
-	 *         {@link JToggleButton}, {@link JCheckBox} and
-	 *         {@link JRadioButton} in default states.
+	 * @return A row of buttons, consisting of {@link JButton}, {@link
+	 * 	JToggleButton}, {@link JCheckBox} and {@link JRadioButton} in default
+	 * 	states.
 	 */
 	private AbstractButton[] getRow() {
 		AbstractButton[] result = new AbstractButton[4];
@@ -82,13 +80,13 @@ public class ColorizedControlsPanel extends JPanel {
 	 * configuration command.
 	 * 
 	 * @param builder
-	 *            Form builder.
+	 * 		Form builder.
 	 * @param label
-	 *            Text to set.
+	 * 		Text to set.
 	 * @param icon
-	 *            Icon to set.
+	 * 		Icon to set.
 	 * @param configurationCmd
-	 *            Configuration command to apply.
+	 * 		Configuration command to apply.
 	 */
 	private void addButtonRow(DefaultFormBuilder builder, String label,
 			Icon icon, ConfigurationCommand<Component> configurationCmd) {
@@ -113,13 +111,13 @@ public class ColorizedControlsPanel extends JPanel {
 	 * command.
 	 * 
 	 * @param builder
-	 *            Form builder.
+	 * 		Form builder.
 	 * @param label
-	 *            Text to set.
+	 * 		Text to set.
 	 * @param creationCmd
-	 *            Creation command.
+	 * 		Creation command.
 	 * @param configurationCmd
-	 *            Configuration command to apply.
+	 * 		Configuration command to apply.
 	 */
 	private void addControlRow(DefaultFormBuilder builder, String label,
 			CreationCommand<Component> creationCmd,
@@ -128,13 +126,17 @@ public class ColorizedControlsPanel extends JPanel {
 		Component[] row = new Component[4];
 		row[0] = creationCmd.create();
 		row[0].setBackground(Color.yellow);
+		row[0].setName(row[0].getName() + ": yellow backgr");
 		row[1] = creationCmd.create();
 		row[1].setBackground(Color.blue);
 		row[1].setForeground(Color.red);
+		row[1].setName(row[1].getName() + ": blue backgr, red foregr");
 		row[2] = creationCmd.create();
 		row[2].setBackground(Color.green);
+		row[2].setName(row[2].getName() + ": green backgr");
 		row[3] = creationCmd.create();
 		row[3].setBackground(Color.green.darker());
+		row[3].setName(row[3].getName() + ": darker green backgr");
 
 		if (configurationCmd != null) {
 			for (Component comp : row) {
@@ -393,7 +395,8 @@ public class ColorizedControlsPanel extends JPanel {
 		builderOther.setDefaultDialogBorder();
 
 		builderOther.append("");
-		builderOther.append(new JLabel("Yellow"), new JLabel("Blue"));
+		builderOther.append(new JLabel("Yellow"), new JLabel(
+				"Blue + red foreground"));
 		builderOther.append(new JLabel("Green"), new JLabel("Dark green"));
 
 		builderOther.appendSeparator("Labels");
@@ -409,19 +412,6 @@ public class ColorizedControlsPanel extends JPanel {
 						return new JLabel("label");
 					}
 				}, new DisableCommand());
-		addControlRow(builderOther, "Regular + foreground",
-				new CreationCommand<Component>() {
-					public Component create() {
-						return new JLabel("label");
-					}
-				}, new ForegroundColorCommand(Color.red));
-		addControlRow(builderOther, "Disabled + foreground",
-				new CreationCommand<Component>() {
-					public Component create() {
-						return new JLabel("label");
-					}
-				}, new ChainCommand<Component>(new ForegroundColorCommand(
-						Color.red), new DisableCommand()));
 		addControlRow(builderOther, "HTML", new CreationCommand<Component>() {
 			public Component create() {
 				return new JLabel(
@@ -578,7 +568,8 @@ public class ColorizedControlsPanel extends JPanel {
 		builderText.setDefaultDialogBorder();
 
 		builderText.append("");
-		builderText.append(new JLabel("Yellow"), new JLabel("Blue + red"));
+		builderText.append(new JLabel("Yellow"), new JLabel(
+				"Blue + red foreground"));
 		builderText.append(new JLabel("Green"), new JLabel("Dark green"));
 
 		builderText.appendSeparator("Text fields");
@@ -595,7 +586,7 @@ public class ColorizedControlsPanel extends JPanel {
 						return field;
 					}
 				}, new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderText, "Disabled",
 				new CreationCommand<Component>() {
 					public Component create() {
@@ -630,7 +621,7 @@ public class ColorizedControlsPanel extends JPanel {
 						return field;
 					}
 				}, new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderText, "Disabled",
 				new CreationCommand<Component>() {
 					public Component create() {
@@ -666,7 +657,7 @@ public class ColorizedControlsPanel extends JPanel {
 						return field;
 					}
 				}, new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderText, "Disabled",
 				new CreationCommand<Component>() {
 					public Component create() {
@@ -701,7 +692,7 @@ public class ColorizedControlsPanel extends JPanel {
 						return pane;
 					}
 				}, new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderText, "Disabled",
 				new CreationCommand<Component>() {
 					public Component create() {
@@ -735,7 +726,7 @@ public class ColorizedControlsPanel extends JPanel {
 						return area;
 					}
 				}, new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderText, "Disabled",
 				new CreationCommand<Component>() {
 					public Component create() {
@@ -772,7 +763,7 @@ public class ColorizedControlsPanel extends JPanel {
 						return pane;
 					}
 				}, new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderText, "Disabled",
 				new CreationCommand<Component>() {
 					public Component create() {
@@ -813,7 +804,8 @@ public class ColorizedControlsPanel extends JPanel {
 		builderCells.setDefaultDialogBorder();
 
 		builderCells.append("");
-		builderCells.append(new JLabel("Yellow"), new JLabel("Blue"));
+		builderCells.append(new JLabel("Yellow"), new JLabel(
+				"Blue + red foreground"));
 		builderCells.append(new JLabel("Green"), new JLabel("Dark green"));
 
 		builderCells.appendSeparator("Lists");
@@ -832,7 +824,7 @@ public class ColorizedControlsPanel extends JPanel {
 						return list;
 					}
 				}, new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderCells, "List disabled",
 				new CreationCommand<Component>() {
 					public Component create() {
@@ -849,11 +841,20 @@ public class ColorizedControlsPanel extends JPanel {
 						{ "entry21", "entry22", "entry23" },
 						{ "entry31", "entry32", "entry33" } }, new Object[] {
 						"Column1", "Column2", "Column3" });
+				table.setName("Table ");
 				JScrollPane tableScroll = new JScrollPane(table) {
 					@Override
 					public void setBackground(Color bg) {
 						super.setBackground(bg);
 						table.setBackground(bg);
+						table.getTableHeader().setBackground(bg);
+					}
+
+					@Override
+					public void setForeground(Color fg) {
+						super.setForeground(fg);
+						table.setForeground(fg);
+						table.getTableHeader().setForeground(fg);
 					}
 				};
 				Dimension prefTable = table.getPreferredSize();
@@ -866,7 +867,7 @@ public class ColorizedControlsPanel extends JPanel {
 		addControlRow(builderCells, "Table", tableCreationCmd, null);
 		addControlRow(builderCells, "Table watermark", tableCreationCmd,
 				new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderCells, "Table disabled", tableCreationCmd,
 				new DisableViewportCommand());
 
@@ -943,7 +944,7 @@ public class ColorizedControlsPanel extends JPanel {
 		addControlRow(builderCells, "Tree", treeCreationCmd, null);
 		addControlRow(builderCells, "Tree watermark", treeCreationCmd,
 				new ClientPropertyCommand(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean.TRUE));
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean.TRUE));
 		addControlRow(builderCells, "Tree disabled", treeCreationCmd,
 				new DisableCommand());
 
@@ -1046,30 +1047,8 @@ public class ColorizedControlsPanel extends JPanel {
 		this.add(tabbed, BorderLayout.CENTER);
 
 		JPanel controls = new JPanel(new FlowLayout(FlowLayout.LEFT));
-		final JCheckBox hasColorization = new JCheckBox("colorize");
 		final JSlider colorizationSlider = new JSlider(0, 100, 50);
-		colorizationSlider.setEnabled(false);
-		hasColorization.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				SwingUtilities.invokeLater(new Runnable() {
-					public void run() {
-						if (hasColorization.isSelected()) {
-							double val = colorizationSlider.getValue() / 100.0;
-							ColorizedControlsPanel.this.putClientProperty(
-									SubstanceLookAndFeel.COLORIZATION_FACTOR,
-									new Double(val));
-						} else {
-							ColorizedControlsPanel.this.putClientProperty(
-									SubstanceLookAndFeel.COLORIZATION_FACTOR,
-									null);
-						}
-						colorizationSlider.setEnabled(hasColorization
-								.isSelected());
-						tabbed.repaint();
-					}
-				});
-			}
-		});
+		this.putClientProperty(SubstanceLookAndFeel.COLORIZATION_FACTOR, 0.5);
 		colorizationSlider.addChangeListener(new ChangeListener() {
 			public void stateChanged(ChangeEvent e) {
 				double val = colorizationSlider.getValue() / 100.0;
@@ -1079,7 +1058,6 @@ public class ColorizedControlsPanel extends JPanel {
 				tabbed.repaint();
 			}
 		});
-		controls.add(hasColorization);
 		controls.add(colorizationSlider);
 		this.add(controls, BorderLayout.NORTH);
 	}
