@@ -32,21 +32,14 @@ package test.check;
 import java.awt.*;
 import java.awt.event.*;
 import java.io.IOException;
-import java.util.Map;
-import java.util.Vector;
 
 import javax.imageio.ImageIO;
 import javax.swing.*;
 
-import org.jvnet.lafwidget.LafWidget;
-import org.jvnet.lafwidget.combo.AutoCompletionMatcher;
 import org.jvnet.substance.*;
-import org.jvnet.substance.combo.ComboPopupPrototypeCallback;
-import org.jvnet.substance.combo.WidestComboPopupPrototype;
-import org.jvnet.substance.theme.SubstanceTheme;
-import org.jvnet.substance.theme.ThemeInfo;
+import org.jvnet.substance.api.combo.ComboPopupPrototypeCallback;
+import org.jvnet.substance.api.combo.WidestComboPopupPrototype;
 
-import test.Check;
 import test.check.command.*;
 
 import com.jgoodies.forms.builder.DefaultFormBuilder;
@@ -58,38 +51,6 @@ import com.jgoodies.forms.layout.FormLayout;
  * @author Kirill Grouchnikov
  */
 public class CombosPanel extends ControllablePanel {
-	/**
-	 * Renderer for the combobox containing the theme list.
-	 * 
-	 * @author Kirill Grouchnikov
-	 */
-	protected static class ThemeComboRenderer extends JLabel implements
-			ListCellRenderer {
-		/**
-		 * The current theme.
-		 */
-		protected SubstanceTheme currTheme;
-
-		public Component getListCellRendererComponent(JList list, Object value,
-				int index, boolean isSelected, boolean cellHasFocus) {
-			SubstanceTheme theme = (SubstanceTheme) value;
-			this.setBackground(theme.getColorScheme().getMidColor());
-			this.setForeground(theme.getColorScheme().getForegroundColor());
-			this.setText(theme.getDisplayName());
-			this.currTheme = theme;
-			return this;
-		}
-
-		@Override
-		public void paint(Graphics g) {
-			int width = this.getWidth();
-			int height = this.getHeight();
-			SubstanceImageCreator.paintRectangularBackground(g, 0, 0, width,
-					height, this.currTheme.getColorScheme(), 0.0f, false);
-			super.paint(g);
-		}
-	}
-
 	/**
 	 * A configure command that allows editing the specified combobox.
 	 * 
@@ -276,14 +237,6 @@ public class CombosPanel extends ControllablePanel {
 		comboRegular.setMaximumRowCount(4);
 		builder.append("Regular", comboRegular);
 
-		JComboBox comboActive = new JComboBox(new Object[] { "entry1",
-				"entry2", "entry3", "entry4", "entry5", "entry6" });
-		comboActive.setToolTipText("This combo is active");
-		comboActive.setMaximumRowCount(4);
-		comboActive.putClientProperty(
-				SubstanceLookAndFeel.PAINT_ACTIVE_PROPERTY, Boolean.TRUE);
-		builder.append("Active", comboActive);
-
 		JComboBox comboDisabled = new JComboBox(new Object[] { "entry1",
 				"entry2", "entry3" });
 		comboDisabled.setEnabled(false);
@@ -297,24 +250,22 @@ public class CombosPanel extends ControllablePanel {
 		comboColored.setForeground(new Color(0, 0, 128));
 		builder.append("Pink background", comboColored);
 
-		JComboBox comboColors = new JComboBox(new Object[] {
-				new Color(255, 128, 128), new Color(128, 255, 128),
-				new Color(128, 128, 255), new Color(255, 255, 128),
-				new Color(255, 128, 255), new Color(128, 255, 255), });
-		comboColors.setName("Colors combo");
-		comboColors.setRenderer(new SubstanceDefaultListCellRenderer() {
-			public Component getListCellRendererComponent(JList list,
-					Object value, int index, boolean isSelected,
-					boolean cellHasFocus) {
-				JLabel comp = (JLabel) super.getListCellRendererComponent(list,
-						value, index, isSelected, cellHasFocus);
-				Color color = (Color) value;
-				comp.setBackground(color);
-				comp.setText(color.getRed() + ":" + color.getGreen() + ":"
-						+ color.getBlue());
-				return comp;
+		JComboBox comboColors = new FlexiComboBox<Color>(new Color(255, 128,
+				128), new Color(128, 255, 128), new Color(128, 128, 255),
+				new Color(255, 255, 128), new Color(255, 128, 255), new Color(
+						128, 255, 255)) {
+			@Override
+			public String getCaption(Color item) {
+				return item.getRed() + ":" + item.getGreen() + ":"
+						+ item.getBlue();
 			}
-		});
+
+			@Override
+			public Color getItemColor(Color item) {
+				return item;
+			}
+		};
+		comboColors.setName("Colors combo");
 		builder.append("Colors", comboColors);
 
 		JComboBox comboFlat = new JComboBox(new Object[] { "entry1", "entry2",
@@ -328,135 +279,10 @@ public class CombosPanel extends ControllablePanel {
 		comboDefaultCoreRenderer.setRenderer(new DefaultListCellRenderer());
 		builder.append("Default core renderer", comboDefaultCoreRenderer);
 
-		builder.appendSeparator("Auto-completion");
-
-		final JComboBox comboCompletionTest = new JComboBox(new Object[] {
-				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
-		comboCompletionTest.setEditable(true);
-		comboCompletionTest.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				Check.out("'" + comboCompletionTest.getSelectedItem()
-						+ "' selected");
-			}
-		});
-		builder.append("Free w/ Object[]", comboCompletionTest);
-
-		final JComboBox comboCompletionTest2 = new JComboBox();
-		comboCompletionTest2.setEditable(true);
-		comboCompletionTest2.setModel(new DefaultComboBoxModel(new Object[] {
-				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" }));
-		comboCompletionTest2.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				Check.out("'" + comboCompletionTest2.getSelectedItem()
-						+ "' selected");
-			}
-		});
-		builder.append("Free w/ default", comboCompletionTest2);
-
-		final JComboBox comboCompletionModelOnlyTest = new JComboBox(
-				new Object[] { "Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
-		comboCompletionModelOnlyTest.setEditable(true);
-		comboCompletionModelOnlyTest.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				Check.out("'" + comboCompletionModelOnlyTest.getSelectedItem()
-						+ "' selected");
-			}
-		});
-		comboCompletionModelOnlyTest.putClientProperty(
-				LafWidget.COMBO_BOX_USE_MODEL_ONLY, Boolean.TRUE);
-		builder.append("Model-based", comboCompletionModelOnlyTest);
-
-		final JComboBox comboCompletionModelOnlyTestNoLock = new JComboBox(
-				new Object[] { "Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
-		comboCompletionModelOnlyTestNoLock.setEditable(true);
-		comboCompletionModelOnlyTestNoLock
-				.addActionListener(new ActionListener() {
-					public void actionPerformed(ActionEvent e) {
-						Check.out("'"
-								+ comboCompletionModelOnlyTest
-										.getSelectedItem() + "' selected");
-					}
-				});
-		comboCompletionModelOnlyTestNoLock.putClientProperty(
-				LafWidget.COMBO_BOX_USE_MODEL_ONLY, Boolean.TRUE);
-		comboCompletionModelOnlyTestNoLock.putClientProperty(
-				LafWidget.NO_LOCK_ICON, Boolean.TRUE);
-		builder.append("Model-based no lock",
-				comboCompletionModelOnlyTestNoLock);
-
-		final JComboBox comboCompletionCaseSensitiveTest = new JComboBox(
-				new Object[] { "AJ", "Ester", "Jordi", "Jordina", "Jorge",
-						"sergi" });
-		comboCompletionCaseSensitiveTest.setEditable(true);
-		comboCompletionCaseSensitiveTest
-				.addActionListener(new ActionListener() {
-					public void actionPerformed(ActionEvent e) {
-						Check.out("'"
-								+ comboCompletionCaseSensitiveTest
-										.getSelectedItem() + "' selected");
-					}
-				});
-		try {
-			comboCompletionCaseSensitiveTest.putClientProperty(
-					LafWidget.COMBO_BOX_AUTOCOMPLETION_MATCHER,
-					new AutoCompletionMatcher() {
-						public Object getFirstMatching(ComboBoxModel model,
-								String pattern) {
-							Object selectedItem = model.getSelectedItem();
-							// only search for a different item if the currently
-							// selected does not match
-							if ((selectedItem != null)
-									&& this.startsWith(selectedItem.toString(),
-											pattern)) {
-								return selectedItem;
-							} else {
-								// iterate over all items
-								for (int i = 0, n = model.getSize(); i < n; i++) {
-									Object currentItem = model.getElementAt(i);
-									// current item starts with the pattern?
-									if ((currentItem != null)
-											&& this.startsWith(currentItem
-													.toString(), pattern)) {
-										return currentItem;
-									}
-								}
-							}
-							// no item starts with the pattern => return null
-							return null;
-						}
-
-						protected boolean startsWith(String str1, String str2) {
-							return str1.startsWith(str2);
-						}
-					});
-		} catch (NoClassDefFoundError cndfe) {
-		}
-		builder.append("Case sensitive", comboCompletionCaseSensitiveTest);
-
-		final JComboBox comboNoAutoCompletion = new JComboBox(new Object[] {
-				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
-		comboNoAutoCompletion.setEditable(true);
-		comboNoAutoCompletion.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				Check.out("'" + comboNoAutoCompletion.getSelectedItem()
-						+ "' selected");
-			}
-		});
-		comboNoAutoCompletion.putClientProperty(
-				LafWidget.COMBO_BOX_NO_AUTOCOMPLETION, Boolean.TRUE);
-		builder.append("No completion", comboNoAutoCompletion);
-
-		final JComboBox comboCompletionColored = new JComboBox(new Object[] {
-				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
-		comboCompletionColored.setEditable(true);
-		comboCompletionColored.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				Check.out("'" + comboCompletionColored.getSelectedItem()
-						+ "' selected");
-			}
-		});
-		comboCompletionColored.setBackground(new Color(255, 128, 128));
-		builder.append("Colored", comboCompletionColored);
+		JComboBox comboEditable = new JComboBox(new Object[] { "Ester",
+				"Jordi", "Jordina", "Jorge", "Sergi" });
+		comboEditable.setEditable(true);
+		builder.append("Editable", comboEditable);
 
 		builder.appendSeparator("Miscellaneous");
 
@@ -474,41 +300,32 @@ public class CombosPanel extends ControllablePanel {
 		comboHebrew.setMaximumRowCount(6);
 		builder.append("RTL (Hebrew)", comboHebrew);
 
-		JComboBox comboHebrewCustomRenderer = new JComboBox(new Object[] {
-				"\u05e8\u05d0\u05e9\u05d9 1", "\u05e8\u05d0\u05e9\u05d9 2",
-				"\u05e8\u05d0\u05e9\u05d9 3", "\u05e8\u05d0\u05e9\u05d9 4",
-				"\u05e8\u05d0\u05e9\u05d9 5", "\u05e8\u05d0\u05e9\u05d9 6",
-				"\u05e8\u05d0\u05e9\u05d9 7", "\u05e8\u05d0\u05e9\u05d9 8",
-				"\u05e8\u05d0\u05e9\u05d9 9" });
-		comboHebrewCustomRenderer.setToolTipText("RTL combo");
-		comboHebrewCustomRenderer
-				.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
-		comboHebrewCustomRenderer.setMaximumRowCount(6);
-		comboHebrewCustomRenderer
-				.setRenderer(new SubstanceDefaultComboBoxRenderer(
-						comboHebrewCustomRenderer) {
-					private static final long serialVersionUID = 2629717016682715818L;
+		try {
+			final Icon israelFlag = new ImageIcon(ImageIO.read(getClass()
+					.getResource("/test/check/icons/flag_israel.png")));
+			JComboBox comboHebrewCustomRenderer = new FlexiComboBox<String>(
+					"\u05e8\u05d0\u05e9\u05d9 1", "\u05e8\u05d0\u05e9\u05d9 2",
+					"\u05e8\u05d0\u05e9\u05d9 3", "\u05e8\u05d0\u05e9\u05d9 4",
+					"\u05e8\u05d0\u05e9\u05d9 5", "\u05e8\u05d0\u05e9\u05d9 6",
+					"\u05e8\u05d0\u05e9\u05d9 7", "\u05e8\u05d0\u05e9\u05d9 8",
+					"\u05e8\u05d0\u05e9\u05d9 9") {
+				@Override
+				public String getCaption(String item) {
+					return item;
+				}
 
-					@Override
-					public Component getListCellRendererComponent(JList list,
-							Object value, int index, boolean isSelected,
-							boolean hasFocus) {
-						Component c = super.getListCellRendererComponent(list,
-								value, index, isSelected, hasFocus);
-						try {
-							((JLabel) c)
-									.setIcon(new ImageIcon(
-											ImageIO
-													.read(getClass()
-															.getResource(
-																	"/test/check/icons/flag_israel.png"))));
-						} catch (IOException e) {
-							e.printStackTrace();
-						}
-						return c;
-					}
-				});
-		builder.append("RTL (Hebrew) with icon", comboHebrewCustomRenderer);
+				@Override
+				public Icon getItemIcon(String item) {
+					return israelFlag;
+				}
+			};
+			comboHebrewCustomRenderer.setToolTipText("RTL combo");
+			comboHebrewCustomRenderer
+					.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
+			comboHebrewCustomRenderer.setMaximumRowCount(6);
+			builder.append("RTL (Hebrew) with icon", comboHebrewCustomRenderer);
+		} catch (IOException ioe) {
+		}
 
 		final JComboBox combo151 = new JComboBox();
 		final ComboBoxTextEditor151 editor = new ComboBoxTextEditor151();
@@ -532,44 +349,6 @@ public class CombosPanel extends ControllablePanel {
 		});
 		builder.append("Defect 151", combo151);
 
-		if (UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) {
-			Vector<SubstanceTheme> themeVector = new Vector<SubstanceTheme>();
-			Map<String, ThemeInfo> allThemes = SubstanceLookAndFeel
-					.getAllThemes();
-			for (Map.Entry<String, ThemeInfo> themeEntry : allThemes.entrySet()) {
-				themeVector.add(SubstanceTheme.createInstance(themeEntry
-						.getValue()));
-			}
-			JComboBox themeCombo = new JComboBox(themeVector);
-			themeCombo.addItemListener(new ItemListener() {
-				public void itemStateChanged(ItemEvent evt) {
-					// Get the affected item
-					final Object item = evt.getItem();
-
-					if (evt.getStateChange() == ItemEvent.SELECTED) {
-						SwingUtilities.invokeLater(new Runnable() {
-							public void run() {
-								try {
-									SubstanceLookAndFeel
-											.setCurrentTheme((SubstanceTheme) item);
-									// UIManager
-									// .setLookAndFeel(new
-									// SubstanceLookAndFeel());
-									for (Frame frame : Frame.getFrames()) {
-										SwingUtilities
-												.updateComponentTreeUI(frame);
-									}
-								} catch (Exception exc) {
-								}
-							};
-						});
-					}
-				}
-			});
-			themeCombo.setRenderer(new ThemeComboRenderer());
-			builder.append("Themes", themeCombo);
-		}
-
 		return builder.getPanel();
 	}
 
