@@ -45,6 +45,7 @@ import org.jvnet.substance.SubstanceLookAndFeel;
 import org.jvnet.substance.api.SubstanceConstants.*;
 import org.jvnet.substance.skin.NebulaBrickWallSkin;
 import org.jvnet.substance.utils.SubstanceCoreUtilities;
+import org.jvnet.substance.utils.SubstanceImageCreator;
 
 import test.Check;
 import test.Check.MyMainTabPreviewPainter;
@@ -136,7 +137,7 @@ public class ControlPanelFactory {
 		final JCheckBox useThemedDefaultIconsCheckBox = new JCheckBox(
 				"use themed icons");
 		useThemedDefaultIconsCheckBox.setSelected(SubstanceCoreUtilities
-				.useThemedDefaultIcon(null));
+				.useThemedDefaultIcon());
 		useThemedDefaultIconsCheckBox.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				SwingUtilities.invokeLater(new Runnable() {
@@ -737,6 +738,43 @@ public class ControlPanelFactory {
 		});
 		builder.append("Disposable dialog", openDisposable);
 
+		JButton launchFrameDialogWithIcon = new JButton("Open");
+		launchFrameDialogWithIcon.addActionListener(new ActionListener() {
+			public void actionPerformed(ActionEvent e) {
+				SwingUtilities.invokeLater(new Runnable() {
+					public void run() {
+						SimpleDialog sd = new SimpleDialog();
+						if (UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) {
+							sd
+									.setIconImage(SubstanceImageCreator
+											.getColorSchemeImage(
+													null,
+													new ImageIcon(
+															Thread
+																	.currentThread()
+																	.getContextClassLoader()
+																	.getResource(
+																			"test/resource/image-x-generic.png")),
+													SubstanceLookAndFeel
+															.getCurrentSkin(
+																	sd
+																			.getRootPane())
+															.getMainActiveColorScheme(),
+													0.0f));
+						}
+						sd.getRootPane().setWindowDecorationStyle(
+								JRootPane.FRAME);
+						sd.setModal(false);
+						sd.pack();
+						sd.setLocationRelativeTo(null);
+						sd.setVisible(true);
+						sd.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
+					}
+				});
+			};
+		});
+		builder.append("Dialog with icon", launchFrameDialogWithIcon);
+
 		JButton bd = new JButton("Open");
 		bd.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
@@ -745,11 +783,7 @@ public class ControlPanelFactory {
 						SimpleDialog sd = new SimpleDialog();
 						sd.setModal(false);
 						sd.pack();
-						Dimension d = Toolkit.getDefaultToolkit()
-								.getScreenSize();
-						// center the frame in the physical screen
-						sd.setLocation((d.width - sd.getWidth()) / 2,
-								(d.height - sd.getHeight()) / 2);
+						sd.setLocationRelativeTo(null);
 						sd.setVisible(true);
 						simpleDialog = sd;
 					}
