@@ -472,6 +472,8 @@ public class SampleMenuFactory {
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame,
 				"Creme Coffee",
 				"org.jvnet.substance.skin.SubstanceCremeCoffeeLookAndFeel"));
+		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame, "Dust",
+				"org.jvnet.substance.skin.SubstanceDustLookAndFeel"));
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame, "Moderate",
 				"org.jvnet.substance.skin.SubstanceModerateLookAndFeel"));
 		substanceMenus.add(SubstanceLafChanger.getMenuItem(frame, "Nebula",
@@ -828,6 +830,11 @@ public class SampleMenuFactory {
 		localeTurkish.addActionListener(new MyLocaleChangeListener("tr", "TR",
 				frame));
 
+		JMenuItem localeVietnamese = new JMenuItem("Vietnamese Locale", Check
+				.getIcon("flag_vietnam"));
+		localeVietnamese.addActionListener(new MyLocaleChangeListener("vi",
+				"VN", frame));
+
 		localeMenus.add(localeEnglish);
 		localeMenus.add(localeArabic);
 		localeMenus.add(localeBulgarian);
@@ -857,6 +864,7 @@ public class SampleMenuFactory {
 		localeMenus.add(localeSwedish);
 		localeMenus.add(localeThai);
 		localeMenus.add(localeTurkish);
+		localeMenus.add(localeVietnamese);
 
 		return lafMenu;
 	}
