@@ -192,6 +192,15 @@ public class SliderPanel extends ControllablePanel {
 		disHorizontalLTR.setEnabled(false);
 		horPanel.add(disHorizontalLTR);
 
+		JSlider invertedHorizontalLTR = new JSlider(0, 100, 30);
+		invertedHorizontalLTR.setSnapToTicks(true);
+		invertedHorizontalLTR.setMajorTickSpacing(20);
+		invertedHorizontalLTR.setMinorTickSpacing(5);
+		invertedHorizontalLTR.setPaintTicks(true);
+		invertedHorizontalLTR.setPaintLabels(true);
+		invertedHorizontalLTR.setInverted(true);
+		horPanel.add(invertedHorizontalLTR);
+
 		JSlider horizontalRTL = new JSlider(0, 100, 50);
 		horizontalRTL.setSnapToTicks(true);
 		horizontalRTL.setMajorTickSpacing(20);
@@ -269,6 +278,15 @@ public class SliderPanel extends ControllablePanel {
 		disVerticalLTR.setEnabled(false);
 		verPanel.add(disVerticalLTR);
 
+		JSlider invertedVerticalLTR = new JSlider(JSlider.VERTICAL, 0, 100, 30);
+		invertedVerticalLTR.setSnapToTicks(true);
+		invertedVerticalLTR.setMajorTickSpacing(20);
+		invertedVerticalLTR.setMinorTickSpacing(5);
+		invertedVerticalLTR.setPaintTicks(true);
+		invertedVerticalLTR.setPaintLabels(true);
+		invertedVerticalLTR.setInverted(true);
+		verPanel.add(invertedVerticalLTR);
+
 		JSlider verticalRTL3 = new JSlider(JSlider.VERTICAL, 0, 100, 50);
 		verticalRTL3.setSnapToTicks(true);
 		verticalRTL3.setMajorTickSpacing(20);
