@@ -38,13 +38,11 @@ import javax.swing.*;
 
 import org.jvnet.lafwidget.LafWidgetUtilities2;
 import org.jvnet.lafwidget.utils.LafConstants.TabOverviewKind;
-import org.jvnet.substance.*;
-import org.jvnet.substance.button.ClassicButtonShaper;
-import org.jvnet.substance.theme.*;
-import org.jvnet.substance.theme.SubstanceTheme.ThemeKind;
+import org.jvnet.substance.SubstanceLookAndFeel;
+import org.jvnet.substance.api.SubstanceConstants.TabContentPaneBorderKind;
+import org.jvnet.substance.skin.SkinChangeListener;
 import org.jvnet.substance.utils.SubstanceCoreUtilities;
-import org.jvnet.substance.utils.SubstanceConstants.TabContentPaneBorderKind;
-import org.jvnet.substance.utils.SubstanceConstants.TabTextAlignmentKind;
+import org.jvnet.substance.utils.SubstanceImageCreator;
 
 import com.jgoodies.forms.builder.PanelBuilder;
 import com.jgoodies.forms.layout.CellConstraints;
@@ -81,7 +79,7 @@ public class TabControlPanel extends JPanel {
 		 * Creates the combobox model.
 		 * 
 		 * @param jtp
-		 *            Tabbed pane.
+		 * 		Tabbed pane.
 		 */
 		public TabComboBoxModel(JTabbedPane jtp) {
 			this.jtp = jtp;
@@ -134,8 +132,9 @@ public class TabControlPanel extends JPanel {
 		/*
 		 * (non-Javadoc)
 		 * 
-		 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
-		 *      java.lang.Object, int, boolean, boolean)
+		 * @see
+		 * javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing
+		 * .JList, java.lang.Object, int, boolean, boolean)
 		 */
 		public Component getListCellRendererComponent(JList list, Object value,
 				int index, boolean isSelected, boolean cellHasFocus) {
@@ -154,9 +153,9 @@ public class TabControlPanel extends JPanel {
 	 * Returns the contents of the control panel.
 	 * 
 	 * @param jtp
-	 *            Tabbed pane.
+	 * 		Tabbed pane.
 	 * @param mainTabPreviewPainter
-	 *            Tab preview painter.
+	 * 		Tab preview painter.
 	 * @return Control panel contents.
 	 */
 	private JPanel getContents(final JTabbedPane jtp,
@@ -179,19 +178,24 @@ public class TabControlPanel extends JPanel {
 		final JComboBox addKindCombo = new JComboBox(new Object[] { "regular",
 				"null", "modified" });
 		JButton addNewTabButton = new JButton("Add");
-		addNewTabButton.putClientProperty(
-				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
-				new ClassicButtonShaper());
 		addNewTabButton.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				String selectedKind = (String) addKindCombo.getSelectedItem();
 				if ("null".equals(selectedKind)) {
 					SwingUtilities.invokeLater(new Runnable() {
 						public void run() {
-							jtp.addTab("null tab", new ImageIcon(
-									SubstanceImageCreator.getBigHexaMarker(0,
-											SubstanceLookAndFeel.getTheme())),
-									null);
+							jtp
+									.addTab(
+											"null tab",
+											new ImageIcon(
+													SubstanceImageCreator
+															.getBigHexaMarker(
+																	0,
+																	SubstanceLookAndFeel
+																			.getCurrentSkin(
+																					jtp)
+																			.getMainActiveColorScheme())),
+											null);
 						}
 					});
 					return;
@@ -207,19 +211,28 @@ public class TabControlPanel extends JPanel {
 					public void run() {
 						jtp.addTab("tab" + count, new ImageIcon(
 								SubstanceImageCreator.getBigHexaMarker(count,
-										SubstanceLookAndFeel.getTheme())),
+										SubstanceLookAndFeel
+												.getCurrentSkin(jtp)
+												.getMainActiveColorScheme())),
 								tabComp);
 					}
 				});
 				SubstanceLookAndFeel
-						.registerThemeChangeListener(new ThemeChangeListener() {
-							public void themeChanged() {
+						.registerSkinChangeListener(new SkinChangeListener() {
+							@Override
+							public void skinChanged() {
 								int index = jtp.indexOfComponent(tabComp);
 								if (index >= 0)
-									jtp.setIconAt(index, SubstanceImageCreator
-											.getHexaMarker(count,
-													SubstanceLookAndFeel
-															.getTheme()));
+									jtp
+											.setIconAt(
+													index,
+													SubstanceImageCreator
+															.getHexaMarker(
+																	count,
+																	SubstanceLookAndFeel
+																			.getCurrentSkin(
+																					jtp)
+																			.getMainActiveColorScheme()));
 							}
 						});
 			}
@@ -249,33 +262,15 @@ public class TabControlPanel extends JPanel {
 		builder.addLabel("Placement", cc.xy(1, row));
 		builder.add(placementCombo, cc.xyw(3, row, 3));
 
-		final JComboBox alignmentCombo = new JComboBox(new Object[] {
-				TabTextAlignmentKind.DEFAULT, TabTextAlignmentKind.ALWAYS_LEFT,
-				TabTextAlignmentKind.ALWAYS_RIGHT,
-				TabTextAlignmentKind.FOLLOW_PLACEMENT,
-				TabTextAlignmentKind.FOLLOW_ORIENTATION });
-		alignmentCombo.setPrototypeDisplayValue(TabTextAlignmentKind.DEFAULT);
-		alignmentCombo.putClientProperty(
-				SubstanceLookAndFeel.COMBO_POPUP_PROTOTYPE,
-				TabTextAlignmentKind.DEFAULT);
-		alignmentCombo.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				TabTextAlignmentKind alignment = (TabTextAlignmentKind) alignmentCombo
-						.getSelectedItem();
-				jtp.putClientProperty(
-						SubstanceLookAndFeel.TABBED_PANE_TEXT_ALIGNMENT_KIND,
-						alignment);
-				jtp.repaint();
-			}
-		});
-		row += 2;
-		builder.addLabel("Alignment", cc.xy(1, row));
-		builder.add(alignmentCombo, cc.xyw(3, row, 3));
-
 		try {
-			final JComboBox overviewKindCombo = new JComboBox(new Object[] {
+			final JComboBox overviewKindCombo = new FlexiComboBox<TabOverviewKind>(
 					TabOverviewKind.GRID, TabOverviewKind.MENU_CAROUSEL,
-					TabOverviewKind.ROUND_CAROUSEL });
+					TabOverviewKind.ROUND_CAROUSEL) {
+				@Override
+				public String getCaption(TabOverviewKind item) {
+					return item.getName();
+				}
+			};
 			overviewKindCombo.setSelectedItem(LafWidgetUtilities2
 					.getTabPreviewPainter(jtp).getOverviewKind(jtp));
 			overviewKindCombo.addActionListener(new ActionListener() {
@@ -285,27 +280,12 @@ public class TabControlPanel extends JPanel {
 									.getSelectedItem());
 				}
 			});
-			overviewKindCombo.setRenderer(new SubstanceDefaultComboBoxRenderer(
-					overviewKindCombo) {
-				@Override
-				public Component getListCellRendererComponent(JList list,
-						Object value, int index, boolean isSelected,
-						boolean cellHasFocus) {
-					TabOverviewKind kind = (TabOverviewKind) value;
-					return super.getListCellRendererComponent(list, kind
-							.getName(), index, isSelected, cellHasFocus);
-				}
-			});
 			row += 2;
 			builder.addLabel("Overview kind", cc.xy(1, row));
 			builder.add(overviewKindCombo, cc.xyw(3, row, 3));
 		} catch (NoClassDefFoundError ncdfe) {
 		}
 
-		row += 2;
-		builder.addLabel("Composite", cc.xy(1, row));
-		builder.add(new BackgroundCompositeCombo(jtp), cc.xyw(3, row, 3));
-
 		final JCheckBox useScrollLayout = new JCheckBox("Uses scroll layout");
 		useScrollLayout.setSelected(false);
 		useScrollLayout.addActionListener(new ActionListener() {
@@ -342,9 +322,6 @@ public class TabControlPanel extends JPanel {
 		builder.add(contentBorderCombo, cc.xyw(3, row, 3));
 
 		JButton enableAll = new JButton("+ all");
-		enableAll.putClientProperty(
-				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
-				new ClassicButtonShaper());
 		enableAll.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				for (int i = 0; i < jtp.getTabCount(); i++) {
@@ -354,9 +331,6 @@ public class TabControlPanel extends JPanel {
 		});
 
 		JButton disableAll = new JButton("- all");
-		disableAll.putClientProperty(
-				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
-				new ClassicButtonShaper());
 		disableAll.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				for (int i = 0; i < jtp.getTabCount(); i++) {
@@ -371,9 +345,6 @@ public class TabControlPanel extends JPanel {
 		builder.add(disableAll, cc.xy(5, row));
 
 		JButton closeAllEnabled = new JButton("Close");
-		closeAllEnabled.putClientProperty(
-				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
-				new ClassicButtonShaper());
 		closeAllEnabled.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				Set<Component> toRemove = new HashSet<Component>();
@@ -387,9 +358,6 @@ public class TabControlPanel extends JPanel {
 		});
 
 		JButton restoreClosed = new JButton("Restore");
-		restoreClosed.putClientProperty(
-				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
-				new ClassicButtonShaper());
 		restoreClosed.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				for (Component tnp : closed) {
@@ -403,69 +371,6 @@ public class TabControlPanel extends JPanel {
 		builder.add(closeAllEnabled, cc.xy(3, row));
 		builder.add(restoreClosed, cc.xy(5, row));
 
-		row += 2;
-		builder.addSeparator("General Theme", cc.xyw(1, row, 5));
-
-		ButtonGroup paneThemeGroup = new ButtonGroup();
-		final JRadioButton paneNoThemeRb = new JRadioButton("Global");
-		paneNoThemeRb.setSelected(true);
-		final JRadioButton paneBarbyPinkThemeRb = new JRadioButton("Barby Pink");
-		final JRadioButton paneComplexThemeRb = new JRadioButton(
-				"Custom complex");
-		paneThemeGroup.add(paneNoThemeRb);
-		paneThemeGroup.add(paneBarbyPinkThemeRb);
-		paneThemeGroup.add(paneComplexThemeRb);
-		paneNoThemeRb.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				jtp
-						.putClientProperty(SubstanceLookAndFeel.THEME_PROPERTY,
-								null);
-				jtp.repaint();
-			}
-		});
-		paneBarbyPinkThemeRb.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				jtp.putClientProperty(SubstanceLookAndFeel.THEME_PROPERTY,
-						new SubstanceBarbyPinkTheme());
-				jtp.repaint();
-			}
-		});
-		paneComplexThemeRb.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				jtp.putClientProperty(SubstanceLookAndFeel.THEME_PROPERTY,
-						new SubstanceComplexTheme("Test theme",
-								ThemeKind.BRIGHT,
-								new SubstanceBottleGreenTheme(),
-								new SubstanceLimeGreenTheme(),
-								new SubstancePurpleTheme(),
-								new SubstanceBottleGreenTheme()));
-				jtp.repaint();
-			}
-		});
-
-		row += 2;
-		builder.addLabel("Theme", cc.xy(1, row));
-		builder.add(paneNoThemeRb, cc.xyw(3, row, 3));
-		row += 2;
-		builder.add(paneBarbyPinkThemeRb, cc.xyw(3, row, 3));
-		row += 2;
-		builder.add(paneComplexThemeRb, cc.xyw(3, row, 3));
-
-		final JCheckBox paintActiveTheme = new JCheckBox("Paint active");
-		paintActiveTheme.setSelected(false);
-		paintActiveTheme.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				jtp.putClientProperty(
-						SubstanceLookAndFeel.PAINT_ACTIVE_PROPERTY,
-						(paintActiveTheme.isSelected() ? Boolean.TRUE : null));
-				jtp.repaint();
-			}
-		});
-
-		row += 2;
-		builder.addLabel("Active", cc.xy(1, row));
-		builder.add(paintActiveTheme, cc.xyw(3, row, 3));
-
 		row += 2;
 		builder.addSeparator("Single Tab", cc.xyw(1, row, 5));
 
@@ -594,9 +499,6 @@ public class TabControlPanel extends JPanel {
 		});
 
 		JButton closeButton = new JButton("Close");
-		closeButton.putClientProperty(
-				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
-				new ClassicButtonShaper());
 		closeButton.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				SwingUtilities.invokeLater(new Runnable() {
@@ -616,9 +518,6 @@ public class TabControlPanel extends JPanel {
 		});
 
 		JButton selectButton = new JButton("Select");
-		selectButton.putClientProperty(
-				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
-				new ClassicButtonShaper());
 		selectButton.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				SwingUtilities.invokeLater(new Runnable() {
@@ -636,132 +535,6 @@ public class TabControlPanel extends JPanel {
 		builder.add(closeButton, cc.xy(3, row));
 		builder.add(selectButton, cc.xy(5, row));
 
-		row += 2;
-		builder.addSeparator("Theme Single", cc.xyw(1, row, 5));
-
-		ButtonGroup tabThemeGroup = new ButtonGroup();
-		final JRadioButton tabNoThemeRb = new JRadioButton("Global");
-		tabNoThemeRb.setSelected(true);
-		final JRadioButton tabBarbyPinkThemeRb = new JRadioButton("Barby Pink");
-		final JRadioButton tabComplexThemeRb = new JRadioButton(
-				"Custom complex");
-		tabThemeGroup.add(tabNoThemeRb);
-		tabThemeGroup.add(tabBarbyPinkThemeRb);
-		tabThemeGroup.add(tabComplexThemeRb);
-		tabNoThemeRb.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				if (tabSelectorCombo.getSelectedItem() == null)
-					return;
-				Component comp = jtp.getComponentAt((Integer) tabSelectorCombo
-						.getSelectedItem());
-				if ((comp != null) && (comp instanceof JComponent)) {
-					JComponent jc = (JComponent) comp;
-					jc.putClientProperty(SubstanceLookAndFeel.THEME_PROPERTY,
-							null);
-					jtp.repaint();
-				}
-			}
-		});
-		tabBarbyPinkThemeRb.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				if (tabSelectorCombo.getSelectedItem() == null)
-					return;
-				Component comp = jtp.getComponentAt((Integer) tabSelectorCombo
-						.getSelectedItem());
-				if ((comp != null) && (comp instanceof JComponent)) {
-					JComponent jc = (JComponent) comp;
-					jc.putClientProperty(SubstanceLookAndFeel.THEME_PROPERTY,
-							new SubstanceBarbyPinkTheme());
-					jtp.repaint();
-				}
-			}
-		});
-		tabComplexThemeRb.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				if (tabSelectorCombo.getSelectedItem() == null)
-					return;
-				Component comp = jtp.getComponentAt((Integer) tabSelectorCombo
-						.getSelectedItem());
-				if ((comp != null) && (comp instanceof JComponent)) {
-					JComponent jc = (JComponent) comp;
-					jc.putClientProperty(SubstanceLookAndFeel.THEME_PROPERTY,
-							new SubstanceComplexTheme("Test theme",
-									ThemeKind.BRIGHT,
-									new SubstanceBottleGreenTheme(),
-									new SubstanceLimeGreenTheme(),
-									new SubstancePurpleTheme(),
-									new SubstanceBottleGreenTheme()));
-					jtp.repaint();
-				}
-			}
-		});
-		tabSelectorCombo.addItemListener(new ItemListener() {
-			public void itemStateChanged(ItemEvent e) {
-				Component selComp = jtp
-						.getComponentAt((Integer) tabSelectorCombo
-								.getSelectedItem());
-				if ((selComp != null) && (selComp instanceof JComponent)) {
-					JComponent selected = (JComponent) selComp;
-					Object obj = selected
-							.getClientProperty(SubstanceLookAndFeel.THEME_PROPERTY);
-					if (obj == null) {
-						tabNoThemeRb.setSelected(true);
-						return;
-					}
-					if (obj instanceof SubstanceBarbyPinkTheme) {
-						tabBarbyPinkThemeRb.setSelected(true);
-						return;
-					}
-
-					tabComplexThemeRb.setSelected(true);
-				}
-			}
-		});
-
-		row += 2;
-		builder.addLabel("Theme", cc.xy(1, row));
-		builder.add(tabNoThemeRb, cc.xyw(3, row, 3));
-		row += 2;
-		builder.add(tabBarbyPinkThemeRb, cc.xyw(3, row, 3));
-		row += 2;
-		builder.add(tabComplexThemeRb, cc.xyw(3, row, 3));
-
-		final JCheckBox paintTabAsActive = new JCheckBox("Paint active");
-		paintTabAsActive.setSelected(false);
-		paintTabAsActive.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				if (tabSelectorCombo.getSelectedItem() == null)
-					return;
-				Component comp = jtp.getComponentAt((Integer) tabSelectorCombo
-						.getSelectedItem());
-				if ((comp != null) && (comp instanceof JComponent)) {
-					JComponent jc = (JComponent) comp;
-					jc.putClientProperty(
-							SubstanceLookAndFeel.PAINT_ACTIVE_PROPERTY,
-							(paintTabAsActive.isSelected() ? Boolean.TRUE
-									: null));
-					jtp.repaint();
-				}
-			}
-		});
-		tabSelectorCombo.addItemListener(new ItemListener() {
-			public void itemStateChanged(ItemEvent e) {
-				Component selComp = jtp
-						.getComponentAt((Integer) tabSelectorCombo
-								.getSelectedItem());
-				if ((selComp != null) && (selComp instanceof JComponent)) {
-					JComponent selected = (JComponent) selComp;
-					paintTabAsActive
-							.setSelected(Boolean.TRUE
-									.equals(selected
-											.getClientProperty(SubstanceLookAndFeel.PAINT_ACTIVE_PROPERTY)));
-				}
-			}
-		});
-		row += 2;
-		builder.addLabel("Active", cc.xy(1, row));
-		builder.add(paintTabAsActive, cc.xyw(3, row, 3));
-
 		row += 2;
 		builder.addSeparator("Close Button Single", cc.xyw(1, row, 5));
 
@@ -776,9 +549,9 @@ public class TabControlPanel extends JPanel {
 	 * Creates the tab control panel.
 	 * 
 	 * @param jtp
-	 *            Tabbed pane.
+	 * 		Tabbed pane.
 	 * @param previewPainter
-	 *            Preview painter.
+	 * 		Preview painter.
 	 */
 	public TabControlPanel(JTabbedPane jtp, MyTabPreviewPainter previewPainter) {
 		super();
