@@ -32,7 +32,6 @@ package test.check;
 import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.lang.reflect.Method;
 import java.util.Calendar;
 import java.util.Date;
 
@@ -43,12 +42,13 @@ import javax.swing.table.AbstractTableModel;
 import javax.swing.table.JTableHeader;
 
 import org.jvnet.lafwidget.animation.*;
-import org.jvnet.lafwidget.utils.LookUtils;
 import org.jvnet.lafwidget.utils.LafConstants.AnimationKind;
 import org.jvnet.substance.*;
-import org.jvnet.substance.theme.SubstanceTheme;
-import org.jvnet.substance.theme.ThemeInfo;
+import org.jvnet.substance.api.SubstanceColorScheme;
+import org.jvnet.substance.api.renderers.SubstanceDefaultTableCellRenderer;
+import org.jvnet.substance.colorscheme.*;
 import org.jvnet.substance.utils.SubstanceColorUtilities;
+import org.jvnet.substance.utils.SubstanceImageCreator;
 
 import com.jgoodies.forms.builder.DefaultFormBuilder;
 import com.jgoodies.forms.layout.FormLayout;
@@ -74,8 +74,9 @@ public class TablePanel extends ControllablePanel {
 		/*
 		 * (non-Javadoc)
 		 * 
-		 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable,
-		 *      java.lang.Object, boolean, boolean, int, int)
+		 * @see
+		 * javax.swing.table.TableCellRenderer#getTableCellRendererComponent
+		 * (javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
 		 */
 		@Override
 		public Component getTableCellRendererComponent(JTable table,
@@ -99,8 +100,9 @@ public class TablePanel extends ControllablePanel {
 		/*
 		 * (non-Javadoc)
 		 * 
-		 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable,
-		 *      java.lang.Object, boolean, boolean, int, int)
+		 * @see
+		 * javax.swing.table.TableCellRenderer#getTableCellRendererComponent
+		 * (javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
 		 */
 		@Override
 		public Component getTableCellRendererComponent(JTable table,
@@ -153,6 +155,11 @@ public class TablePanel extends ControllablePanel {
 		public MyTableModel(int rows) {
 			this.rows = rows;
 			this.data = new Object[rows][this.cols];
+			SubstanceColorScheme[] schemes = new SubstanceColorScheme[] {
+					new AquaColorScheme(), new BottleGreenColorScheme(),
+					new BarbyPinkColorScheme(), new PurpleColorScheme(),
+					new OliveColorScheme(), new OrangeColorScheme(),
+					new BrownColorScheme() };
 			for (int i = 0; i < rows; i++) {
 				this.data[i][0] = "cell " + i + ":" + 0;
 				this.data[i][1] = "predef";
@@ -166,21 +173,10 @@ public class TablePanel extends ControllablePanel {
 				this.data[i][7] = cal.getTime();
 				int count = 0;
 				if (UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) {
-					for (ThemeInfo themeInfo : SubstanceLookAndFeel
-							.getAllThemes().values()) {
-						if (count++ == i) {
-							try {
-								this.data[i][8] = SubstanceImageCreator
-										.getHexaMarker(
-												6,
-												(SubstanceTheme) Class
-														.forName(
-																themeInfo
-																		.getClassName())
-														.newInstance());
-							} catch (Exception exc) {
-							}
-						}
+					try {
+						this.data[i][8] = SubstanceImageCreator.getHexaMarker(
+								6, schemes[i % schemes.length]);
+					} catch (Exception exc) {
 					}
 				}
 
@@ -252,8 +248,9 @@ public class TablePanel extends ControllablePanel {
 		/*
 		 * (non-Javadoc)
 		 * 
-		 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object,
-		 *      int, int)
+		 * @see
+		 * javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object,
+		 * int, int)
 		 */
 		@Override
 		public void setValueAt(Object value, int row, int col) {
@@ -310,11 +307,10 @@ public class TablePanel extends ControllablePanel {
 				AnimationKind.DEBUG_FAST, instructional, 0, false,
 				new FadeTrackerAdapter() {
 					@Override
-					public void fadePerformed(FadeKind fadeKind,
-							float fadeCycle10) {
+					public void fadePerformed(FadeKind fadeKind, float fadeCycle) {
 						instructional.setForeground(SubstanceColorUtilities
 								.getInterpolatedColor(Color.black, Color.blue,
-										fadeCycle10 / 10.0));
+										fadeCycle));
 					}
 				}, -1, true);
 
@@ -389,42 +385,25 @@ public class TablePanel extends ControllablePanel {
 		});
 		builder.append("", areColsSelectable);
 
-		// TODO JDK 6 replace with regular code
-		if (LookUtils.IS_JAVA_6_OR_LATER) {
-			try {
-				final JCheckBox isSorted = new JCheckBox("Sorted");
-				Method isAutoSorter = JTable.class.getDeclaredMethod(
-						"getAutoCreateRowSorter", new Class[0]);
-				isSorted.setSelected((Boolean) isAutoSorter.invoke(this.table,
-						new Object[0]));
-				isSorted.addActionListener(new ActionListener() {
-					public void actionPerformed(ActionEvent e) {
-						try {
-							Method setAutoSorter = JTable.class
-									.getDeclaredMethod(
-											"setAutoCreateRowSorter",
-											new Class[] { boolean.class });
-							setAutoSorter.invoke(table, new Object[] { isSorted
-									.isSelected() });
-							table.repaint();
-							table.getTableHeader().repaint();
-						} catch (Exception exc) {
-						}
-					}
-				});
-				builder.append("Sorted", isSorted);
-			} catch (Exception exc) {
+		final JCheckBox isSorted = new JCheckBox("Sorted");
+		isSorted.setSelected(table.getAutoCreateRowSorter());
+		isSorted.addActionListener(new ActionListener() {
+			public void actionPerformed(ActionEvent e) {
+				table.setAutoCreateRowSorter(isSorted.isSelected());
+				table.repaint();
+				table.getTableHeader().repaint();
 			}
-		}
+		});
+		builder.append("Sorted", isSorted);
 
 		final JCheckBox watermarkBleed = new JCheckBox("Watermark bleed");
 		watermarkBleed.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				TablePanel.this.table.putClientProperty(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean
 								.valueOf(watermarkBleed.isSelected()));
 				tableScrollpane.putClientProperty(
-						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean
+						SubstanceLookAndFeel.WATERMARK_VISIBLE, Boolean
 								.valueOf(watermarkBleed.isSelected()));
 				tableScrollpane.repaint();
 			}
@@ -450,44 +429,29 @@ public class TablePanel extends ControllablePanel {
 		});
 		builder.append("", linesHorizontal);
 
-		final JComboBox resizeModeCombo = new JComboBox(
-				new Object[] { JTable.AUTO_RESIZE_OFF,
-						JTable.AUTO_RESIZE_NEXT_COLUMN,
-						JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS,
-						JTable.AUTO_RESIZE_LAST_COLUMN,
-						JTable.AUTO_RESIZE_ALL_COLUMNS });
-		resizeModeCombo.setSelectedItem(this.table.getAutoResizeMode());
-		resizeModeCombo.setRenderer(new SubstanceDefaultComboBoxRenderer(
-				resizeModeCombo) {
+		final JComboBox resizeModeCombo = new FlexiComboBox<Integer>(
+				JTable.AUTO_RESIZE_OFF, JTable.AUTO_RESIZE_NEXT_COLUMN,
+				JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS,
+				JTable.AUTO_RESIZE_LAST_COLUMN, JTable.AUTO_RESIZE_ALL_COLUMNS) {
 			@Override
-			public Component getListCellRendererComponent(JList list,
-					Object value, int index, boolean isSelected,
-					boolean cellHasFocus) {
-				JLabel result = (JLabel) super.getListCellRendererComponent(
-						list, value, index, isSelected, cellHasFocus);
-
-				int iv = (Integer) value;
+			public String getCaption(Integer item) {
+				int iv = item;
 				switch (iv) {
 				case JTable.AUTO_RESIZE_OFF:
-					result.setText("off");
-					break;
+					return "off";
 				case JTable.AUTO_RESIZE_NEXT_COLUMN:
-					result.setText("next");
-					break;
+					return "next";
 				case JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS:
-					result.setText("subsequent");
-					break;
+					return "subsequent";
 				case JTable.AUTO_RESIZE_LAST_COLUMN:
-					result.setText("last");
-					break;
+					return "last";
 				case JTable.AUTO_RESIZE_ALL_COLUMNS:
-					result.setText("all");
-					break;
+					return "all";
 				}
-
-				return result;
+				return null;
 			}
-		});
+		};
+		resizeModeCombo.setSelectedItem(this.table.getAutoResizeMode());
 
 		resizeModeCombo.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
@@ -531,31 +495,6 @@ public class TablePanel extends ControllablePanel {
 		});
 		builder.append("Selections", hasSelectionAnimations);
 
-		builder.appendSeparator("Header settings");
-		final JCheckBox useCurrentTheme = new JCheckBox("Global theme");
-		useCurrentTheme.setSelected(true);
-		useCurrentTheme.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				TablePanel.this.table.getTableHeader().putClientProperty(
-						SubstanceLookAndFeel.THEME_PROPERTY,
-						(useCurrentTheme.isSelected() ? null : "Bottle Green"));
-				TablePanel.this.table.getTableHeader().repaint();
-			}
-		});
-		builder.append("Theme", useCurrentTheme);
-
-		final JCheckBox isAlwaysActive = new JCheckBox("Is always active");
-		isAlwaysActive.setSelected(false);
-		isAlwaysActive.addActionListener(new ActionListener() {
-			public void actionPerformed(ActionEvent e) {
-				TablePanel.this.table.getTableHeader().putClientProperty(
-						SubstanceLookAndFeel.PAINT_ACTIVE_PROPERTY,
-						(isAlwaysActive.isSelected() ? Boolean.TRUE : null));
-				TablePanel.this.table.getTableHeader().repaint();
-			}
-		});
-		builder.append("Active", isAlwaysActive);
-
 		builder.appendSeparator("Font settings");
 		JButton tahoma12 = new JButton("Tahoma 12");
 		tahoma12.addActionListener(new ActionListener() {
@@ -575,4 +514,5 @@ public class TablePanel extends ControllablePanel {
 
 		this.controlPanel = builder.getPanel();
 	}
+
 }
\ No newline at end of file
