@@ -38,7 +38,7 @@ import javax.swing.*;
 import org.jvnet.lafwidget.LafWidget;
 import org.jvnet.lafwidget.LafWidgetUtilities;
 import org.jvnet.lafwidget.utils.LafConstants.AnimationKind;
-import org.jvnet.substance.SubstanceDefaultComboBoxRenderer;
+import org.jvnet.substance.api.renderers.SubstanceDefaultComboBoxRenderer;
 import org.jvnet.substance.skin.SubstanceBusinessBlackSteelLookAndFeel;
 
 /**
