@@ -35,17 +35,14 @@ import java.awt.event.ActionListener;
 
 import javax.swing.*;
 
-import org.jvnet.substance.SubstanceDefaultComboBoxRenderer;
 import org.jvnet.substance.SubstanceLookAndFeel;
+import org.jvnet.substance.api.SubstanceConstants.MenuGutterFillKind;
+import org.jvnet.substance.api.renderers.SubstanceDefaultComboBoxRenderer;
 import org.jvnet.substance.skin.SubstanceBusinessBlackSteelLookAndFeel;
-import org.jvnet.substance.utils.SubstanceCoreUtilities;
-import org.jvnet.substance.utils.SubstanceConstants.MenuGutterFillKind;
-
-import test.Check;
 
 /**
- * Test application that shows the use of the
- * {@link SubstanceLookAndFeel#MENU_GUTTER_FILL_KIND} client property.
+ * Test application that shows the use of the {@link
+ * SubstanceLookAndFeel#MENU_GUTTER_FILL_KIND} client property.
  * 
  * @author Kirill Grouchnikov
  * @see SubstanceLookAndFeel#MENU_GUTTER_FILL_KIND
@@ -62,14 +59,18 @@ public class MenuGutterFillKindProperty extends JFrame {
 		// create sample menu bar with one menu and a few menu items
 		JMenuBar jmb = new JMenuBar();
 		JMenu menu = new JMenu("menu");
-		menu.add(new JMenuItem("test item 1", Check.getIcon("flag_mexico")));
+		String packageName = MenuGutterFillKindProperty.class.getPackage()
+				.getName();
+		menu.add(new JMenuItem("test item 1", new ImageIcon(
+				MenuGutterFillKindProperty.class.getClassLoader().getResource(
+						packageName.replace('.', '/') + "/flag_mexico.png"))));
 		menu.add(new JMenuItem("test item 2"));
 		menu.add(new JMenuItem("test item 3"));
 		menu.addSeparator();
 		menu.add(new JMenuItem("test menu item 4"));
-		menu
-				.add(new JMenuItem("test menu item 5", Check
-						.getIcon("flag_sweden")));
+		menu.add(new JMenuItem("test menu item 5", new ImageIcon(
+				MenuGutterFillKindProperty.class.getClassLoader().getResource(
+						packageName.replace('.', '/') + "/flag_sweden.png"))));
 		menu.add(new JMenuItem("test menu item 6"));
 		jmb.add(menu);
 
@@ -92,8 +93,7 @@ public class MenuGutterFillKindProperty extends JFrame {
 						.toLowerCase(), index, isSelected, cellHasFocus);
 			}
 		});
-		menuGutterFillCombo.setSelectedItem(SubstanceCoreUtilities
-				.getMenuGutterFillKind());
+		menuGutterFillCombo.setSelectedItem(MenuGutterFillKind.HARD);
 		menuGutterFillCombo.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				// based on the selected item, set the global menu gutter
@@ -113,15 +113,13 @@ public class MenuGutterFillKindProperty extends JFrame {
 	}
 
 	/**
-	 * The main method for <code>this</code> sample. The arguments are
-	 * ignored.
+	 * The main method for <code>this</code> sample. The arguments are ignored.
 	 * 
 	 * @param args
-	 *            Ignored.
+	 * 		Ignored.
 	 * @throws Exception
-	 *             If some exception occured. Note that there is no special
-	 *             treatment of exception conditions in <code>this</code>
-	 *             sample code.
+	 * 		If some exception occured. Note that there is no special treatment
+	 * 		of exception conditions in <code>this</code> sample code.
 	 */
 	public static void main(String[] args) throws Exception {
 		UIManager.setLookAndFeel(new SubstanceBusinessBlackSteelLookAndFeel());
